@@ -1,12 +1,8 @@
-"""
-    weasyprint.tests.conftest
-    -------------------------
+"""Configuration for WeasyPrint tests.
 
-    Configuration for WeasyPrint tests.
+This module adds a PNG export based on Ghostscript.
 
-    This module adds a PNG export based on Ghostscript.
-
-    Note that Ghostscript is released under AGPL.
+Note that Ghostscript is released under AGPL.
 
 """
 
@@ -21,6 +17,8 @@ from PIL import Image
 from weasyprint import HTML
 from weasyprint.document import Document
 
+from . import draw
+
 MAGIC_NUMBER = b'\x89\x50\x4e\x47\x0d\x0a\x1a\x0a'
 
 
@@ -36,9 +34,8 @@ def document_write_png(self, target=None, resolution=96, antialiasing=1,
     pngs = run(command, stdout=PIPE).stdout
     os.remove(pdf.name)
 
-    assert pngs.startswith(MAGIC_NUMBER), (
-        'Ghostscript error: '
-        f'{pngs.split(MAGIC_NUMBER)[0].decode("ascii").strip()}')
+    error = pngs.split(MAGIC_NUMBER)[0].decode().strip()
+    assert pngs.startswith(MAGIC_NUMBER), f'Ghostscript error: {error}'
 
     if split_images:
         assert target is None
@@ -86,7 +83,35 @@ def html_write_png(self, target=None, stylesheets=None, resolution=96,
             target, resolution)
 
 
-@pytest.fixture(autouse=True)
-def monkey_write_png(monkeypatch):
-    Document.write_png = document_write_png
-    HTML.write_png = html_write_png
+Document.write_png = document_write_png
+HTML.write_png = html_write_png
+
+
+def test_filename(filename):
+    return ''.join(
+        character if character.isalnum() else '_'
+        for character in filename[5:50]).rstrip('_')
+
+
+@pytest.fixture
+def assert_pixels(request, *args, **kwargs):
+    return lambda *args, **kwargs: draw.assert_pixels(
+        test_filename(request.node.name), *args, **kwargs)
+
+
+@pytest.fixture
+def assert_same_renderings(request, *args, **kwargs):
+    return lambda *args, **kwargs: draw.assert_same_renderings(
+        test_filename(request.node.name), *args, **kwargs)
+
+
+@pytest.fixture
+def assert_different_renderings(request, *args, **kwargs):
+    return lambda *args, **kwargs: draw.assert_different_renderings(
+        test_filename(request.node.name), *args, **kwargs)
+
+
+@pytest.fixture
+def assert_pixels_equal(request, *args, **kwargs):
+    return lambda *args, **kwargs: draw.assert_pixels_equal(
+        test_filename(request.node.name), *args, **kwargs)
