@@ -1,20 +1,13 @@
-"""
-    weasyprint.tests.test_draw.svg.test_bounding_box
-    ------------------------------------------------
-
-    Test how bounding boxes are defined for SVG tags.
-
-"""
+"""Test how bounding boxes are defined for SVG tags."""
 
 import pytest
 
 from ...testing_utils import assert_no_logs
-from .. import assert_pixels
 
 
 @assert_no_logs
-def test_bounding_box_rect():
-    assert_pixels('bounding_box_rect', 5, 5, '''
+def test_bounding_box_rect(assert_pixels):
+    assert_pixels('''
         BBBBB
         BBBBR
         BBBRR
@@ -25,7 +18,7 @@ def test_bounding_box_rect():
         @page { size: 5px }
         svg { display: block }
       </style>
-      <svg width="5px" height="5px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="5px" height="5px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -39,8 +32,8 @@ def test_bounding_box_rect():
 
 
 @assert_no_logs
-def test_bounding_box_circle():
-    assert_pixels('bounding_box_circle', 10, 10, '''
+def test_bounding_box_circle(assert_pixels):
+    assert_pixels('''
         __________
         __BBBBBB__
         _BBBBBBBR_
@@ -56,7 +49,7 @@ def test_bounding_box_circle():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -70,8 +63,8 @@ def test_bounding_box_circle():
 
 
 @assert_no_logs
-def test_bounding_box_ellipse():
-    assert_pixels('bounding_box_ellipse', 10, 10, '''
+def test_bounding_box_ellipse(assert_pixels):
+    assert_pixels('''
         __________
         __BBBBBB__
         _BBBBBBBR_
@@ -87,7 +80,7 @@ def test_bounding_box_ellipse():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -101,8 +94,8 @@ def test_bounding_box_ellipse():
 
 
 @assert_no_logs
-def test_bounding_box_line():
-    assert_pixels('bounding_box_line', 5, 5, '''
+def test_bounding_box_line(assert_pixels):
+    assert_pixels('''
         BB___
         BBB__
         _BRR_
@@ -113,7 +106,7 @@ def test_bounding_box_line():
         @page { size: 5px }
         svg { display: block }
       </style>
-      <svg width="5px" height="5px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="5px" height="5px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -128,8 +121,8 @@ def test_bounding_box_line():
 
 
 @assert_no_logs
-def test_bounding_box_polygon():
-    assert_pixels('bounding_box_polygon', 5, 5, '''
+def test_bounding_box_polygon(assert_pixels):
+    assert_pixels('''
         BBBBB
         BBBBR
         BBBRR
@@ -140,7 +133,7 @@ def test_bounding_box_polygon():
         @page { size: 5px }
         svg { display: block }
       </style>
-      <svg width="5px" height="5px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="5px" height="5px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -154,8 +147,8 @@ def test_bounding_box_polygon():
 
 
 @assert_no_logs
-def test_bounding_box_polyline():
-    assert_pixels('bounding_box_polyline', 5, 5, '''
+def test_bounding_box_polyline(assert_pixels):
+    assert_pixels('''
         BBBBB
         BBBBR
         BBBRR
@@ -166,7 +159,7 @@ def test_bounding_box_polyline():
         @page { size: 5px }
         svg { display: block }
       </style>
-      <svg width="5px" height="5px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="5px" height="5px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -181,8 +174,8 @@ def test_bounding_box_polyline():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_bounding_box_text():
-    assert_pixels('bounding_box_text', 2, 2, '''
+def test_bounding_box_text(assert_pixels):
+    assert_pixels('''
         BB
         BR
     ''', '''
@@ -191,7 +184,7 @@ def test_bounding_box_text():
         @page { size: 2px }
         svg { display: block }
       </style>
-      <svg width="2px" height="2px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="2px" height="2px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -208,8 +201,8 @@ def test_bounding_box_text():
 
 
 @assert_no_logs
-def test_bounding_box_path_hv():
-    assert_pixels('bounding_box_path_hv', 5, 5, '''
+def test_bounding_box_path_hv(assert_pixels):
+    assert_pixels('''
         BBBBB
         BBBBR
         BBBRR
@@ -220,7 +213,7 @@ def test_bounding_box_path_hv():
         @page { size: 5px }
         svg { display: block }
       </style>
-      <svg width="5px" height="5px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="5px" height="5px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -234,8 +227,8 @@ def test_bounding_box_path_hv():
 
 
 @assert_no_logs
-def test_bounding_box_path_l():
-    assert_pixels('bounding_box_path_l', 5, 5, '''
+def test_bounding_box_path_l(assert_pixels):
+    assert_pixels('''
         BBBBB
         BBBBR
         BBBRR
@@ -246,7 +239,7 @@ def test_bounding_box_path_l():
         @page { size: 5px }
         svg { display: block }
       </style>
-      <svg width="5px" height="5px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="5px" height="5px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -261,8 +254,8 @@ def test_bounding_box_path_l():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_bounding_box_path_c():
-    assert_pixels('bounding_box_path_c', 5, 5, '''
+def test_bounding_box_path_c(assert_pixels):
+    assert_pixels('''
         BBB__
         BBR__
         _____
@@ -273,7 +266,7 @@ def test_bounding_box_path_c():
         @page { size: 5px }
         svg { display: block }
       </style>
-      <svg width="5px" height="5px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="5px" height="5px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
@@ -291,8 +284,8 @@ def test_bounding_box_path_c():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_bounding_box_path_s():
-    assert_pixels('bounding_box_path_s', 5, 5, '''
+def test_bounding_box_path_s(assert_pixels):
+    assert_pixels('''
         BBB__
         BBR__
         _____
@@ -303,7 +296,7 @@ def test_bounding_box_path_s():
         @page { size: 5px }
         svg { display: block }
       </style>
-      <svg width="5px" height="5px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="5px" height="5px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <linearGradient id="grad" x1="0" y1="0" x2="1" y2="1"
             gradientUnits="objectBoundingBox">
