@@ -1,20 +1,13 @@
-"""
-    weasyprint.tests.test_draw.svg.test_clip
-    ----------------------------------------
-
-    Test clip-path attribute.
-
-"""
+"""Test clip-path attribute."""
 
 import pytest
 
 from ...testing_utils import assert_no_logs
-from .. import assert_pixels
 
 
 @assert_no_logs
-def test_clip_path():
-    assert_pixels('clip_path', 9, 9, '''
+def test_clip_path(assert_pixels):
+    assert_pixels('''
         _________
         _________
         __RRRRR__
@@ -29,7 +22,7 @@ def test_clip_path():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <clipPath id="clip">
             <rect x="2" y="2" width="5" height="5" />
@@ -42,8 +35,8 @@ def test_clip_path():
 
 
 @assert_no_logs
-def test_clip_path_on_group():
-    assert_pixels('clip_path_on_group', 9, 9, '''
+def test_clip_path_on_group(assert_pixels):
+    assert_pixels('''
         _________
         _________
         __BBBB___
@@ -58,7 +51,7 @@ def test_clip_path_on_group():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <clipPath id="clip">
             <rect x="2" y="2" width="5" height="5" />
@@ -74,8 +67,8 @@ def test_clip_path_on_group():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_clip_path_group_on_group():
-    assert_pixels('clip_path_group_on_group', 9, 9, '''
+def test_clip_path_group_on_group(assert_pixels):
+    assert_pixels(9, 9, '''
         _________
         _________
         __BB_____
@@ -90,7 +83,7 @@ def test_clip_path_group_on_group():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <defs>
           <clipPath id="clip">
             <rect x="2" y="2" width="2" height="2" />
