@@ -1,21 +1,14 @@
-"""
-    weasyprint.tests.test_draw.svg.test_images
-    ------------------------------------------
-
-    Test how images are drawn in SVG.
-
-"""
+"""Test how images are drawn in SVG."""
 
 import pytest
 from weasyprint.urls import path2url
 
 from ...testing_utils import assert_no_logs, resource_filename
-from .. import assert_pixels
 
 
 @assert_no_logs
-def test_image_svg():
-    assert_pixels('test_image_svg', 4, 4, '''
+def test_image_svg(assert_pixels):
+    assert_pixels('''
         ____
         ____
         __B_
@@ -25,7 +18,7 @@ def test_image_svg():
         @page { size: 4px 4px }
         svg { display: block }
       </style>
-      <svg width="4px" height="4px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="4px" height="4px" xmlns="https://www.w3.org/2000/svg">
         <svg x="1" y="1" width="2" height="2" viewBox="0 0 10 10">
           <rect x="5" y="5" width="5" height="5" fill="blue" />
         </svg>
@@ -34,8 +27,8 @@ def test_image_svg():
 
 
 @assert_no_logs
-def test_image_svg_viewbox():
-    assert_pixels('test_image_svg_viewbox', 4, 4, '''
+def test_image_svg_viewbox(assert_pixels):
+    assert_pixels('''
         ____
         ____
         __B_
@@ -45,7 +38,7 @@ def test_image_svg_viewbox():
         @page { size: 4px 4px }
         svg { display: block }
       </style>
-      <svg viewBox="0 0 4 4" xmlns="http://www.w3.org/2000/svg">
+      <svg viewBox="0 0 4 4" xmlns="https://www.w3.org/2000/svg">
         <svg x="1" y="1" width="2" height="2" viewBox="10 10 10 10">
           <rect x="15" y="15" width="5" height="5" fill="blue" />
         </svg>
@@ -54,8 +47,8 @@ def test_image_svg_viewbox():
 
 
 @assert_no_logs
-def test_image_svg_align_default():
-    assert_pixels('test_image_svg_align_default', 8, 8, '''
+def test_image_svg_align_default(assert_pixels):
+    assert_pixels('''
         __BRRR__
         __BRRR__
         __RRRG__
@@ -70,7 +63,7 @@ def test_image_svg_align_default():
         svg { display: block }
       </style>
       <svg width="8px" height="4px" viewBox="0 0 4 4"
-           xmlns="http://www.w3.org/2000/svg">
+           xmlns="https://www.w3.org/2000/svg">
         <rect width="4" height="4" fill="red" />
         <rect width="1" height="2" fill="blue" />
         <rect x="3" y="2" width="1" height="2" fill="lime" />
@@ -79,8 +72,8 @@ def test_image_svg_align_default():
 
 
 @assert_no_logs
-def test_image_svg_align_none():
-    assert_pixels('test_image_svg_align_none', 8, 8, '''
+def test_image_svg_align_none(assert_pixels):
+    assert_pixels('''
         BBRRRRRR
         BBRRRRRR
         RRRRRRGG
@@ -96,7 +89,7 @@ def test_image_svg_align_none():
       </style>
       <svg width="8px" height="4px" viewBox="0 0 4 4"
            preserveAspectRatio="none"
-           xmlns="http://www.w3.org/2000/svg">
+           xmlns="https://www.w3.org/2000/svg">
         <rect width="4" height="4" fill="red" />
         <rect width="1" height="2" fill="blue" />
         <rect x="3" y="2" width="1" height="2" fill="lime" />
@@ -105,8 +98,8 @@ def test_image_svg_align_none():
 
 
 @assert_no_logs
-def test_image_svg_align_meet_x():
-    assert_pixels('test_image_svg_align_meet_x', 8, 8, '''
+def test_image_svg_align_meet_x(assert_pixels):
+    assert_pixels('''
         ____BRRR
         ____BRRR
         ____RRRG
@@ -122,7 +115,7 @@ def test_image_svg_align_meet_x():
       </style>
       <svg width="8px" height="4px" viewBox="0 0 4 4"
            preserveAspectRatio="xMaxYMax meet"
-           xmlns="http://www.w3.org/2000/svg">
+           xmlns="https://www.w3.org/2000/svg">
         <rect width="4" height="4" fill="red" />
         <rect width="1" height="2" fill="blue" />
         <rect x="3" y="2" width="1" height="2" fill="lime" />
@@ -131,8 +124,8 @@ def test_image_svg_align_meet_x():
 
 
 @assert_no_logs
-def test_image_svg_align_meet_y():
-    assert_pixels('test_image_svg_align_meet_y', 8, 8, '''
+def test_image_svg_align_meet_y(assert_pixels):
+    assert_pixels('''
         ________
         ________
         ________
@@ -148,7 +141,7 @@ def test_image_svg_align_meet_y():
       </style>
       <svg width="4px" height="8px" viewBox="0 0 4 4"
            preserveAspectRatio="xMaxYMax meet"
-           xmlns="http://www.w3.org/2000/svg">
+           xmlns="https://www.w3.org/2000/svg">
         <rect width="4" height="4" fill="red" />
         <rect width="1" height="2" fill="blue" />
         <rect x="3" y="2" width="1" height="2" fill="lime" />
@@ -157,8 +150,8 @@ def test_image_svg_align_meet_y():
 
 
 @assert_no_logs
-def test_image_svg_align_slice_x():
-    assert_pixels('test_image_svg_align_slice_x', 8, 8, '''
+def test_image_svg_align_slice_x(assert_pixels):
+    assert_pixels('''
         BBRRRRRR
         BBRRRRRR
         BBRRRRRR
@@ -174,7 +167,7 @@ def test_image_svg_align_slice_x():
       </style>
       <svg width="8px" height="4px" viewBox="0 0 4 4"
            preserveAspectRatio="xMinYMin slice"
-           xmlns="http://www.w3.org/2000/svg">
+           xmlns="https://www.w3.org/2000/svg">
         <rect width="4" height="4" fill="red" />
         <rect width="1" height="2" fill="blue" />
         <rect x="3" y="2" width="1" height="2" fill="lime" />
@@ -183,8 +176,8 @@ def test_image_svg_align_slice_x():
 
 
 @assert_no_logs
-def test_image_svg_align_slice_y():
-    assert_pixels('test_image_svg_align_slice_y', 8, 8, '''
+def test_image_svg_align_slice_y(assert_pixels):
+    assert_pixels('''
         BBRR____
         BBRR____
         BBRR____
@@ -200,7 +193,7 @@ def test_image_svg_align_slice_y():
       </style>
       <svg width="4px" height="8px" viewBox="0 0 4 4"
            preserveAspectRatio="xMinYMin slice"
-           xmlns="http://www.w3.org/2000/svg">
+           xmlns="https://www.w3.org/2000/svg">
         <rect width="4" height="4" fill="red" />
         <rect width="1" height="2" fill="blue" />
         <rect x="3" y="2" width="1" height="2" fill="lime" />
@@ -210,8 +203,8 @@ def test_image_svg_align_slice_y():
 
 @pytest.mark.xfail
 @assert_no_logs
-def test_image_svg_percentage():
-    assert_pixels('test_image_svg_percentage', 4, 4, '''
+def test_image_svg_percentage(assert_pixels):
+    assert_pixels('''
         ____
         ____
         __B_
@@ -221,7 +214,7 @@ def test_image_svg_percentage():
         @page { size: 4px 4px }
         svg { display: block }
       </style>
-      <svg width="100%" height="100%" xmlns="http://www.w3.org/2000/svg">
+      <svg width="100%" height="100%" xmlns="https://www.w3.org/2000/svg">
         <svg x="1" y="1" width="50%" height="50%" viewBox="0 0 10 10">
           <rect x="5" y="5" width="5" height="5" fill="blue" />
         </svg>
@@ -229,8 +222,8 @@ def test_image_svg_percentage():
     ''')
 
 
-def test_image_svg_wrong():
-    assert_pixels('test_image_svg_wrong', 4, 4, '''
+def test_image_svg_wrong(assert_pixels):
+    assert_pixels('''
         ____
         ____
         ____
@@ -240,15 +233,15 @@ def test_image_svg_wrong():
         @page { size: 4px 4px }
         svg { display: block }
       </style>
-      <svg width="4px" height="4px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="4px" height="4px" xmlns="https://www.w3.org/2000/svg">
         <That’s bad!
       </svg>
     ''')
 
 
 @assert_no_logs
-def test_image_image():
-    assert_pixels('test_image_image', 4, 4, '''
+def test_image_image(assert_pixels):
+    assert_pixels('''
         rBBB
         BBBB
         BBBB
@@ -258,14 +251,14 @@ def test_image_image():
         @page { size: 4px 4px }
         svg { display: block }
       </style>
-      <svg width="4px" height="4px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="4px" height="4px" xmlns="https://www.w3.org/2000/svg">
         <image xlink:href="%s" />
       </svg>
     ''' % path2url(resource_filename('pattern.png')))
 
 
-def test_image_image_wrong():
-    assert_pixels('test_image_image_wrong', 4, 4, '''
+def test_image_image_wrong(assert_pixels):
+    assert_pixels('''
         ____
         ____
         ____
@@ -275,7 +268,7 @@ def test_image_image_wrong():
         @page { size: 4px 4px }
         svg { display: block }
       </style>
-      <svg width="4px" height="4px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="4px" height="4px" xmlns="https://www.w3.org/2000/svg">
         <image xlink:href="it doesn’t exist, mouhahahaha" />
       </svg>
     ''')
