@@ -1,12 +1,11 @@
 """Test how SVG simple paths are drawn."""
 
 from ...testing_utils import assert_no_logs
-from .. import assert_pixels
 
 
 @assert_no_logs
-def test_path_Hh():
-    assert_pixels('path_Hh', 10, 10, '''
+def test_path_Hh(assert_pixels):
+    assert_pixels('''
         BBBBBBBB__
         BBBBBBBB__
         __________
@@ -22,7 +21,7 @@ def test_path_Hh():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 0 1 H 8 H 1"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 0 4 H 8 4"
@@ -36,8 +35,8 @@ def test_path_Hh():
 
 
 @assert_no_logs
-def test_path_Vv():
-    assert_pixels('path_Vv', 10, 10, '''
+def test_path_Vv(assert_pixels):
+    assert_pixels('''
         BB____GG__
         BB____GG__
         BB____GG__
@@ -53,7 +52,7 @@ def test_path_Vv():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 1 0 V 1 V 4"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 4 6 V 4 10"
@@ -67,8 +66,8 @@ def test_path_Vv():
 
 
 @assert_no_logs
-def test_path_Ll():
-    assert_pixels('path_Ll', 10, 10, '''
+def test_path_Ll(assert_pixels):
+    assert_pixels('''
         ______RR__
         ______RR__
         ______RR__
@@ -84,7 +83,7 @@ def test_path_Ll():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 4 3 L 4 10"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 7 0 l 0 6"
@@ -94,8 +93,8 @@ def test_path_Ll():
 
 
 @assert_no_logs
-def test_path_Zz():
-    assert_pixels('path_Zz', 10, 10, '''
+def test_path_Zz(assert_pixels):
+    assert_pixels('''
         BBBBBBB___
         BBBBBBB___
         BB___BB___
@@ -111,7 +110,7 @@ def test_path_Zz():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 1 1 H 6 V 5 H 1 Z"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 9 10 V 7 H 5 V 10 z"
@@ -121,8 +120,8 @@ def test_path_Zz():
 
 
 @assert_no_logs
-def test_path_Zz_fill():
-    assert_pixels('path_Zz_fill', 10, 10, '''
+def test_path_Zz_fill(assert_pixels):
+    assert_pixels('''
         BBBBBBB___
         BBBBBBB___
         BBGGGBB___
@@ -138,7 +137,7 @@ def test_path_Zz_fill():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 1 1 H 6 V 5 H 1 Z"
           stroke="blue" stroke-width="2" fill="lime"/>
         <path d="M 9 10 V 7 H 5 V 10 z"
@@ -148,8 +147,8 @@ def test_path_Zz_fill():
 
 
 @assert_no_logs
-def test_path_Cc():
-    assert_pixels('path_Cc', 10, 10, '''
+def test_path_Cc(assert_pixels):
+    assert_pixels('''
         __________
         __________
         __________
@@ -165,7 +164,7 @@ def test_path_Cc():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 2 5 C 2 5 3 5 5 5"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 2 8 c 0 0 1 0 3 0"
@@ -175,8 +174,8 @@ def test_path_Cc():
 
 
 @assert_no_logs
-def test_path_Ss():
-    assert_pixels('path_Ss', 10, 10, '''
+def test_path_Ss(assert_pixels):
+    assert_pixels('''
         __________
         __________
         __________
@@ -192,7 +191,7 @@ def test_path_Ss():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 2 5 S 3 5 5 5"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 2 8 s 1 0 3 0"
@@ -202,8 +201,8 @@ def test_path_Ss():
 
 
 @assert_no_logs
-def test_path_CcSs():
-    assert_pixels('path_CcSs', 10, 12, '''
+def test_path_CcSs(assert_pixels):
+    assert_pixels('''
         __BBBBBB__
         __BBBBBBB_
         _____BBBB_
@@ -221,7 +220,7 @@ def test_path_CcSs():
         @page { size: 10px 12px }
         svg { display: block }
       </style>
-      <svg width="10px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 2 1 C 2 1 3 1 5 1 S 8 3 8 1"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 2 4 C 2 4 3 4 5 4 s 3 2 1 0"
@@ -235,8 +234,8 @@ def test_path_CcSs():
 
 
 @assert_no_logs
-def test_path_Qq():
-    assert_pixels('path_Qq', 10, 10, '''
+def test_path_Qq(assert_pixels):
+    assert_pixels('''
         __________
         __________
         __________
@@ -252,7 +251,7 @@ def test_path_Qq():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 2 5 Q 4 5 6 5"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 2 8 q 2 0 4 0"
@@ -262,8 +261,8 @@ def test_path_Qq():
 
 
 @assert_no_logs
-def test_path_Tt():
-    assert_pixels('path_Tt', 10, 10, '''
+def test_path_Tt(assert_pixels):
+    assert_pixels('''
         __________
         __________
         __________
@@ -279,7 +278,7 @@ def test_path_Tt():
         @page { size: 10px }
         svg { display: block }
       </style>
-      <svg width="10px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="10px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 2 5 T 6 5"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 2 8 t 4 0"
@@ -289,8 +288,8 @@ def test_path_Tt():
 
 
 @assert_no_logs
-def test_path_QqTt():
-    assert_pixels('path_QqTt', 12, 12, '''
+def test_path_QqTt(assert_pixels):
+    assert_pixels('''
         _BBBB_______
         BBBBBBB_____
         BBBBBBBB__BB
@@ -308,7 +307,7 @@ def test_path_QqTt():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 0 3 Q 3 0 6 3 T 12 3"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 0 9 Q 3 6 6 9 t 6 0"
@@ -318,8 +317,8 @@ def test_path_QqTt():
 
 
 @assert_no_logs
-def test_path_QqTt2():
-    assert_pixels('path_QqTt2', 12, 12, '''
+def test_path_QqTt2(assert_pixels):
+    assert_pixels('''
         _BBBB_______
         BBBBBBB_____
         BBBBBBBB__BB
@@ -337,7 +336,7 @@ def test_path_QqTt2():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 0 3 q 3 -3 6 0 T 12 3"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 0 9 q 3 -3 6 0 t 6 0"
@@ -347,8 +346,8 @@ def test_path_QqTt2():
 
 
 @assert_no_logs
-def test_path_Aa():
-    assert_pixels('path_Aa', 12, 12, '''
+def test_path_Aa(assert_pixels):
+    assert_pixels('''
         __BBBB______
         _BBBBB______
         BBBBBB______
@@ -366,7 +365,7 @@ def test_path_Aa():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 1 6 A 5 5 0 0 1 6 1"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 6 11 a 5 5 0 0 1 5 -5"
@@ -376,8 +375,8 @@ def test_path_Aa():
 
 
 @assert_no_logs
-def test_path_Aa2():
-    assert_pixels('path_Aa2', 12, 12, '''
+def test_path_Aa2(assert_pixels):
+    assert_pixels('''
         ______GGGG__
         ______GGGGG_
         ______GGGGGG
@@ -395,7 +394,7 @@ def test_path_Aa2():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 1 6 A 5 5 0 1 0 6 1"
           stroke="lime" stroke-width="2" fill="none"/>
       </svg>
@@ -403,8 +402,8 @@ def test_path_Aa2():
 
 
 @assert_no_logs
-def test_path_Aa3():
-    assert_pixels('path_Aa3', 12, 12, '''
+def test_path_Aa3(assert_pixels):
+    assert_pixels('''
         ______GGGG__
         ______GGGGG_
         ______GGGGGG
@@ -422,7 +421,7 @@ def test_path_Aa3():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 1 6 a 5 5 0 1 0 5 -5"
           stroke="lime" stroke-width="2" fill="none"/>
       </svg>
@@ -430,8 +429,8 @@ def test_path_Aa3():
 
 
 @assert_no_logs
-def test_path_Aa4():
-    assert_pixels('path_Aa4', 12, 12, '''
+def test_path_Aa4(assert_pixels):
+    assert_pixels('''
         ____________
         ____BBB_____
         ____BBB_____
@@ -449,7 +448,7 @@ def test_path_Aa4():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 1 6 A 5 5 0 0 0 6 1"
           stroke="blue" stroke-width="2" fill="none"/>
         <path d="M 6 11 a 5 5 0 0 0 5 -5"
@@ -459,8 +458,8 @@ def test_path_Aa4():
 
 
 @assert_no_logs
-def test_path_Aa5():
-    assert_pixels('path_Aa5', 12, 12, '''
+def test_path_Aa5(assert_pixels):
+    assert_pixels('''
         __BBBBBBBB__
         _BBBBBBBBBB_
         BBBBBBBBBBBB
@@ -478,7 +477,7 @@ def test_path_Aa5():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 6 11 A 5 5 0 1 1 11 6"
           stroke="blue" stroke-width="2" fill="none"/>
       </svg>
@@ -486,8 +485,8 @@ def test_path_Aa5():
 
 
 @assert_no_logs
-def test_path_Aa6():
-    assert_pixels('path_Aa6', 12, 12, '''
+def test_path_Aa6(assert_pixels):
+    assert_pixels('''
         __BBBBBBBB__
         _BBBBBBBBBB_
         BBBBBBBBBBBB
@@ -505,7 +504,7 @@ def test_path_Aa6():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 6 11 a 5 5 0 1 1 5 -5"
           stroke="blue" stroke-width="2" fill="none"/>
       </svg>
@@ -513,8 +512,8 @@ def test_path_Aa6():
 
 
 @assert_no_logs
-def test_path_Aa7():
-    assert_pixels('path_Aa7', 12, 12, '''
+def test_path_Aa7(assert_pixels):
+    assert_pixels('''
         ____________
         ____________
         ____________
@@ -532,7 +531,7 @@ def test_path_Aa7():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 1 6 A 5 5 0 0 0 11 6"
           stroke="lime" stroke-width="2" fill="none"/>
       </svg>
@@ -540,8 +539,8 @@ def test_path_Aa7():
 
 
 @assert_no_logs
-def test_path_wrong_point():
-    assert_pixels('path_wrong_point', 12, 12, '''
+def test_path_wrong_point(assert_pixels):
+    assert_pixels('''
         ____________
         GG__________
         GG__________
@@ -559,7 +558,7 @@ def test_path_wrong_point():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <path d="M 1 1 L 1 5 L"
           stroke="lime" stroke-width="2" fill="none"/>
       </svg>
@@ -567,8 +566,8 @@ def test_path_wrong_point():
 
 
 @assert_no_logs
-def test_path_markers_l():
-    assert_pixels('path_markers_l', 12, 12, '''
+def test_path_markers_l(assert_pixels):
+    assert_pixels('''
         _________zz_
         _RR_____zzRz
         _RRGGGGzzRzz
@@ -586,7 +585,7 @@ def test_path_markers_l():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <marker id="line"
           viewBox="0 0 1 2" refX="0.5" refY="1"
           markerUnits="strokeWidth"
@@ -601,8 +600,8 @@ def test_path_markers_l():
 
 
 @assert_no_logs
-def test_path_markers_hv():
-    assert_pixels('path_markers_hv', 12, 12, '''
+def test_path_markers_hv(assert_pixels):
+    assert_pixels('''
         _________zz_
         _RR_____zzRz
         _RRGGGGzzRzz
@@ -620,7 +619,7 @@ def test_path_markers_hv():
         @page { size: 12px }
         svg { display: block }
       </style>
-      <svg width="12px" height="12px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="12px" height="12px" xmlns="https://www.w3.org/2000/svg">
         <marker id="line"
           viewBox="0 0 1 2" refX="0.5" refY="1"
           markerUnits="strokeWidth"
