@@ -1,18 +1,11 @@
-"""
-    weasyprint.tests.test_draw.svg.test_text
-    ----------------------------------------
-
-    Test how SVG text is drawn.
-
-"""
+"""Test how SVG text is drawn."""
 
 from ...testing_utils import assert_no_logs
-from .. import assert_pixels
 
 
 @assert_no_logs
-def test_text_fill():
-    assert_pixels('text_fill', 20, 2, '''
+def test_text_fill(assert_pixels):
+    assert_pixels('''
         BBBBBB__BBBBBB______
         BBBBBB__BBBBBB______
     ''', '''
@@ -21,7 +14,7 @@ def test_text_fill():
         @page { size: 20px 2px }
         svg { display: block }
       </style>
-      <svg width="20px" height="2px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="20px" height="2px" xmlns="https://www.w3.org/2000/svg">
         <text x="0" y="1.5" font-family="weasyprint" font-size="2" fill="blue">
           ABC DEF
         </text>
@@ -30,8 +23,8 @@ def test_text_fill():
 
 
 @assert_no_logs
-def test_text_stroke():
-    assert_pixels('text_stroke', 20, 4, '''
+def test_text_stroke(assert_pixels):
+    assert_pixels('''
         _BBBBBBBBBBBB_______
         _BBBBBBBBBBBB_______
         _BBBBBBBBBBBB_______
@@ -39,12 +32,12 @@ def test_text_stroke():
     ''', '''
       <style>
         @font-face { src: url(weasyprint.otf); font-family: weasyprint }
-        @page { size: 20px 4px }
+        @page { font-size: 1px; size: 20em 8ex }
         svg { display: block }
       </style>
-      <svg width="20px" height="4px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="20px" height="4px" xmlns="https://www.w3.org/2000/svg">
         <text x="2" y="2.5" font-family="weasyprint" font-size="2"
-              fill="transparent" stroke="blue" stroke-width="2">
+              fill="transparent" stroke="blue" stroke-width="1ex">
           A B C
         </text>
       </svg>
@@ -52,8 +45,8 @@ def test_text_stroke():
 
 
 @assert_no_logs
-def test_text_x():
-    assert_pixels('text_x', 20, 2, '''
+def test_text_x(assert_pixels):
+    assert_pixels('''
         BB__BB_BBBB_________
         BB__BB_BBBB_________
     ''', '''
@@ -62,7 +55,7 @@ def test_text_x():
         @page { size: 20px 2px }
         svg { display: block }
       </style>
-      <svg width="20px" height="2px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="20px" height="2px" xmlns="https://www.w3.org/2000/svg">
         <text x="0 4 7" y="1.5" font-family="weasyprint" font-size="2"
               fill="blue">
           ABCD
@@ -72,8 +65,8 @@ def test_text_x():
 
 
 @assert_no_logs
-def test_text_y():
-    assert_pixels('text_y', 30, 10, '''
+def test_text_y(assert_pixels):
+    assert_pixels('''
         __________BBBBB_____BBBBBBBBBB
         __________BBBBB_____BBBBBBBBBB
         __________BBBBB_____BBBBBBBBBB
@@ -90,7 +83,7 @@ def test_text_y():
         @page { size: 30px 10px }
         svg { display: block }
       </style>
-      <svg width="30px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="30px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <text x="0" y="9 9 4 9 4" font-family="weasyprint" font-size="5"
               fill="blue">
           ABCDEF
@@ -100,8 +93,8 @@ def test_text_y():
 
 
 @assert_no_logs
-def test_text_xy():
-    assert_pixels('text_xy', 30, 10, '''
+def test_text_xy(assert_pixels):
+    assert_pixels('''
         __________BBBBB_____BBBBBBBBBB
         __________BBBBB_____BBBBBBBBBB
         __________BBBBB_____BBBBBBBBBB
@@ -118,7 +111,7 @@ def test_text_xy():
         @page { size: 30px 10px }
         svg { display: block }
       </style>
-      <svg width="30px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="30px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <text x="0 10" y="9 4 9 4" font-family="weasyprint" font-size="5"
               fill="blue">
           ABCDE
@@ -128,8 +121,8 @@ def test_text_xy():
 
 
 @assert_no_logs
-def test_text_dx():
-    assert_pixels('text_dx', 20, 2, '''
+def test_text_dx(assert_pixels):
+    assert_pixels('''
         BB__BB_BBBB_________
         BB__BB_BBBB_________
     ''', '''
@@ -138,7 +131,7 @@ def test_text_dx():
         @page { size: 20px 2px }
         svg { display: block }
       </style>
-      <svg width="20px" height="2px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="20px" height="2px" xmlns="https://www.w3.org/2000/svg">
         <text dx="0 2 1" y="1.5" font-family="weasyprint" font-size="2"
               fill="blue">
           ABCD
@@ -148,8 +141,8 @@ def test_text_dx():
 
 
 @assert_no_logs
-def test_text_dy():
-    assert_pixels('text_dy', 30, 10, '''
+def test_text_dy(assert_pixels):
+    assert_pixels('''
         __________BBBBB_____BBBBBBBBBB
         __________BBBBB_____BBBBBBBBBB
         __________BBBBB_____BBBBBBBBBB
@@ -166,7 +159,7 @@ def test_text_dy():
         @page { size: 30px 10px }
         svg { display: block }
       </style>
-      <svg width="30px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="30px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <text x="0" dy="9 0 -5 5 -5" font-family="weasyprint" font-size="5"
               fill="blue">
           ABCDEF
@@ -176,8 +169,8 @@ def test_text_dy():
 
 
 @assert_no_logs
-def test_text_dx_dy():
-    assert_pixels('text_dx_dy', 30, 10, '''
+def test_text_dx_dy(assert_pixels):
+    assert_pixels('''
         __________BBBBB_____BBBBBBBBBB
         __________BBBBB_____BBBBBBBBBB
         __________BBBBB_____BBBBBBBBBB
@@ -194,7 +187,7 @@ def test_text_dx_dy():
         @page { size: 30px 10px }
         svg { display: block }
       </style>
-      <svg width="30px" height="10px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="30px" height="10px" xmlns="https://www.w3.org/2000/svg">
         <text dx="0 5" dy="9 -5 5 -5" font-family="weasyprint" font-size="5"
               fill="blue">
           ABCDE
@@ -204,8 +197,8 @@ def test_text_dx_dy():
 
 
 @assert_no_logs
-def test_text_anchor_start():
-    assert_pixels('text_anchor_start', 20, 4, '''
+def test_text_anchor_start(assert_pixels):
+    assert_pixels('''
         __BBBBBB____________
         __BBBBBB____________
         ____BBBBBB__________
@@ -216,7 +209,7 @@ def test_text_anchor_start():
         @page { size: 20px 4px }
         svg { display: block }
       </style>
-      <svg width="20px" height="4px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="20px" height="4px" xmlns="https://www.w3.org/2000/svg">
         <text x="2" y="1.5" font-family="weasyprint" font-size="2"
               fill="blue">
           ABC
@@ -230,8 +223,8 @@ def test_text_anchor_start():
 
 
 @assert_no_logs
-def test_text_anchor_middle():
-    assert_pixels('text_anchor_middle', 20, 2, '''
+def test_text_anchor_middle(assert_pixels):
+    assert_pixels('''
         _______BBBBBB_______
         _______BBBBBB_______
     ''', '''
@@ -240,7 +233,7 @@ def test_text_anchor_middle():
         @page { size: 20px 2px }
         svg { display: block }
       </style>
-      <svg width="20px" height="2px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="20px" height="2px" xmlns="https://www.w3.org/2000/svg">
         <text x="10" y="1.5" font-family="weasyprint" font-size="2"
               fill="blue" text-anchor="middle">
           ABC
@@ -250,8 +243,8 @@ def test_text_anchor_middle():
 
 
 @assert_no_logs
-def test_text_anchor_end():
-    assert_pixels('text_anchor_end', 20, 2, '''
+def test_text_anchor_end(assert_pixels):
+    assert_pixels('''
         ____________BBBBBB__
         ____________BBBBBB__
     ''', '''
@@ -260,7 +253,7 @@ def test_text_anchor_end():
         @page { size: 20px 2px }
         svg { display: block }
       </style>
-      <svg width="20px" height="2px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="20px" height="2px" xmlns="https://www.w3.org/2000/svg">
         <text x="18" y="1.5" font-family="weasyprint" font-size="2"
               fill="blue" text-anchor="end">
           ABC
@@ -270,8 +263,8 @@ def test_text_anchor_end():
 
 
 @assert_no_logs
-def test_text_tspan():
-    assert_pixels('text_tspan', 20, 2, '''
+def test_text_tspan(assert_pixels):
+    assert_pixels('''
         BBBBBB__BBBBBB______
         BBBBBB__BBBBBB______
     ''', '''
@@ -280,9 +273,31 @@ def test_text_tspan():
         @page { size: 20px 2px }
         svg { display: block }
       </style>
-      <svg width="20px" height="2px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="20px" height="2px" xmlns="https://www.w3.org/2000/svg">
         <text x="10" y="10" font-family="weasyprint" font-size="2" fill="blue">
           <tspan x="0" y="1.5">ABC DEF</tspan>
         </text>
       </svg>
     ''')
+
+
+@assert_no_logs
+def test_text_rotate(assert_pixels):
+    assert_pixels('''
+        __RR__RR__RR________
+        __RR__RR__RR________
+        BB__BB__BB__________
+        BB__BB__BB__________
+    ''', '''
+      <style>
+        @font-face { src: url(weasyprint.otf); font-family: weasyprint }
+        @page { size: 20px 4px }
+        svg { display: block }
+      </style>
+      <svg width="20px" height="4px" xmlns="https://www.w3.org/2000/svg">
+        <text x="2" y="1.5" font-family="weasyprint" font-size="2" fill="red"
+          letter-spacing="2">abc</text>
+        <text x="2" y="1.5" font-family="weasyprint" font-size="2" fill="blue"
+          rotate="180" letter-spacing="2">abc</text>
+      </svg>
+    ''')
