@@ -1,19 +1,11 @@
-"""
-    weasyprint.tests.test_draw.svg.test_visibility
-    ----------------------------------------------
-
-    Test how the visibility is controlled with "visibility" and "display"
-    attributes.
-
-"""
+"""Test how the visibility is controlled with "visibility" and "display"."""
 
 from ...testing_utils import assert_no_logs
-from .. import assert_pixels
 
 
 @assert_no_logs
-def test_visibility_visible():
-    assert_pixels('visibility_visible', 9, 9, '''
+def test_visibility_visible(assert_pixels):
+    assert_pixels('''
         _________
         _________
         __RRRRR__
@@ -28,7 +20,7 @@ def test_visibility_visible():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <rect visibility="visible"
               x="2" y="2" width="5" height="5" fill="red" />
       </svg>
@@ -36,8 +28,8 @@ def test_visibility_visible():
 
 
 @assert_no_logs
-def test_visibility_hidden():
-    assert_pixels('visibility_hidden', 9, 9, '''
+def test_visibility_hidden(assert_pixels):
+    assert_pixels('''
         _________
         _________
         _________
@@ -52,7 +44,7 @@ def test_visibility_hidden():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <rect visibility="hidden"
               x="2" y="2" width="5" height="5" fill="red" />
       </svg>
@@ -60,8 +52,8 @@ def test_visibility_hidden():
 
 
 @assert_no_logs
-def test_visibility_inherit_hidden():
-    assert_pixels('visibility_inherit_hidden', 9, 9, '''
+def test_visibility_inherit_hidden(assert_pixels):
+    assert_pixels('''
         _________
         _________
         _________
@@ -76,7 +68,7 @@ def test_visibility_inherit_hidden():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <g visibility="hidden">
           <rect x="2" y="2" width="5" height="5" fill="red" />
         </g>
@@ -85,8 +77,8 @@ def test_visibility_inherit_hidden():
 
 
 @assert_no_logs
-def test_visibility_inherit_visible():
-    assert_pixels('visibility_inherit_visible', 9, 9, '''
+def test_visibility_inherit_visible(assert_pixels):
+    assert_pixels('''
         _________
         _________
         __RRRRR__
@@ -101,7 +93,7 @@ def test_visibility_inherit_visible():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <g visibility="hidden">
           <rect visibility="visible"
                 x="2" y="2" width="5" height="5" fill="red" />
@@ -111,8 +103,8 @@ def test_visibility_inherit_visible():
 
 
 @assert_no_logs
-def test_display_inline():
-    assert_pixels('display_inline', 9, 9, '''
+def test_display_inline(assert_pixels):
+    assert_pixels('''
         _________
         _________
         __RRRRR__
@@ -127,7 +119,7 @@ def test_display_inline():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <rect display="inline"
               x="2" y="2" width="5" height="5" fill="red" />
       </svg>
@@ -135,8 +127,8 @@ def test_display_inline():
 
 
 @assert_no_logs
-def test_display_none():
-    assert_pixels('display_none', 9, 9, '''
+def test_display_none(assert_pixels):
+    assert_pixels('''
         _________
         _________
         _________
@@ -151,7 +143,7 @@ def test_display_none():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <rect display="none"
               x="2" y="2" width="5" height="5" fill="red" />
       </svg>
@@ -159,8 +151,8 @@ def test_display_none():
 
 
 @assert_no_logs
-def test_display_inherit_none():
-    assert_pixels('display_inherit_none', 9, 9, '''
+def test_display_inherit_none(assert_pixels):
+    assert_pixels('''
         _________
         _________
         _________
@@ -175,7 +167,7 @@ def test_display_inherit_none():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <g display="none">
           <rect x="2" y="2" width="5" height="5" fill="red" />
         </g>
@@ -184,8 +176,8 @@ def test_display_inherit_none():
 
 
 @assert_no_logs
-def test_display_inherit_inline():
-    assert_pixels('display_inherit_inline', 9, 9, '''
+def test_display_inherit_inline(assert_pixels):
+    assert_pixels('''
         _________
         _________
         _________
@@ -200,7 +192,7 @@ def test_display_inherit_inline():
         @page { size: 9px }
         svg { display: block }
       </style>
-      <svg width="9px" height="9px" xmlns="http://www.w3.org/2000/svg">
+      <svg width="9px" height="9px" xmlns="https://www.w3.org/2000/svg">
         <g display="none">
           <rect display="inline"
                 x="2" y="2" width="5" height="5" fill="red" />
