@@ -1,23 +1,13 @@
-"""
-    weasyprint.tests.test_draw.test_current_color
-    ---------------------------------------------
+"""Test the currentColor value."""
 
-    Test the currentColor value.
-
-"""
+import pytest
 
 from ..testing_utils import assert_no_logs
-from . import assert_pixels
-
-GREEN_2x2 = '''
-GG
-GG
-'''
 
 
 @assert_no_logs
-def test_current_color_1():
-    assert_pixels('background_current_color', 2, 2, GREEN_2x2, '''
+def test_current_color_1(assert_pixels):
+    assert_pixels('GG\nGG', '''
       <style>
         @page { size: 2px }
         html, body { height: 100%; margin: 0 }
@@ -28,8 +18,8 @@ def test_current_color_1():
 
 
 @assert_no_logs
-def test_current_color_2():
-    assert_pixels('border_current_color', 2, 2, GREEN_2x2, '''
+def test_current_color_2(assert_pixels):
+    assert_pixels('GG\nGG', '''
       <style>
         @page { size: 2px }
         html { color: red; border-color: currentColor }
@@ -40,8 +30,8 @@ def test_current_color_2():
 
 
 @assert_no_logs
-def test_current_color_3():
-    assert_pixels('outline_current_color', 2, 2, GREEN_2x2, '''
+def test_current_color_3(assert_pixels):
+    assert_pixels('GG\nGG', '''
       <style>
         @page { size: 2px }
         html { color: red; outline-color: currentColor }
@@ -52,8 +42,8 @@ def test_current_color_3():
 
 
 @assert_no_logs
-def test_current_color_4():
-    assert_pixels('border_collapse_current_color', 2, 2, GREEN_2x2, '''
+def test_current_color_4(assert_pixels):
+    assert_pixels('GG\nGG', '''
       <style>
         @page { size: 2px }
         html { color: red; border-color: currentColor; }
@@ -62,3 +52,31 @@ def test_current_color_4():
                 color: lime; border: 1px solid; border-color: inherit }
       </style>
       <table><td>''')
+
+
+@assert_no_logs
+def test_current_color_svg_1(assert_pixels):
+    assert_pixels('KK\nKK', '''
+      <style>
+        @page { size: 2px }
+        svg { display: block }
+      </style>
+      <svg xmlns="http://www.w3.org/2000/svg"
+           width="2" height="2" fill="currentColor">
+        <rect width="2" height="2"></rect>
+      </svg>''')
+
+
+@pytest.mark.xfail
+@assert_no_logs
+def test_current_color_svg_2(assert_pixels):
+    assert_pixels('GG\nGG', '''
+      <style>
+        @page { size: 2px }
+        svg { display: block }
+        body { color: lime }
+      </style>
+      <svg xmlns="http://www.w3.org/2000/svg"
+           width="2" height="2">
+        <rect width="2" height="2" fill="currentColor"></rect>
+      </svg>''')
