@@ -3,13 +3,12 @@
 import pytest
 
 from ..testing_utils import assert_no_logs
-from . import assert_pixels
 
 
 @assert_no_logs
-def test_overflow_1():
+def test_overflow_1(assert_pixels):
     # See test_images
-    assert_pixels('inline_image_overflow', 8, 8, '''
+    assert_pixels('''
         ________
         ________
         __rBBB__
@@ -21,17 +20,17 @@ def test_overflow_1():
     ''', '''
       <style>
         @page { size: 8px }
-        body { margin: 2px 0 0 2px; background: #fff; font-size:0 }
+        body { margin: 2px 0 0 2px; font-size: 0 }
         div { height: 2px; overflow: hidden }
       </style>
       <div><img src="pattern.png"></div>''')
 
 
 @assert_no_logs
-def test_overflow_2():
+def test_overflow_2(assert_pixels):
     # <body> is only 1px high, but its overflow is propageted to the viewport
     # ie. the padding edge of the page box.
-    assert_pixels('inline_image_viewport_overflow', 8, 8, '''
+    assert_pixels('''
         ________
         ________
         __rBBB__
@@ -42,16 +41,16 @@ def test_overflow_2():
         ________
     ''', '''
       <style>
-        @page { size: 8px; background: #fff; margin: 2px 2px 3px 2px }
+        @page { size: 8px; margin: 2px 2px 3px 2px }
         body { height: 1px; overflow: hidden; font-size: 0 }
       </style>
       <div><img src="pattern.png"></div>''')
 
 
 @assert_no_logs
-def test_overflow_3():
+def test_overflow_3(assert_pixels):
     # Assert that the border is not clipped by overflow: hidden
-    assert_pixels('border_box_overflow', 8, 8, '''
+    assert_pixels('''
         ________
         ________
         __BBBB__
@@ -62,7 +61,7 @@ def test_overflow_3():
         ________
     ''', '''
       <style>
-        @page { size: 8px; background: #fff; margin: 2px; }
+        @page { size: 8px; margin: 2px; }
         div { width: 2px; height: 2px; overflow: hidden;
               border: 1px solid blue; }
       </style>
@@ -70,9 +69,9 @@ def test_overflow_3():
 
 
 @assert_no_logs
-def test_overflow_4():
+def test_overflow_4(assert_pixels):
     # Assert that the page margins aren't clipped by body's overflow
-    assert_pixels('border_box_overflow', 8, 8, '''
+    assert_pixels('''
         rr______
         rr______
         __BBBB__
@@ -168,10 +167,10 @@ def test_overflow_4():
         ______________
     '''),
 ))
-def test_clip(number, css, pixels):
-    assert_pixels('background_repeat_clipped_%s' % number, 14, 16, pixels, '''
+def test_clip(assert_pixels, number, css, pixels):
+    assert_pixels(pixels, '''
       <style>
-        @page { size: 14px 16px; background: #fff }
+        @page { size: 14px 16px }
         div { margin: 1px; border: 1px green solid;
               background: url(pattern.png);
               position: absolute; /* clip only applies on abspos */
