@@ -752,6 +752,35 @@ def test_text_float_text_decoration(assert_pixels):
       <div>a<span>b</span></div>''')
 
 
+def test_text_decoration_var(assert_pixels):
+    # Test regression: https://github.com/Kozea/WeasyPrint/issues/1697
+    assert_pixels('''
+        _____________
+        _zzzzzzzzzzz_
+        _zRRRRRRRRRz_
+        _zBBBBBBBBBz_
+        _zRRRRRRRRRz_
+        _zzzzzzzzzzz_
+        _____________
+    ''', '''
+      <style>
+        @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+        @page {
+          size: 13px 7px;
+          margin: 2px;
+        }
+        body {
+          --blue: blue;
+          color: red;
+          font-family: weasyprint;
+          font-size: 3px;
+          text-decoration-color: var(--blue);
+          text-decoration-line: line-through;
+        }
+      </style>
+      <div>abc</div>''')
+
+
 def test_zero_width_character(assert_pixels):
     # Test regression: https://github.com/Kozea/WeasyPrint/issues/1508
     assert_pixels('''
@@ -798,6 +827,30 @@ def test_font_size_very_small(assert_pixels):
     ''')
 
 
+def test_missing_glyph_fallback(assert_pixels):
+    # The apostrophe is not included in weasyprint.otf
+    assert_pixels('''
+        zzzzzzzzzzzzzzzzz___
+        zzzzzzzzzzzzzzzzzRR_
+        zzzzzzzzzzzzzzzzzRR_
+        zzzzzzzzzzzzzzzzz___
+    ''', '''
+      <style>
+        @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+        @page {
+          size: 20px 4px;
+        }
+        body {
+          color: red;
+          font-family: weasyprint;
+          font-size: 2px;
+          line-height: 0;
+          margin: 2px 1px;
+          text-align: right;
+        }
+      </style>'a''')
+
+
 def test_tabulation_character(assert_pixels):
     # Test regression: https://github.com/Kozea/WeasyPrint/issues/1515
     assert_pixels('''
