@@ -1,10 +1,4 @@
-"""
-    weasyprint.tests.layout.block
-    -----------------------------
-
-    Tests for blocks layout.
-
-"""
+"""Tests for blocks layout."""
 
 import pytest
 from weasyprint.formatting_structure import boxes
@@ -263,7 +257,7 @@ def test_block_percentage_heights():
     ('width: 10%; height: 1000px; min-width: auto; max-height: none',),
 ))
 def test_box_sizing(size):
-    # http://www.w3.org/TR/css3-ui/#box-sizing
+    # https://www.w3.org/TR/css-ui-3/#box-sizing
     page, = render_pages('''
       <style>
         @page { size: 100000px }
@@ -318,7 +312,7 @@ def test_box_sizing(size):
     ('min-width: 0; min-height: 0; width: 0; height: 0'),
 ))
 def test_box_sizing_zero(size):
-    # http://www.w3.org/TR/css3-ui/#box-sizing
+    # https://www.w3.org/TR/css-ui-3/#box-sizing
     page, = render_pages('''
       <style>
         @page { size: 100000px }
@@ -571,7 +565,7 @@ def test_vertical_space_9(margin_1, margin_2, result):
 
 @assert_no_logs
 def test_box_decoration_break_block_slice():
-    # http://www.w3.org/TR/css3-background/#the-box-decoration-break
+    # https://www.w3.org/TR/css-backgrounds-3/#the-box-decoration-break
     page_1, page_2 = render_pages('''
       <style>
         @page { size: 100px }
@@ -620,7 +614,7 @@ def test_box_decoration_break_block_slice():
 
 @assert_no_logs
 def test_box_decoration_break_block_clone():
-    # http://www.w3.org/TR/css3-background/#the-box-decoration-break
+    # https://www.w3.org/TR/css-backgrounds-3/#the-box-decoration-break
     page_1, page_2 = render_pages('''
       <style>
         @page { size: 100px }
@@ -842,7 +836,7 @@ def test_block_in_block_with_bottom_padding():
       <style>
         @font-face { src: url(weasyprint.otf); font-family: weasyprint }
         @page { size: 8em 3.5em }
-        body { line-height: 1; orphans: 1; widows: 1; font-family: weasyprint }
+        body { line-height: 1; font-family: weasyprint }
         div { padding-bottom: 1em }
       </style>
       abc def
@@ -876,3 +870,112 @@ def test_block_in_block_with_bottom_padding():
     assert line.height == 16
     assert line.content_box_y() == 16 + 16  # p content + div padding
     assert line.children[0].text == 'stu vwx'
+
+
+@assert_no_logs
+def test_page_breaks_1():
+    # last div does not fit, pushed to next page
+    pages = render_pages('''
+      <style>
+        @page{
+          size: 110px;
+          margin: 10px;
+          padding: 0;
+        }
+        .large {
+          width: 10px;
+          height: 60px;
+        }
+        .small {
+          width: 10px;
+          height: 20px;
+        }
+      </style>
+      <body>
+        <div class="large"></div>
+        <div class="small"></div>
+        <div class="large"></div>
+    ''')
+
+    assert len(pages) == 2
+    page_divs = []
+    for page in pages:
+        divs = [div for div in page.descendants() if div.element_tag == 'div']
+        assert all([div.element_tag == 'div' for div in divs])
+        page_divs.append(divs)
+    positions_y = [[div.position_y for div in divs] for divs in page_divs]
+    assert positions_y == [[10, 70], [10]]
+
+
+@assert_no_logs
+def test_page_breaks_2():
+    # last div does not fit, pushed to next page
+    # center div must not
+    pages = render_pages('''
+      <style>
+        @page{
+          size: 110px;
+          margin: 10px;
+          padding: 0;
+        }
+        .large {
+          width: 10px;
+          height: 60px;
+        }
+        .small {
+          width: 10px;
+          height: 20px;
+          page-break-after: avoid;
+        }
+      </style>
+      <body>
+        <div class="large"></div>
+        <div class="small"></div>
+        <div class="large"></div>
+    ''')
+
+    assert len(pages) == 2
+    page_divs = []
+    for page in pages:
+        divs = [div for div in page.descendants() if div.element_tag == 'div']
+        assert all([div.element_tag == 'div' for div in divs])
+        page_divs.append(divs)
+    positions_y = [[div.position_y for div in divs] for divs in page_divs]
+    assert positions_y == [[10], [10, 30]]
+
+
+@assert_no_logs
+def test_page_breaks_3():
+    # center div must be the last element,
+    # but div won't fit and will get pushed anyway
+    pages = render_pages('''
+      <style>
+        @page{
+          size: 110px;
+          margin: 10px;
+          padding: 0;
+        }
+        .large {
+          width: 10px;
+          height: 80px;
+        }
+        .small {
+          width: 10px;
+          height: 20px;
+          page-break-after: avoid;
+        }
+      </style>
+      <body>
+        <div class="large"></div>
+        <div class="small"></div>
+        <div class="large"></div>
+    ''')
+
+    assert len(pages) == 3
+    page_divs = []
+    for page in pages:
+        divs = [div for div in page.descendants() if div.element_tag == 'div']
+        assert all([div.element_tag == 'div' for div in divs])
+        page_divs.append(divs)
+    positions_y = [[div.position_y for div in divs] for divs in page_divs]
+    assert positions_y == [[10], [10], [10]]
