@@ -1,10 +1,4 @@
-"""
-    weasyprint.tests.test_boxes
-    ---------------------------
-
-    Test that the "before layout" box tree is correctly constructed.
-
-"""
+"""Test that the "before layout" box tree is correctly constructed."""
 
 import pytest
 from weasyprint.css import PageType, get_all_computed_styles
@@ -228,8 +222,6 @@ def test_styles():
 
 @assert_no_logs
 def test_whitespace():
-    # TODO: test more cases
-    # http://www.w3.org/TR/CSS21/text.html#white-space-model
     assert_tree(parse_all('''
       <p>Lorem \t\r\n  ipsum\t<strong>  dolor
         <img src=pattern.png> sit
@@ -344,10 +336,10 @@ def test_images_2():
 
 @assert_no_logs
 def test_tables_1():
-    # Rules in http://www.w3.org/TR/CSS21/tables.html#anonymous-boxes
+    # Rules in https://www.w3.org/TR/CSS21/tables.html#anonymous-boxes
 
     # Rule 1.3
-    # Also table model: http://www.w3.org/TR/CSS21/tables.html#model
+    # Also table model: https://www.w3.org/TR/CSS21/tables.html#model
     assert_tree(parse_all('''
       <x-table>
         <x-tr>
@@ -414,7 +406,7 @@ def test_tables_2():
 
 @assert_no_logs
 def test_tables_3():
-    # http://www.w3.org/TR/CSS21/tables.html#anonymous-boxes
+    # https://www.w3.org/TR/CSS21/tables.html#anonymous-boxes
     # Rules 1.1 and 1.2
     # Rule XXX (not in the spec): column groups have at least one column child
     assert_tree(parse_all('''
@@ -1064,6 +1056,41 @@ def test_margin_box_string_set_9():
     assert text_box.text == 'first assignment second assignment'
 
 
+@assert_no_logs
+def test_margin_box_string_set_10():
+    page_1, page_2, page_3, page_4 = render_pages('''
+      <style>
+        @page { @top-left  { content: '[' string(p, start) ']' } }
+        p { string-set: p content(); page-break-after: always }
+      </style>
+      <article></article>
+      <p>1</p>
+      <article></article>
+      <p>2</p>
+      <p>3</p>
+      <article></article>
+    ''')
+    html, top_left = page_1.children
+    left_line_box, = top_left.children
+    left_text_box, = left_line_box.children
+    assert left_text_box.text == '[]'
+
+    html, top_left = page_2.children
+    left_line_box, = top_left.children
+    left_text_box, = left_line_box.children
+    assert left_text_box.text == '[1]'
+
+    html, top_left = page_3.children
+    left_line_box, = top_left.children
+    left_text_box, = left_line_box.children
+    assert left_text_box.text == '[3]'
+
+    html, top_left = page_4.children
+    left_line_box, = top_left.children
+    left_text_box, = left_line_box.children
+    assert left_text_box.text == '[3]'
+
+
 @assert_no_logs
 def test_page_counters():
     """Test page-based counters."""
