@@ -1,6 +1,6 @@
 """Test expanders for shorthand properties."""
 
-import math
+from math import pi
 
 import pytest
 import tinycss2
@@ -18,7 +18,7 @@ def expand_to_dict(css, expected_error=None):
     declarations = tinycss2.parse_declaration_list(css)
 
     with capture_logs() as logs:
-        base_url = 'http://weasyprint.org/foo/'
+        base_url = 'https://weasyprint.org/foo/'
         declarations = list(preprocess_declarations(base_url, declarations))
 
     if expected_error:
@@ -223,8 +223,7 @@ def test_size_invalid(rule):
     ('transform: none', {'transform': ()}),
     ('transform: translate(6px) rotate(90deg)', {
         'transform': (
-            ('translate', ((6, 'px'), (0, 'px'))),
-            ('rotate', math.pi / 2))}),
+            ('translate', ((6, 'px'), (0, 'px'))), ('rotate', pi / 2))}),
     ('transform: translate(-4px, 0)', {
         'transform': (('translate', ((-4, 'px'), (0, None))),)}),
     ('transform: translate(6px, 20%)', {
@@ -360,7 +359,7 @@ def test_expand_borders_invalid():
         'list_style_type': 'inherit',
     }),
     ('list-style: url(../bar/lipsum.png)', {
-        'list_style_image': ('url', 'http://weasyprint.org/bar/lipsum.png'),
+        'list_style_image': ('url', 'https://weasyprint.org/bar/lipsum.png'),
     }),
     ('list-style: square', {
         'list_style_type': 'square',
@@ -422,7 +421,7 @@ def test_expand_background():
     assert_background('red', background_color=(1, 0, 0, 1))
     assert_background(
         'url(lipsum.png)',
-        background_image=[('url', 'http://weasyprint.org/foo/lipsum.png')])
+        background_image=[('url', 'https://weasyprint.org/foo/lipsum.png')])
     assert_background(
         'no-repeat',
         background_repeat=[('no-repeat', 'no-repeat')])
@@ -468,7 +467,7 @@ def test_expand_background():
     assert_background(
         'url(bar) #f00 repeat-y center left fixed',
         background_color=(1, 0, 0, 1),
-        background_image=[('url', 'http://weasyprint.org/foo/bar')],
+        background_image=[('url', 'https://weasyprint.org/foo/bar')],
         background_repeat=[('no-repeat', 'repeat')],
         background_attachment=['fixed'],
         background_position=[('left', (0, '%'), 'top', (50, '%'))])
@@ -515,7 +514,7 @@ def test_expand_background():
     assert_background(
         'url(bar) center, no-repeat',
         background_color=(0, 0, 0, 0),
-        background_image=[('url', 'http://weasyprint.org/foo/bar'),
+        background_image=[('url', 'https://weasyprint.org/foo/bar'),
                           ('none', None)],
         background_position=[('left', (50, '%'), 'top', (50, '%')),
                              ('left', (0, '%'), 'top', (0, '%'))],
@@ -818,7 +817,6 @@ def test_linear_gradient():
     red = (1, 0, 0, 1)
     lime = (0, 1, 0, 1)
     blue = (0, 0, 1, 1)
-    pi = math.pi
 
     def gradient(css, direction, colors=(blue,), stop_positions=(None,)):
         for repeating, prefix in ((False, ''), (True, 'repeating-')):
@@ -1218,3 +1216,35 @@ def test_text_align(rule, result):
 ))
 def test_text_align_invalid(rule, reason):
     assert_invalid(rule, reason)
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule, result', (
+    ('image-orientation: none', {'image_orientation': 'none'}),
+    ('image-orientation: from-image', {'image_orientation': 'from-image'}),
+    ('image-orientation: 90deg', {'image_orientation': (pi / 2, False)}),
+    ('image-orientation: 30deg', {'image_orientation': (pi / 6, False)}),
+    ('image-orientation: 180deg flip', {'image_orientation': (pi, True)}),
+    ('image-orientation: 0deg flip', {'image_orientation': (0, True)}),
+    ('image-orientation: flip 90deg', {'image_orientation': (pi / 2, True)}),
+    ('image-orientation: flip', {'image_orientation': (0, True)}),
+))
+def test_image_orientation(rule, result):
+    assert expand_to_dict(rule) == result
+
+
+@assert_no_logs
+@pytest.mark.parametrize('rule, reason', (
+    ('image-orientation: none none', 'invalid'),
+    ('image-orientation: unknown', 'invalid'),
+    ('image-orientation: none flip', 'invalid'),
+    ('image-orientation: from-image flip', 'invalid'),
+    ('image-orientation: 10', 'invalid'),
+    ('image-orientation: 10 flip', 'invalid'),
+    ('image-orientation: flip 10', 'invalid'),
+    ('image-orientation: flip flip', 'invalid'),
+    ('image-orientation: 90deg flop', 'invalid'),
+    ('image-orientation: 90deg 180deg', 'invalid'),
+))
+def test_image_orientation_invalid(rule, reason):
+    assert_invalid(rule, reason)
