@@ -1,10 +1,4 @@
-"""
-    weasyprint.tests.layout
-    -----------------------
-
-    Tests for floating boxes layout.
-
-"""
+"""Tests for floating boxes layout."""
 
 import pytest
 from weasyprint.formatting_structure import boxes
@@ -246,7 +240,6 @@ def test_floats_page_breaks_1():
         assert all([img.element_tag == 'img' for img in images])
         assert all([img.position_x == 10 for img in images])
         page_images.append(images)
-        del images
     positions_y = [[img.position_y for img in images]
                    for images in page_images]
     assert positions_y == [[10], [10]]
@@ -274,7 +267,6 @@ def test_floats_page_breaks_2():
         assert all([img.element_tag == 'img' for img in images])
         assert all([img.position_x == 10 for img in images])
         page_images.append(images)
-        del images
     positions_y = [[img.position_y for img in images]
                    for images in page_images]
     assert positions_y == [[10], [10]]
@@ -306,127 +298,11 @@ def test_floats_page_breaks_3():
         assert all([img.element_tag == 'img' for img in images])
         assert all([img.position_x == 10 for img in images])
         page_images.append(images)
-        del images
     positions_y = [[img.position_y for img in images]
                    for images in page_images]
     assert positions_y == [[10, 40], [10, 40], [10]]
 
 
-@assert_no_logs
-def test_floats_page_breaks_4():
-    # last float does not fit, pushed to next page
-    pages = render_pages('''
-      <style>
-        @page{
-          size: 110px;
-          margin: 10px;
-          padding: 0;
-        }
-        .large {
-          width: 10px;
-          height: 60px;
-        }
-        .small {
-          width: 10px;
-          height: 20px;
-        }
-      </style>
-      <body>
-        <div class="large"></div>
-        <div class="small"></div>
-        <div class="large"></div>
-    ''')
-
-    assert len(pages) == 2
-    page_divs = []
-    for page in pages:
-        divs = [div for div in page.descendants() if div.element_tag == 'div']
-        assert all([div.element_tag == 'div' for div in divs])
-        page_divs.append(divs)
-        del divs
-
-    positions_y = [[div.position_y for div in divs] for divs in page_divs]
-    assert positions_y == [[10, 70], [10]]
-
-
-@assert_no_logs
-def test_floats_page_breaks_5():
-    # last float does not fit, pushed to next page
-    # center div must not
-    pages = render_pages('''
-      <style>
-        @page{
-          size: 110px;
-          margin: 10px;
-          padding: 0;
-        }
-        .large {
-          width: 10px;
-          height: 60px;
-        }
-        .small {
-          width: 10px;
-          height: 20px;
-          page-break-after: avoid;
-        }
-      </style>
-      <body>
-        <div class="large"></div>
-        <div class="small"></div>
-        <div class="large"></div>
-    ''')
-
-    assert len(pages) == 2
-    page_divs = []
-    for page in pages:
-        divs = [div for div in page.descendants() if div.element_tag == 'div']
-        assert all([div.element_tag == 'div' for div in divs])
-        page_divs.append(divs)
-        del divs
-
-    positions_y = [[div.position_y for div in divs] for divs in page_divs]
-    assert positions_y == [[10], [10, 30]]
-
-
-@assert_no_logs
-def test_floats_page_breaks_6():
-    # center div must be the last element,
-    # but float won't fit and will get pushed anyway
-    pages = render_pages('''
-      <style>
-        @page{
-          size: 110px;
-          margin: 10px;
-          padding: 0;
-        }
-        .large {
-          width: 10px;
-          height: 80px;
-        }
-        .small {
-          width: 10px;
-          height: 20px;
-          page-break-after: avoid;
-        }
-      </style>
-      <body>
-        <div class="large"></div>
-        <div class="small"></div>
-        <div class="large"></div>
-    ''')
-
-    assert len(pages) == 3
-    page_divs = []
-    for page in pages:
-        divs = [div for div in page.descendants() if div.element_tag == 'div']
-        assert all([div.element_tag == 'div' for div in divs])
-        page_divs.append(divs)
-        del divs
-
-    positions_y = [[div.position_y for div in divs] for divs in page_divs]
-    assert positions_y == [[10], [10], [10]]
-
-
 @assert_no_logs
 def test_preferred_widths_1():
     def get_float_width(body_width):
