@@ -1,13 +1,8 @@
-"""
-    weasyprint.tests.test_text
-    --------------------------
-
-    Test the text layout.
-
-"""
+"""Test the text layout."""
 
 import pytest
 from weasyprint.css.properties import INITIAL_VALUES
+from weasyprint.formatting_structure.build import capitalize
 from weasyprint.text.line_break import split_first_line
 
 from .testing_utils import MONO_FONTS, SANS_FONTS, assert_no_logs, render_pages
@@ -50,13 +45,11 @@ def test_line_breaking():
     assert resume_index is None
 
     _, _, resume_index, _, _, _ = make_text(string, 90, font_size=100)
-    assert string.encode('utf-8')[resume_index:].decode('utf-8') == (
-        'is a text for test')
+    assert string.encode()[resume_index:].decode() == 'is a text for test'
 
     _, _, resume_index, _, _, _ = make_text(
         string, 100, font_family=SANS_FONTS.split(','), font_size=19)
-    assert string.encode('utf-8')[resume_index:].decode('utf-8') == (
-        'text for test')
+    assert string.encode()[resume_index:].decode() == 'text for test'
 
 
 @assert_no_logs
@@ -68,8 +61,7 @@ def test_line_breaking_rtl():
     assert resume_index is None
 
     _, _, resume_index, _, _, _ = make_text(string, 90, font_size=100)
-    assert string.encode('utf-8')[resume_index:].decode('utf-8') == (
-        'ايبسوم دولا')
+    assert string.encode()[resume_index:].decode() == 'ايبسوم دولا'
 
 
 @assert_no_logs
@@ -713,53 +705,53 @@ def test_hyphenate_character_5():
 
 
 @assert_no_logs
-def test_hyphenate_manual_1():
-    for i in range(1, len('hyphénation')):
-        for hyphenate_character in ('!', 'ù ù'):
-            word = 'hyphénation'[:i] + '\u00ad' + 'hyphénation'[i:]
-            page, = render_pages(
-                '<html style="width: 5em; font-family: weasyprint">'
-                '<style>@font-face {'
-                '  src: url(weasyprint.otf); font-family: weasyprint}</style>'
-                '<body style="hyphens: manual;'
-                f'  hyphenate-character: \'{hyphenate_character}\'"'
-                f'  lang=fr>{word}')
-            html, = page.children
-            body, = html.children
-            lines = body.children
-            assert len(lines) == 2
-            assert lines[0].children[0].text.endswith(hyphenate_character)
-            full_text = ''.join(
-                child.text for line in lines for child in line.children)
-            assert full_text.replace(hyphenate_character, '') == word
+@pytest.mark.parametrize('i', (range(1, len('hyphénation'))))
+def test_hyphenate_manual_1(i):
+    for hyphenate_character in ('!', 'ù ù'):
+        word = f'{"hyphénation"[:i]}\xad{"hyphénation"[i:]}'
+        page, = render_pages(
+            '<html style="width: 5em; font-family: weasyprint">'
+            '<style>@font-face {'
+            '  src: url(weasyprint.otf); font-family: weasyprint}</style>'
+            '<body style="hyphens: manual;'
+            f'  hyphenate-character: \'{hyphenate_character}\'"'
+            f'  lang=fr>{word}')
+        html, = page.children
+        body, = html.children
+        lines = body.children
+        assert len(lines) == 2
+        assert lines[0].children[0].text.endswith(hyphenate_character)
+        full_text = ''.join(
+            child.text for line in lines for child in line.children)
+        assert full_text.replace(hyphenate_character, '') == word
 
 
 @assert_no_logs
-def test_hyphenate_manual_2():
-    for i in range(1, len('hy phénation')):
-        for hyphenate_character in ('!', 'ù ù'):
-            word = 'hy phénation'[:i] + '\u00ad' + 'hy phénation'[i:]
-            page, = render_pages(
-                '<html style="width: 5em; font-family: weasyprint">'
-                '<style>@font-face {'
-                '  src: url(weasyprint.otf); font-family: weasyprint}</style>'
-                '<body style="hyphens: manual;'
-                f'  hyphenate-character: \'{hyphenate_character}\'"'
-                f'  lang=fr>{word}')
-            html, = page.children
-            body, = html.children
-            lines = body.children
-            assert len(lines) in (2, 3)
-            full_text = ''.join(
-                child.text for line in lines for child in line.children)
-            full_text = full_text.replace(hyphenate_character, '')
-            if lines[0].children[0].text.endswith(hyphenate_character):
-                assert full_text == word
-            else:
-                assert lines[0].children[0].text.endswith('y')
-                if len(lines) == 3:
-                    assert lines[1].children[0].text.endswith(
-                        hyphenate_character)
+@pytest.mark.parametrize('i', (range(1, len('hy phénation'))))
+def test_hyphenate_manual_2(i):
+    for hyphenate_character in ('!', 'ù ù'):
+        word = f'{"hy phénation"[:i]}\xad{"hy phénation"[i:]}'
+        page, = render_pages(
+            '<html style="width: 5em; font-family: weasyprint">'
+            '<style>@font-face {'
+            '  src: url(weasyprint.otf); font-family: weasyprint}</style>'
+            '<body style="hyphens: manual;'
+            f'  hyphenate-character: \'{hyphenate_character}\'"'
+            f'  lang=fr>{word}')
+        html, = page.children
+        body, = html.children
+        lines = body.children
+        assert len(lines) in (2, 3)
+        full_text = ''.join(
+            child.text for line in lines for child in line.children)
+        full_text = full_text.replace(hyphenate_character, '')
+        if lines[0].children[0].text.endswith(hyphenate_character):
+            assert full_text == word
+        else:
+            assert lines[0].children[0].text.rstrip('\xad').endswith('y')
+            if len(lines) == 3:
+                assert lines[1].children[0].text.rstrip('\xad').endswith(
+                    hyphenate_character)
 
 
 @assert_no_logs
@@ -941,6 +933,41 @@ def test_overflow_wrap(wrap, text, test, full_text):
     assert full_text == lines_full_text
 
 
+@assert_no_logs
+@pytest.mark.parametrize('span_css, expected_lines', (
+    # overflow-wrap: anywhere and break-word are only allowed to break a word
+    # "if there are no otherwise-acceptable break points in the line", which
+    # means they should not split a word if it fits cleanly into the next line.
+    # This can be done accidentally if it is in its own inline element.
+    ('overflow-wrap: anywhere', ['aaa', 'bbb']),
+    ('overflow-wrap: break-word', ['aaa', 'bbb']),
+
+    # On the other hand, word-break: break-all mandates a break anywhere at the
+    # end of a line, even if the word could fit cleanly onto the next line.
+    ('word-break: break-all', ['aaa b', 'bb']),
+))
+def test_wrap_overflow_word_break(span_css, expected_lines):
+    page, = render_pages('''
+      <style>
+        @font-face {src: url(weasyprint.otf); font-family: weasyprint}
+        body {width: 80px; overflow: hidden; font-family: weasyprint}
+        span {%s}
+      </style>
+      <body>
+        <span>aaa </span><span>bbb
+    ''' % span_css)
+    html, = page.children
+    body, = html.children
+    lines = body.children
+    lines = []
+    for line in body.children:
+        line_text = ''
+        for span_box in line.children:
+            line_text += span_box.children[0].text
+        lines.append(line_text)
+    assert lines == expected_lines
+
+
 @assert_no_logs
 @pytest.mark.parametrize('wrap, text, body_width, expected_width', (
     ('anywhere', 'aaaaaa', 10, 20),
@@ -968,9 +995,7 @@ def test_overflow_wrap_2(wrap, text, body_width, expected_width):
 
 @assert_no_logs
 @pytest.mark.parametrize('wrap, text, body_width, expected_width', (
-    # TODO: broken with Pango 1.50,
-    # see https://gitlab.gnome.org/GNOME/pango/-/issues/646
-    # ('anywhere', 'aaaaaa', 10, 20),
+    ('anywhere', 'aaaaaa', 10, 20),
     ('anywhere', 'aaaaaa', 40, 40),
     ('break-word', 'aaaaaa', 40, 120),
     ('normal', 'abcdef', 40, 120),
@@ -1204,7 +1229,7 @@ def test_text_transform():
     p1, p2, p3, p4, p5 = body.children
     line1, = p1.children
     text1, = line1.children
-    assert text1.text == 'Hé Lo1'
+    assert text1.text == 'Hé LO1'
     line2, = p2.children
     text2, = line2.children
     assert text2.text == 'HÉ LO1'
@@ -1219,6 +1244,28 @@ def test_text_transform():
     assert text5.text == 'hé lO1'
 
 
+@assert_no_logs
+@pytest.mark.parametrize(
+    'original, transformed', (
+        ('abc def ghi', 'Abc Def Ghi'),
+        ('AbC def ghi', 'AbC Def Ghi'),
+        ('I’m SO cool', 'I’m SO Cool'),
+        ('Wow.wow!wow', 'Wow.wow!wow'),
+        ('!now not tomorrow', '!Now Not Tomorrow'),
+        ('SUPER cool', 'SUPER Cool'),
+        ('i 😻 non‑breaking characters', 'I 😻 Non‑breaking Characters'),
+        ('3lite 3lite', '3lite 3lite'),
+        ('one/two/three', 'One/two/three'),
+        ('supernatural,super', 'Supernatural,super'),
+        ('éternel αιώνια', 'Éternel Αιώνια'),
+    )
+)
+def test_text_transform_capitalize(original, transformed):
+    # Results are different for different browsers, we almost get the same
+    # results as Firefox, that’s good enough!
+    assert capitalize(original) == transformed
+
+
 @assert_no_logs
 def test_text_floating_pre_line():
     # Test regression: https://github.com/Kozea/WeasyPrint/issues/610
