@@ -6,26 +6,23 @@ import tempfile
 
 from weasyprint.urls import ensure_url
 
-from .draw import assert_pixels_equal, document_to_pixels, html_to_pixels
+from .draw import document_to_pixels, html_to_pixels
 from .testing_utils import FakeHTML, assert_no_logs, resource_filename
 
 
 @assert_no_logs
-def test_unicode():
+def test_unicode(assert_pixels_equal):
     text = 'I løvë Unicode'
     style = '''
-      @page {
-        background: #fff;
-        size: 200px 50px;
-      }
+      @page { size: 200px 50px }
       p { color: blue }
     '''
-    expected_lines = html_to_pixels('unicode_reference', 200, 50, '''
+    expected_width, expected_height, expected_lines = html_to_pixels('''
       <style>{0}</style>
       <p><img src="pattern.png"> {1}</p>
     '''.format(style, text))
 
-    temp = tempfile.mkdtemp(prefix=text + '-')
+    temp = tempfile.mkdtemp(prefix=f'{text}-')
     try:
         stylesheet = os.path.join(temp, 'style.css')
         image = os.path.join(temp, 'pattern.png')
@@ -46,7 +43,8 @@ def test_unicode():
             fd.write(html_content.encode())
 
         document = FakeHTML(html, encoding='utf-8')
-        lines = document_to_pixels(document, 'unicode', 200, 50)
-        assert_pixels_equal('unicode', 200, 50, lines, expected_lines)
+        width, height, lines = document_to_pixels(document)
+        assert (expected_width, expected_height) == (width, height)
+        assert_pixels_equal(width, height, lines, expected_lines)
     finally:
         shutil.rmtree(temp)
