@@ -4,7 +4,7 @@ This module takes care of steps 3 and 4 of “CSS 2.1 processing model”: Retri
 stylesheets associated with a document and annotate every element with a value
 for every CSS property.
 
-http://www.w3.org/TR/CSS21/intro.html#processing-model
+https://www.w3.org/TR/CSS21/intro.html#processing-model
 
 This module does this in more than two steps. The
 :func:`get_all_computed_styles` function does everything, but it is itsef based
@@ -48,7 +48,7 @@ class StyleFor:
         #     values: (values, weight)
         #         values: a PropertyValue-like object
         #         weight: values with a greater weight take precedence, see
-        #             http://www.w3.org/TR/CSS21/cascade.html#cascading-order
+        #             https://www.w3.org/TR/CSS21/cascade.html#cascading-order
         self._cascaded_styles = cascaded_styles = {}
 
         # keys: (element, pseudo_element_type), like cascaded_styles
@@ -123,13 +123,13 @@ class StyleFor:
             if ('table' in style['display'] and
                     style['border_collapse'] == 'collapse'):
                 # Padding do not apply
-                for side in ['top', 'bottom', 'left', 'right']:
+                for side in ('top', 'bottom', 'left', 'right'):
                     style[f'padding_{side}'] = computed_values.ZERO_PIXELS
             if (len(style['display']) == 1 and
                     style['display'][0].startswith('table-') and
                     style['display'][0] != 'table-caption'):
                 # Margins do not apply
-                for side in ['top', 'bottom', 'left', 'right']:
+                for side in ('top', 'bottom', 'left', 'right'):
                     style[f'margin_{side}'] = computed_values.ZERO_PIXELS
 
         return style
@@ -586,7 +586,7 @@ def declaration_precedence(origin, importance):
     and ``'user agent'``.
 
     """
-    # See http://www.w3.org/TR/CSS21/cascade.html#cascading-order
+    # See https://www.w3.org/TR/CSS21/cascade.html#cascading-order
     if origin == 'user agent':
         return 1
     elif origin == 'user' and not importance:
@@ -626,18 +626,18 @@ class AnonymousStyle(dict):
         return copy
 
     def __missing__(self, key):
-        if key in INHERITED or key.startswith('__'):
-            self[key] = self.parent_style[key]
+        if key in INHERITED or key[:2] == '__':
+            value = self[key] = self.parent_style[key]
         elif key == 'page':
             # page is not inherited but taken from the ancestor if 'auto'
-            self[key] = self.parent_style[key]
-        elif key.startswith('text_decoration_'):
-            self[key] = text_decoration(
+            value = self[key] = self.parent_style[key]
+        elif key[:16] == 'text_decoration_':
+            value = self[key] = text_decoration(
                 key, INITIAL_VALUES[key], self.parent_style[key],
                 cascaded=False)
         else:
-            self[key] = INITIAL_VALUES[key]
-        return self[key]
+            value = self[key] = INITIAL_VALUES[key]
+        return value
 
 
 class ComputedStyle(dict):
@@ -676,7 +676,7 @@ class ComputedStyle(dict):
         if key in self.cascaded:
             value = keyword = self.cascaded[key][0]
         else:
-            if key in INHERITED or key.startswith('__'):
+            if key in INHERITED or key[:2] == '__':
                 keyword = 'inherit'
             else:
                 keyword = 'initial'
@@ -686,7 +686,7 @@ class ComputedStyle(dict):
             keyword = 'initial'
 
         if keyword == 'initial':
-            value = None if key.startswith('__') else INITIAL_VALUES[key]
+            value = None if key[:2] == '__' else INITIAL_VALUES[key]
             if key not in INITIAL_NOT_COMPUTED:
                 # The value is the same as when computed
                 self[key] = value
@@ -694,20 +694,22 @@ class ComputedStyle(dict):
             # Values in parent_style are already computed.
             self[key] = value = self.parent_style[key]
 
-        if key.startswith('text_decoration_') and self.parent_style:
-            self[key] = text_decoration(
+        if key[:16] == 'text_decoration_' and self.parent_style:
+            value = text_decoration(
                 key, value, self.parent_style[key], key in self.cascaded)
-
-        if key == 'page' and value == 'auto':
+            if key in self:
+                del self[key]
+        elif key == 'page' and value == 'auto':
             # The page property does not inherit. However, if the page
             # value on an element is auto, then its used value is the value
             # specified on its nearest ancestor with a non-auto value. When
             # specified on the root element, the used value for auto is the
             # empty string.
-            self['page'] = value = (
+            value = (
                 '' if self.parent_style is None else self.parent_style['page'])
-
-        if key in ('position', 'float', 'display'):
+            if key in self:
+                del self[key]
+        elif key in ('position', 'float', 'display'):
             self.specified[key] = value
 
         if key in self:
@@ -748,7 +750,7 @@ def computed_from_cascaded(element, cascaded, parent_style, pseudo_type=None,
 
     style = ComputedStyle(
         parent_style, cascaded, element, pseudo_type, root_style, base_url)
-    if target_collector:
+    if target_collector and style['anchor']:
         target_collector.collect_anchor(style['anchor'])
     return style
 
