@@ -1,20 +1,15 @@
-"""
-    weasyprint.css.computed_values
-    ------------------------------
-
-    Convert *specified* property values (the result of the cascade and
-    inhertance) into *computed* values (that are inherited).
-
-"""
+"""Convert specified property values into computed values."""
 
 from collections import OrderedDict
+from contextlib import suppress
+from math import pi
 from urllib.parse import unquote
 
 from tinycss2.color3 import parse_color
 
 from ..logger import LOGGER
 from ..text.ffi import ffi, pango, units_to_double
-from ..text.line_break import Layout, first_line_metrics, line_size
+from ..text.line_break import Layout, first_line_metrics
 from ..urls import get_link_attribute
 from .properties import (
     INHERITED, INITIAL_NOT_COMPUTED, INITIAL_VALUES, Dimension)
@@ -28,7 +23,7 @@ ZERO_PIXELS = Dimension(0, 'px')
 
 # Value in pixels of font-size for <absolute-size> keywords: 12pt (16px) for
 # medium, and scaling factors given in CSS3 for others:
-# http://www.w3.org/TR/css3-fonts/#font-size-prop
+# https://www.w3.org/TR/css-fonts-3/#font-size-prop
 FONT_SIZE_KEYWORDS = OrderedDict(
     # medium is 16px, others are a ratio of medium
     (name, INITIAL_VALUES['font_size'] * a / b)
@@ -52,7 +47,7 @@ BORDER_WIDTH_KEYWORDS = {
 }
 assert INITIAL_VALUES['border_top_width'] == BORDER_WIDTH_KEYWORDS['medium']
 
-# http://www.w3.org/TR/CSS21/fonts.html#propdef-font-weight
+# https://www.w3.org/TR/CSS21/fonts.html#propdef-font-weight
 FONT_WEIGHT_RELATIVE = dict(
     bolder={
         100: 400,
@@ -78,7 +73,7 @@ FONT_WEIGHT_RELATIVE = dict(
     },
 )
 
-# http://www.w3.org/TR/css3-page/#size
+# https://www.w3.org/TR/css-page-3/#size
 # name=(width in pixels, height in pixels)
 PAGE_SIZES = {
     'a10': (Dimension(26, 'mm'), Dimension(37, 'mm'),),
@@ -155,10 +150,10 @@ COMPUTING_ORDER = _computing_order()
 COMPUTER_FUNCTIONS = {}
 
 
-def _resolve_var(computed, variable_name, default):
+def _resolve_var(computed, variable_name, default, parent_style):
     known_variable_names = [variable_name]
 
-    computed_value = computed.get(variable_name)
+    computed_value = computed[variable_name]
     if computed_value and len(computed_value) == 1:
         value = computed_value[0]
         if value.type == 'ident' and value.value == 'initial':
@@ -175,13 +170,36 @@ def _resolve_var(computed, variable_name, default):
                 computed_value = default
                 break
             known_variable_names.append(new_variable_name)
-            computed_value = computed.get(new_variable_name, new_default)
             default = new_default
+            computed_value = computed[new_variable_name]
+            if computed_value is not None:
+                continue
+            if parent_style is None:
+                computed_value = new_default
+            else:
+                computed_value = parent_style[new_variable_name] or new_default
         else:
             break
     return computed_value
 
 
+def _font_style_cache_key(style):
+    return str((
+        style['font_family'],
+        style['font_style'],
+        style['font_stretch'],
+        style['font_weight'],
+        style['font_variant_ligatures'],
+        style['font_variant_position'],
+        style['font_variant_caps'],
+        style['font_variant_numeric'],
+        style['font_variant_alternates'],
+        style['font_variant_east_asian'],
+        style['font_feature_settings'],
+        style['font_variation_settings'],
+    ))
+
+
 def register_computer(name):
     """Decorator registering a property ``name`` for a function."""
     name = name.replace('-', '_')
@@ -198,7 +216,8 @@ def compute_variable(value, name, computed, base_url, parent_style):
 
     if value and isinstance(value, tuple) and value[0] == 'var()':
         variable_name, default = value[1]
-        computed_value = _resolve_var(computed, variable_name, default)
+        computed_value = _resolve_var(
+            computed, variable_name, default, parent_style)
         if computed_value is None:
             new_value = None
         else:
@@ -210,11 +229,9 @@ def compute_variable(value, name, computed, base_url, parent_style):
 
         # See https://drafts.csswg.org/css-variables/#invalid-variables
         if new_value is None:
-            try:
+            with suppress(BaseException):
                 computed_value = ''.join(
                     token.serialize() for token in computed_value)
-            except BaseException:
-                pass
             LOGGER.warning(
                 'Unsupported computed value "%s" set in variable %r '
                 'for property %r.', computed_value,
@@ -278,8 +295,6 @@ def length_tuple(style, name, values):
 @register_computer('break-before')
 def break_before_after(style, name, value):
     """Compute the ``break-before`` and ``break-after`` properties."""
-    # 'always' is defined as an alias to 'page' in multi-column
-    # https://www.w3.org/TR/css3-multicol/#column-breaks
     if value == 'always':
         return 'page'
     else:
@@ -324,18 +339,12 @@ def length(style, name, value, font_size=None, pixels_only=False):
         if font_size is None:
             font_size = style['font_size']
         if unit == 'ex':
-            # TODO: cache
-            result = value.value * font_size * ex_ratio(style)
-        elif unit == 'ch':
-            # TODO: cache
             # TODO: use context to use @font-face fonts
-            layout = Layout(
-                context=None, font_size=font_size,
-                style=style)
-            layout.set_text('0')
-            line, _ = layout.get_first_line()
-            logical_width, _ = line_size(line, style)
-            result = value.value * logical_width
+            ratio = character_ratio(style, 'x')
+            result = value.value * font_size * ratio
+        elif unit == 'ch':
+            ratio = character_ratio(style, '0')
+            result = value.value * font_size * ratio
         elif unit == 'em':
             result = value.value * font_size
         elif unit == 'rem':
@@ -378,6 +387,15 @@ def background_size(style, name, values):
         for value in values)
 
 
+@register_computer('image-orientation')
+def image_orientation(style, name, values):
+    """Compute the ``image-orientation`` properties."""
+    if values in ('none', 'from-image'):
+        return values
+    angle, flip = values
+    return (int(round(angle / pi * 2)) % 4 * 90, flip)
+
+
 @register_computer('border-top-width')
 @register_computer('border-right-width')
 @register_computer('border-left-width')
@@ -530,7 +548,7 @@ def content(style, name, values):
 def display(style, name, value):
     """Compute the ``display`` property.
 
-    See http://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo
+    See https://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo
 
     """
     float_ = style.specified['float']
@@ -553,10 +571,11 @@ def display(style, name, value):
 def compute_float(style, name, value):
     """Compute the ``float`` property.
 
-    See http://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo
+    See https://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo
 
     """
-    if style.specified['position'] in ('absolute', 'fixed'):
+    position = style.specified['position']
+    if position in ('absolute', 'fixed') or position[0] == 'running()':
         return 'none'
     else:
         return value
@@ -587,7 +606,7 @@ def font_size(style, name, value):
         else:
             return parent_font_size * 0.8
     elif value.unit == '%':
-        return value.value * parent_font_size / 100.
+        return value.value * parent_font_size / 100
     else:
         return length(
             style, name, value, pixels_only=True,
@@ -619,7 +638,7 @@ def line_height(style, name, value):
     elif not value.unit:
         return ('NUMBER', value.value)
     elif value.unit == '%':
-        factor = value.value / 100.
+        factor = value.value / 100
         font_size_value = style['font_size']
         pixels = factor * font_size_value
     else:
@@ -748,10 +767,17 @@ def strut_layout(style, context=None):
     return result
 
 
-def ex_ratio(style):
-    """Return the ratio 1ex/font_size, according to given style."""
+def character_ratio(style, character):
+    """Return the ratio of 1ex/font_size or 1ch/font_size."""
     # TODO: use context to use @font-face fonts
 
+    assert character in ('x', '0')
+
+    cache = style.cache[f'ratio_{"ex" if character == "x" else "ch"}']
+    cache_key = _font_style_cache_key(style)
+    if cache_key in cache:
+        return cache[cache_key]
+
     # Avoid recursion for letter-spacing and word-spacing properties
     style = style.copy()
     style['letter_spacing'] = 'normal'
@@ -761,14 +787,21 @@ def ex_ratio(style):
     font_size = 1000
 
     layout = Layout(context=None, font_size=font_size, style=style)
-    layout.set_text('x')
+    layout.set_text(character)
     line, _ = layout.get_first_line()
 
     ink_extents = ffi.new('PangoRectangle *')
-    pango.pango_layout_line_get_extents(line, ink_extents, ffi.NULL)
-    height_above_baseline = units_to_double(ink_extents.y)
+    logical_extents = ffi.new('PangoRectangle *')
+    pango.pango_layout_line_get_extents(line, ink_extents, logical_extents)
+    if character == 'x':
+        measure = -units_to_double(ink_extents.y)
+    else:
+        measure = units_to_double(logical_extents.width)
     ffi.release(ink_extents)
+    ffi.release(logical_extents)
 
     # Zero means some kind of failure, fallback is 0.5.
     # We round to try keeping exact values that were altered by Pango.
-    return round(-height_above_baseline / font_size, 5) or 0.5
+    ratio = round(measure / font_size, 5) or 0.5
+    cache[cache_key] = ratio
+    return ratio
