@@ -1,20 +1,15 @@
-"""
-    weasyprint.css.properties
-    -------------------------
-
-    Various data about known properties.
-
-"""
+"""Various data about known CSS properties."""
 
 import collections
+from math import inf
 
 from tinycss2.color3 import parse_color
 
 Dimension = collections.namedtuple('Dimension', ['value', 'unit'])
 
 
-# See http://www.w3.org/TR/CSS21/propidx.html
 INITIAL_VALUES = {
+    # CSS 2.1: https://www.w3.org/TR/CSS21/propidx.html
     'bottom': 'auto',
     'caption_side': 'top',
     'clear': 'none',
@@ -31,8 +26,8 @@ INITIAL_VALUES = {
     'margin_right': Dimension(0, 'px'),
     'margin_bottom': Dimension(0, 'px'),
     'margin_left': Dimension(0, 'px'),
-    'max_height': Dimension(float('inf'), 'px'),  # parsed value for 'none'
-    'max_width': Dimension(float('inf'), 'px'),
+    'max_height': Dimension(inf, 'px'),  # parsed value for 'none'
+    'max_width': Dimension(inf, 'px'),
     'padding_top': Dimension(0, 'px'),
     'padding_right': Dimension(0, 'px'),
     'padding_bottom': Dimension(0, 'px'),
@@ -47,7 +42,7 @@ INITIAL_VALUES = {
     'width': 'auto',
     'z_index': 'auto',
 
-    # Backgrounds and Borders 3 (CR): https://www.w3.org/TR/css3-background/
+    # Backgrounds and Borders 3 (CR): https://www.w3.org/TR/css-backgrounds-3/
     'background_attachment': ('scroll',),
     'background_clip': ('border-box',),
     'background_color': parse_color('transparent'),
@@ -76,7 +71,7 @@ INITIAL_VALUES = {
     'border_top_style': 'none',
     'border_top_width': 3,  # computed value for 'medium'
 
-    # Color 3 (REC): https://www.w3.org/TR/css3-color/
+    # Color 3 (REC): https://www.w3.org/TR/css-color-3/
     'opacity': 1,
 
     # Multi-column Layout (WD): https://www.w3.org/TR/css-multicol-1/
@@ -106,6 +101,9 @@ INITIAL_VALUES = {
     'font_variant_position': 'normal',
     'font_weight': 400,
 
+    # Fonts 4 (WD): https://www.w3.org/TR/css-fonts-4/
+    'font_variation_settings': 'normal',
+
     # Fragmentation 3/4 (CR/WD): https://www.w3.org/TR/css-break-4/
     'box_decoration_break': 'slice',
     'break_after': 'auto',
@@ -125,10 +123,10 @@ INITIAL_VALUES = {
     'quotes': list('“”‘’'),  # chosen by the user agent
     'string_set': 'none',
 
-    # Images 3/4 (CR/WD): https://www.w3.org/TR/css4-images/
+    # Images 3/4 (CR/WD): https://www.w3.org/TR/css-images-4/
     'image_resolution': 1,  # dppx
     'image_rendering': 'auto',
-    # https://drafts.csswg.org/css-images-3/
+    'image_orientation': 'from-image',
     'object_fit': 'fill',
     'object_position': (('left', Dimension(50, '%'),
                          'top', Dimension(50, '%')),),
@@ -154,6 +152,7 @@ INITIAL_VALUES = {
     'text_indent': Dimension(0, 'px'),
     'text_transform': 'none',
     'white_space': 'normal',
+    'word_break': 'normal',
     'word_spacing': 0,  # computed value for 'normal'
 
     # Transforms 1 (CR): https://www.w3.org/TR/css-transforms-1/
@@ -215,11 +214,13 @@ KNOWN_PROPERTIES = set(name.replace('_', '-') for name in INITIAL_VALUES)
 
 # Do not list shorthand properties here as we handle them before inheritance.
 #
+# Values inherited but not applicable to print are not included.
+#
 # text_decoration is not a really inherited, see
-# http://www.w3.org/TR/CSS2/text.html#propdef-text-decoration
+# https://www.w3.org/TR/CSS2/text.html#propdef-text-decoration
 #
 # link: click events normally bubble up to link ancestors
-#   See http://lists.w3.org/Archives/Public/www-style/2012Jun/0315.html
+#   See https://lists.w3.org/Archives/Public/www-style/2012Jun/0315.html
 INHERITED = {
     'block_ellipsis',
     'border_collapse',
@@ -242,6 +243,7 @@ INHERITED = {
     'font_variant_ligatures',
     'font_variant_numeric',
     'font_variant_position',
+    'font_variation_settings',
     'font_weight',
     'hyphens',
     'hyphenate_character',
@@ -262,36 +264,18 @@ INHERITED = {
     'tab_size',
     'text_align_all',
     'text_align_last',
-    'text_decoration_line',
-    'text_decoration_color',
-    'text_decoration_style',
     'text_indent',
     'text_transform',
     'visibility',
     'white_space',
     'widows',
+    'word_break',
     'word_spacing',
 }
 
-# Inherited but not applicable to print:
-#    azimuth
-#    cursor
-#    elevation
-#    pitch_range
-#    pitch
-#    richness
-#    speak_header
-#    speak_numeral
-#    speak_punctuation
-#    speak
-#    speech_rate
-#    stress
-#    voice_family
-#    volume
 
-
-# http://www.w3.org/TR/CSS21/tables.html#model
-# See also http://lists.w3.org/Archives/Public/www-style/2012Jun/0066.html
+# https://www.w3.org/TR/CSS21/tables.html#model
+# See also https://lists.w3.org/Archives/Public/www-style/2012Jun/0066.html
 # Only non-inherited properties need to be included here.
 TABLE_WRAPPER_BOX_PROPERTIES = {
     'bottom',
