@@ -1,12 +1,11 @@
-"""
-    weasyprint.css.validation.properties
-    ------------------------------------
+"""Validate properties.
 
-    Validate properties.
-    See http://www.w3.org/TR/CSS21/propidx.html and various CSS3 modules.
+See https://www.w3.org/TR/CSS21/propidx.html and various CSS3 modules.
 
 """
 
+from math import inf
+
 from tinycss2.color3 import parse_color
 
 from .. import computed_values
@@ -306,8 +305,6 @@ def border_style(keyword):
 @single_keyword
 def break_before_after(keyword):
     """``break-before`` and ``break-after`` properties validation."""
-    # 'always' is defined as an alias to 'page' in multi-column
-    # https://www.w3.org/TR/css3-multicol/#column-breaks
     return keyword in ('auto', 'avoid', 'avoid-page', 'page', 'left', 'right',
                        'recto', 'verso', 'avoid-column', 'column', 'always')
 
@@ -844,6 +841,22 @@ def font_variant_east_asian(tokens):
         return tuple(values)
 
 
+@property()
+def font_variation_settings(tokens):
+    """``font-variation-settings`` property validation."""
+    if len(tokens) == 1 and get_keyword(tokens[0]) == 'normal':
+        return 'normal'
+
+    @comma_separated_list
+    def font_variation_settings_list(tokens):
+        if len(tokens) == 2:
+            key, value = tokens
+            if key.type == 'string' and value.type == 'number':
+                return key.value, value.value
+
+    return font_variation_settings_list(tokens)
+
+
 @property()
 @single_token
 def font_size(token):
@@ -1005,7 +1018,7 @@ def max_width_height(token):
     if length:
         return length
     if get_keyword(token) == 'none':
-        return Dimension(float('inf'), 'px')
+        return Dimension(inf, 'px')
 
 
 @property()
@@ -1179,6 +1192,13 @@ def overflow_wrap(keyword):
     return keyword in ('anywhere', 'normal', 'break-word')
 
 
+@property()
+@single_keyword
+def word_break(keyword):
+    """``word-break`` property validation."""
+    return keyword in ('normal', 'break-all')
+
+
 @property()
 @single_token
 def flex_basis(token):
@@ -1260,11 +1280,35 @@ def image_rendering(keyword):
     return keyword in ('auto', 'crisp-edges', 'pixelated')
 
 
+@property(unstable=True)
+def image_orientation(tokens):
+    """Validation for ``image-orientation``."""
+    keyword = get_single_keyword(tokens)
+    if keyword in ('none', 'from-image'):
+        return keyword
+    angle, flip = None, None
+    for token in tokens:
+        keyword = get_keyword(token)
+        if keyword == 'flip':
+            if flip is not None:
+                return
+            flip = True
+            continue
+        if angle is None:
+            angle = get_angle(token)
+            if angle is not None:
+                continue
+        return
+    angle = 0 if angle is None else angle
+    flip = False if flip is None else flip
+    return (angle, flip)
+
+
 @property(unstable=True)
 def size(tokens):
     """``size`` property validation.
 
-    See http://www.w3.org/TR/css3-page/#page-size-prop
+    See https://www.w3.org/TR/css-page-3/#page-size-prop
 
     """
     lengths = [get_length(token, negative=False) for token in tokens]
@@ -1506,7 +1550,7 @@ def transform(tokens):
                 length = get_length(args[0], percentage=True)
                 if name == 'rotate' and angle is not None:
                     transforms.append((name, angle))
-                elif name == 'skewx' and angle is not None:
+                elif name in ('skewx', 'skew') and angle is not None:
                     transforms.append(('skew', (angle, 0)))
                 elif name == 'skewy' and angle is not None:
                     transforms.append(('skew', (0, angle)))
