@@ -1,16 +1,10 @@
-"""
-    weasyprint.draw
-    ---------------
-
-    Take an "after layout" box tree and draw it onto a pydyf stream.
-
-"""
+"""Take an "after layout" box tree and draw it onto a pydyf stream."""
 
 import contextlib
 import operator
 from colorsys import hsv_to_rgb, rgb_to_hsv
 from io import BytesIO
-from math import ceil, floor, pi, sqrt, tan
+from math import ceil, cos, floor, pi, sin, sqrt, tan
 from xml.etree import ElementTree
 
 from PIL import Image
@@ -19,6 +13,7 @@ from .formatting_structure import boxes
 from .images import RasterImage, SVGImage
 from .layout import replaced
 from .layout.background import BackgroundLayer
+from .matrix import Matrix
 from .stacking import StackingContext
 from .text.ffi import ffi, harfbuzz, pango, units_from_double, units_to_double
 from .text.line_break import get_last_word_end
@@ -74,29 +69,21 @@ def draw_page(page, stream):
 
 
 def draw_box_background_and_border(stream, page, box):
-    draw_background(stream, box.background)
     if isinstance(box, boxes.TableBox):
-        draw_table_backgrounds(stream, page, box)
-        if box.style['border_collapse'] == 'separate':
-            draw_border(stream, box)
-            for row_group in box.children:
-                for row in row_group.children:
-                    for cell in row.children:
-                        if (cell.style['empty_cells'] == 'show' or
-                                not cell.empty):
-                            draw_border(stream, cell)
-        else:
-            draw_collapsed_borders(stream, box)
+        draw_table(stream, box)
     else:
+        draw_background(stream, box.background)
         draw_border(stream, box)
 
 
 def draw_stacking_context(stream, stacking_context):
     """Draw a ``stacking_context`` on ``stream``."""
-    # See http://www.w3.org/TR/CSS2/zindex.html
+    # See https://www.w3.org/TR/CSS2/zindex.html
     with stacked(stream):
         box = stacking_context.box
 
+        stream.begin_marked_content(box, mcid=True)
+
         # apply the viewport_overflow to the html box, see #35
         if box.is_for_root_element and (
                 stacking_context.page.style['overflow'] != 'visible'):
@@ -123,15 +110,13 @@ def draw_stacking_context(stream, stacking_context):
 
         if box.style['opacity'] < 1:
             original_stream = stream
-            stream = stream.add_group([
-                box.border_box_x(), box.border_box_y(),
-                box.border_box_x() + box.border_width(),
-                box.border_box_y() + box.border_height()])
+            stream = stream.add_group(stream.page_rectangle)
 
         if box.transformation_matrix:
             if box.transformation_matrix.determinant:
                 stream.transform(*box.transformation_matrix.values)
             else:
+                stream.end_marked_content()
                 return
 
         # Point 1 is done in draw_page
@@ -140,7 +125,7 @@ def draw_stacking_context(stream, stacking_context):
         if isinstance(box, (boxes.BlockBox, boxes.MarginBox,
                             boxes.InlineBlockBox, boxes.TableCellBox,
                             boxes.FlexContainerBox)):
-            # The canvas background was removed by set_canvas_background
+            # The canvas background was removed by layout_backgrounds
             draw_box_background_and_border(stream, stacking_context.page, box)
 
         with stacked(stream):
@@ -174,12 +159,17 @@ def draw_stacking_context(stream, stacking_context):
             # Point 7
             for block in [box] + stacking_context.blocks_and_cells:
                 if isinstance(block, boxes.ReplacedBox):
+                    draw_border(stream, block)
                     draw_replacedbox(stream, block)
-                else:
-                    for child in block.children:
-                        if isinstance(child, boxes.LineBox):
+                elif block.children:
+                    if block != box:
+                        stream.begin_marked_content(block, mcid=True)
+                    if isinstance(block.children[-1], boxes.LineBox):
+                        for child in block.children:
                             draw_inline_level(
                                 stream, stacking_context.page, child)
+                    if block != box:
+                        stream.end_marked_content()
 
             # Point 8
             for child_context in stacking_context.zero_z_contexts:
@@ -200,6 +190,8 @@ def draw_stacking_context(stream, stacking_context):
             stream.draw_x_object(group_id)
             stream.pop_state()
 
+        stream.end_marked_content()
+
 
 def rounded_box_path(stream, radii):
     """Draw the path of the border radius box.
@@ -254,6 +246,8 @@ def draw_background(stream, bg, clip_box=True, bleed=None, marks=()):
         # Background color
         if bg.color.alpha > 0:
             with stacked(stream):
+                stream.set_color_rgb(*bg.color[:3])
+                stream.set_alpha(bg.color.alpha)
                 painting_area = bg.layers[-1].painting_area
                 if painting_area:
                     if bleed:
@@ -267,8 +261,6 @@ def draw_background(stream, bg, clip_box=True, bleed=None, marks=()):
                     stream.clip()
                     stream.end()
                 stream.rectangle(*stream.page_rectangle)
-                stream.set_color_rgb(*bg.color[:3])
-                stream.set_alpha(bg.color.alpha)
                 stream.fill()
 
         if bleed and marks:
@@ -281,7 +273,7 @@ def draw_background(stream, bg, clip_box=True, bleed=None, marks=()):
             svg = f'''
               <svg height="{height}" width="{width}"
                    fill="transparent" stroke="black" stroke-width="1"
-                   xmlns="http://www.w3.org/2000/svg">
+                   xmlns="https://www.w3.org/2000/svg">
             '''
             if 'crop' in marks:
                 svg += f'''
@@ -345,26 +337,7 @@ def draw_background(stream, bg, clip_box=True, bleed=None, marks=()):
             draw_background_image(stream, layer, bg.image_rendering)
 
 
-def draw_table_backgrounds(stream, page, table):
-    """Draw the background color and image of the table children."""
-    for column_group in table.column_groups:
-        draw_background(stream, column_group.background)
-        for column in column_group.children:
-            draw_background(stream, column.background)
-    for row_group in table.children:
-        draw_background(stream, row_group.background)
-        for row in row_group.children:
-            draw_background(stream, row.background)
-            for cell in row.children:
-                if (table.style['border_collapse'] == 'collapse' or
-                        cell.style['empty_cells'] == 'show' or
-                        not cell.empty):
-                    draw_background(stream, cell.background)
-
-
 def draw_background_image(stream, layer, image_rendering):
-    from .document import Matrix
-
     if layer.image is None or 0 in layer.size:
         return
 
@@ -378,8 +351,10 @@ def draw_background_image(stream, layer, image_rendering):
 
     if repeat_x == 'no-repeat':
         # We want at least the whole image_width drawn on sub_surface, but we
-        # want to be sure it will not be repeated on the painting_width.
-        repeat_width = max(image_width, painting_width)
+        # want to be sure it will not be repeated on the painting_width. We
+        # double the painting width to ensure viewers don't incorrectly bleed
+        # the edge of the pattern into the painting area. (See #1539.)
+        repeat_width = max(image_width, 2 * painting_width)
     elif repeat_x in ('repeat', 'round'):
         # We repeat the image each image_width.
         repeat_width = image_width
@@ -395,11 +370,11 @@ def draw_background_image(stream, layer, image_rendering):
             position_x = 0
         else:
             # We don't repeat the image.
-            repeat_width = image_width
+            repeat_width = positioning_width
 
     # Comments above apply here too.
     if repeat_y == 'no-repeat':
-        repeat_height = max(image_height, painting_height)
+        repeat_height = max(image_height, 2 * painting_height)
     elif repeat_y in ('repeat', 'round'):
         repeat_height = image_height
     else:
@@ -410,12 +385,7 @@ def draw_background_image(stream, layer, image_rendering):
                 positioning_height - image_height) / (n_repeats - 1)
             position_y = 0
         else:
-            repeat_height = image_height
-
-    if repeat_x == repeat_y == 'no-repeat':
-        # PDF patterns always repeat, use a big number to hide repetition
-        repeat_width = 2 * stream.page_rectangle[2]
-        repeat_height = 2 * stream.page_rectangle[3]
+            repeat_height = positioning_height
 
     matrix = Matrix(e=position_x + positioning_x, f=position_y + positioning_y)
     matrix @= stream.ctm
@@ -437,11 +407,6 @@ def draw_background_image(stream, layer, image_rendering):
         stream.fill()
 
 
-def xy_offset(x, y, offset_x, offset_y, offset):
-    """Increment X and Y coordinates by the given offsets."""
-    return x + offset_x * offset, y + offset_y * offset
-
-
 def styled_color(style, color, side):
     if style in ('inset', 'outset'):
         do_lighten = (side in ('top', 'left')) ^ (style == 'inset')
@@ -459,6 +424,17 @@ def draw_border(stream, box):
     # We need a plan to draw beautiful borders, and that's difficult, no need
     # to lie. Let's try to find the cases that we can handle in a smart way.
 
+    def get_columns_with_rule():
+        """Yield columns that have a rule drawn on the left."""
+        skip_next = True
+        for child in box.children:
+            if child.style['column_span'] == 'all':
+                skip_next = True
+            elif skip_next:
+                skip_next = False
+            else:
+                yield child
+
     def draw_column_border():
         """Draw column borders."""
         columns = (
@@ -467,14 +443,14 @@ def draw_border(stream, box):
                 box.style['column_count'] != 'auto'))
         if columns and box.style['column_rule_width']:
             border_widths = (0, 0, 0, box.style['column_rule_width'])
-            for child in box.children[1:]:
+            for child in get_columns_with_rule():
                 with stacked(stream):
                     position_x = (child.position_x - (
                         box.style['column_rule_width'] +
                         box.style['column_gap']) / 2)
                     border_box = (
                         position_x, child.position_y,
-                        box.style['column_rule_width'], box.height)
+                        box.style['column_rule_width'], child.height)
                     clip_border_segment(
                         stream, box.style['column_rule_style'],
                         box.style['column_rule_width'], 'left', border_box,
@@ -561,7 +537,7 @@ def clip_border_segment(stream, style, width, side, border_box,
         pi" Quart. J. Pure. Appl. Math., vol. 45 (1913-1914), pp. 350-372],
         wonderfully explained by Dr Rob.
 
-        http://mathforum.org/dr.math/faq/formulas/
+        https://mathforum.org/dr.math/faq/formulas/
 
         """
         x = (a - b) / (a + b)
@@ -710,37 +686,38 @@ def clip_border_segment(stream, style, width, side, border_box,
 
 
 def draw_rounded_border(stream, box, style, color):
-    rounded_box_path(stream, box.rounded_padding_box())
     if style in ('ridge', 'groove'):
-        rounded_box_path(stream, box.rounded_box_ratio(1 / 2))
         stream.set_color_rgb(*color[0][:3])
         stream.set_alpha(color[0][3])
-        stream.fill(even_odd=True)
+        rounded_box_path(stream, box.rounded_padding_box())
         rounded_box_path(stream, box.rounded_box_ratio(1 / 2))
-        rounded_box_path(stream, box.rounded_border_box())
+        stream.fill(even_odd=True)
         stream.set_color_rgb(*color[1][:3])
         stream.set_alpha(color[1][3])
+        rounded_box_path(stream, box.rounded_box_ratio(1 / 2))
+        rounded_box_path(stream, box.rounded_border_box())
         stream.fill(even_odd=True)
         return
+    stream.set_color_rgb(*color[:3])
+    stream.set_alpha(color[3])
+    rounded_box_path(stream, box.rounded_padding_box())
     if style == 'double':
         rounded_box_path(stream, box.rounded_box_ratio(1 / 3))
         rounded_box_path(stream, box.rounded_box_ratio(2 / 3))
     rounded_box_path(stream, box.rounded_border_box())
-    stream.set_color_rgb(*color[:3])
-    stream.set_alpha(color[3])
     stream.fill(even_odd=True)
 
 
 def draw_rect_border(stream, box, widths, style, color):
     bbx, bby, bbw, bbh = box
     bt, br, bb, bl = widths
-    stream.rectangle(*box)
     if style in ('ridge', 'groove'):
+        stream.set_color_rgb(*color[0][:3])
+        stream.set_alpha(color[0][3])
+        stream.rectangle(*box)
         stream.rectangle(
             bbx + bl / 2, bby + bt / 2,
             bbw - (bl + br) / 2, bbh - (bt + bb) / 2)
-        stream.set_color_rgb(*color[0][:3])
-        stream.set_alpha(color[0][3])
         stream.fill(even_odd=True)
         stream.rectangle(
             bbx + bl / 2, bby + bt / 2,
@@ -750,6 +727,9 @@ def draw_rect_border(stream, box, widths, style, color):
         stream.set_alpha(color[1][3])
         stream.fill(even_odd=True)
         return
+    stream.set_color_rgb(*color[:3])
+    stream.set_alpha(color[3])
+    stream.rectangle(*box)
     if style == 'double':
         stream.rectangle(
             bbx + bl / 3, bby + bt / 3,
@@ -758,8 +738,6 @@ def draw_rect_border(stream, box, widths, style, color):
             bbx + bl * 2 / 3, bby + bt * 2 / 3,
             bbw - (bl + br) * 2 / 3, bbh - (bt + bb) * 2 / 3)
     stream.rectangle(bbx + bl, bby + bt, bbw - bl - br, bbh - bt - bb)
-    stream.set_color_rgb(*color[:3])
-    stream.set_alpha(color[3])
     stream.fill(even_odd=True)
 
 
@@ -784,6 +762,34 @@ def draw_outlines(stream, box):
                 draw_outlines(stream, child)
 
 
+def draw_table(stream, table):
+    # Draw backgrounds
+    draw_background(stream, table.background)
+    for column_group in table.column_groups:
+        draw_background(stream, column_group.background)
+        for column in column_group.children:
+            draw_background(stream, column.background)
+    for row_group in table.children:
+        draw_background(stream, row_group.background)
+        for row in row_group.children:
+            draw_background(stream, row.background)
+            for cell in row.children:
+                if (table.style['border_collapse'] == 'collapse' or
+                        cell.style['empty_cells'] == 'show' or
+                        not cell.empty):
+                    draw_background(stream, cell.background)
+
+    # Draw borders
+    if table.style['border_collapse'] == 'collapse':
+        return draw_collapsed_borders(stream, table)
+    draw_border(stream, table)
+    for row_group in table.children:
+        for row in row_group.children:
+            for cell in row.children:
+                if cell.style['empty_cells'] == 'show' or not cell.empty:
+                    draw_border(stream, cell)
+
+
 def draw_collapsed_borders(stream, table):
     """Draw borders of table cells when they collapse."""
     row_heights = [
@@ -952,10 +958,15 @@ def draw_inline_level(stream, page, box, offset_x=0, text_overflow='clip',
         draw_background(stream, box.background)
         draw_border(stream, box)
         if isinstance(box, (boxes.InlineBox, boxes.LineBox)):
+            link_annotation = None
             if isinstance(box, boxes.LineBox):
                 text_overflow = box.text_overflow
                 block_ellipsis = box.block_ellipsis
+            else:
+                link_annotation = box.link_annotation
             ellipsis = 'none'
+            if link_annotation:
+                stream.begin_marked_content(box, mcid=True, tag='Link')
             for i, child in enumerate(box.children):
                 if i == len(box.children) - 1:
                     # Last child
@@ -972,6 +983,8 @@ def draw_inline_level(stream, page, box, offset_x=0, text_overflow='clip',
                     draw_inline_level(
                         stream, page, child, child_offset_x, text_overflow,
                         ellipsis)
+            if link_annotation:
+                stream.end_marked_content()
         elif isinstance(box, boxes.InlineReplacedBox):
             draw_replacedbox(stream, box)
         else:
@@ -1009,16 +1022,19 @@ def draw_text(stream, textbox, offset_x, text_overflow, block_ellipsis):
         thickness = textbox.pango_layout.underline_thickness
         offset_y = (
             textbox.baseline - textbox.pango_layout.ascent + thickness / 2)
+        draw_text_decoration(
+            stream, textbox, offset_x, offset_y, thickness, color)
     if 'underline' in values:
         thickness = textbox.pango_layout.underline_thickness
         offset_y = (
             textbox.baseline - textbox.pango_layout.underline_position +
             thickness / 2)
+        draw_text_decoration(
+            stream, textbox, offset_x, offset_y, thickness, color)
     if 'line-through' in values:
         thickness = textbox.pango_layout.strikethrough_thickness
         offset_y = (
             textbox.baseline - textbox.pango_layout.strikethrough_position)
-    if values != 'none':
         draw_text_decoration(
             stream, textbox, offset_x, offset_y, thickness, color)
 
@@ -1026,16 +1042,20 @@ def draw_text(stream, textbox, offset_x, text_overflow, block_ellipsis):
 
 
 def draw_emojis(stream, font_size, x, y, emojis):
-    for emoji in emojis:
-        for image, font, a, d, e, f in emojis:
-            stream.push_state()
-            stream.transform(a=a, d=d, e=x + e * font_size, f=y + f)
-            image.draw(stream, font_size, font_size, None)
-            stream.pop_state()
+    for image, font, a, d, e, f in emojis:
+        stream.push_state()
+        stream.transform(a=a, d=d, e=x + e * font_size, f=y + f)
+        image.draw(stream, font_size, font_size, None)
+        stream.pop_state()
 
 
-def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
+def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y,
+                    angle=0):
     """Draw the given ``textbox`` line to the document ``stream``."""
+    font_size = textbox.style['font_size']
+    if font_size < 1e-6:  # Default float precision used by pydyf
+        return []
+
     pango.pango_layout_set_single_paragraph_mode(
         textbox.pango_layout.layout, True)
 
@@ -1065,10 +1085,10 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
 
             textbox.pango_layout.set_text(new_text + ellipsis)
 
-    first_line, second_line = textbox.pango_layout.get_first_line()
+    first_line, index = textbox.pango_layout.get_first_line()
 
     if block_ellipsis != 'none':
-        while second_line:
+        while index:
             last_word_end = get_last_word_end(
                 textbox.pango_layout.text[:-len(ellipsis)],
                 textbox.style['lang'])
@@ -1076,25 +1096,26 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
                 break
             new_text = textbox.pango_layout.text[:last_word_end]
             textbox.pango_layout.set_text(new_text + ellipsis)
-            first_line, second_line = textbox.pango_layout.get_first_line()
+            first_line, index = textbox.pango_layout.get_first_line()
 
-    font_size = textbox.style['font_size']
-    utf8_text = textbox.pango_layout.text.encode('utf-8')
+    utf8_text = textbox.pango_layout.text.encode()
     previous_utf8_position = 0
 
-    runs = [first_line.runs[0]]
-    while runs[-1].next != ffi.NULL:
-        runs.append(runs[-1].next)
-
-    stream.text_matrix(font_size, 0, 0, -font_size, x, y)
+    matrix = Matrix(1, 0, 0, -1, x, y)
+    if angle:
+        a, c = cos(angle), sin(angle)
+        b, d = -c, a
+        matrix = Matrix(a, b, c, d) @ matrix
+    stream.text_matrix(*matrix.values)
     last_font = None
     string = ''
-    fonts = stream.get_fonts()
     x_advance = 0
     emojis = []
-    for run in runs:
+    run = first_line.runs[0]
+    while run != ffi.NULL:
         # Pango objects
-        glyph_item = ffi.cast('PangoGlyphItem *', run.data)
+        glyph_item = run.data
+        run = run.next
         glyph_string = glyph_item.glyphs
         glyphs = glyph_string.glyphs
         num_glyphs = glyph_string.num_glyphs
@@ -1103,19 +1124,7 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
 
         # Font content
         pango_font = glyph_item.item.analysis.font
-        hb_font = pango.pango_font_get_hb_font(pango_font)
-        hb_face = harfbuzz.hb_font_get_face(hb_font)
-        font_hash = hash(hb_face)
-        if font_hash in fonts:
-            font = fonts[font_hash]
-        else:
-            hb_blob = ffi.gc(
-                harfbuzz.hb_face_reference_blob(hb_face),
-                harfbuzz.hb_blob_destroy)
-            hb_data = harfbuzz.hb_blob_get_data(hb_blob, stream.length)
-            file_content = ffi.unpack(hb_data, int(stream.length[0]))
-            index = harfbuzz.hb_face_get_index(hb_face)
-            font = stream.add_font(font_hash, file_content, pango_font, index)
+        font = stream.add_font(pango_font)
 
         # Positions of the glyphs in the UTF-8 string
         utf8_positions = [offset + clusters[i] for i in range(1, num_glyphs)]
@@ -1126,14 +1135,15 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
             if string:
                 stream.show_text(string)
             string = ''
+            stream.set_font_size(font.hash, 1 if font.bitmap else font_size)
             last_font = font
-        stream.set_font_size(font.hash, 1)
         string += '<'
         for i in range(num_glyphs):
             glyph_info = glyphs[i]
             glyph = glyph_info.glyph
             width = glyph_info.geometry.width
-            if glyph == pango.PANGO_GLYPH_EMPTY:
+            if (glyph == pango.PANGO_GLYPH_EMPTY or
+                    glyph & pango.PANGO_GLYPH_UNKNOWN_FLAG):
                 string += f'>{-width / font_size}<'
                 continue
             utf8_position = utf8_positions[i]
@@ -1141,7 +1151,7 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
             offset = glyph_info.geometry.x_offset / font_size
             if offset:
                 string += f'>{-offset}<'
-            string += f'{glyph:04x}'
+            string += f'{glyph:02x}' if font.bitmap else f'{glyph:04x}'
 
             # Ink bounding box and logical widths in font
             if glyph not in font.widths:
@@ -1160,9 +1170,9 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
                     font.bbox[2] = int(units_to_double(x2 * 1000) / font_size)
                 if y2 > font.bbox[3]:
                     font.bbox[3] = int(units_to_double(y2 * 1000) / font_size)
-                font.widths[glyph] = int(
+                font.widths[glyph] = int(round(
                     units_to_double(stream.logical_rect.width * 1000) /
-                    font_size)
+                    font_size))
 
             # Kerning, word spacing, letter spacing
             kerning = int(
@@ -1175,10 +1185,12 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
             # Mapping between glyphs and characters
             if glyph not in font.cmap:
                 utf8_slice = slice(previous_utf8_position, utf8_position)
-                font.cmap[glyph] = utf8_text[utf8_slice].decode('utf-8')
+                font.cmap[glyph] = utf8_text[utf8_slice].decode()
             previous_utf8_position = utf8_position
 
             if font.svg:
+                hb_font = pango.pango_font_get_hb_font(pango_font)
+                hb_face = harfbuzz.hb_font_get_face(hb_font)
                 hb_blob = ffi.gc(
                     harfbuzz.hb_ot_color_glyph_reference_svg(hb_face, glyph),
                     harfbuzz.hb_blob_destroy)
@@ -1190,6 +1202,7 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
                     a = d = font.widths[glyph] / 1000 / font.upem * font_size
                     emojis.append([image, font, a, d, x_advance, 0])
             elif font.png:
+                hb_font = pango.pango_font_get_hb_font(pango_font)
                 hb_blob = ffi.gc(
                     harfbuzz.hb_ot_color_glyph_reference_png(hb_font, glyph),
                     harfbuzz.hb_blob_destroy)
