@@ -1,21 +1,25 @@
-"""
-    weasyprint.html
-    ---------------
+"""Specific handling for some HTML elements, especially replaced elements.
 
-    Specific handling for some HTML elements, especially replaced elements.
-
-    Replaced elements (eg. <img> elements) are rendered externally and
-    behave as an atomic opaque box in CSS. In general, they may or may not
-    have intrinsic dimensions. But the only replaced elements currently
-    supported in WeasyPrint are images with intrinsic dimensions.
+Replaced elements (eg. <img> elements) are rendered externally and behave as an
+atomic opaque box in CSS. In general, they may or may not have intrinsic
+dimensions. But the only replaced elements currently supported in WeasyPrint
+are images with intrinsic dimensions.
 
 """
 
+try:
+    # Available in Python 3.9+
+    from importlib.resources import files
+except ImportError:
+    # Deprecated in Python 3.11+
+    from importlib.resources import read_text
+else:
+    def read_text(package, resource):
+        return (files(package) / resource).read_text('utf-8')
+
 import re
-from pkgutil import get_data
-from urllib.parse import urljoin
 
-from . import CSS
+from . import CSS, css
 from .css import get_child_text
 from .css.counters import CounterStyle
 from .formatting_structure import boxes
@@ -24,17 +28,13 @@ from .logger import LOGGER
 from .urls import get_url_attribute
 
 HTML5_UA_COUNTER_STYLE = CounterStyle()
-HTML5_UA = get_data('weasyprint', 'css/html5_ua.css')
-HTML5_PH = get_data('weasyprint', 'css/html5_ph.css')
-if HTML5_UA is None:
-    LOGGER.warning('User agent stylesheet could not be loaded')
-    HTML5_UA_STYLESHEET = HTML5_PH_STYLESHEET = CSS(string='')
-else:
-    HTML5_UA_STYLESHEET = CSS(
-        string=HTML5_UA.decode('utf-8'), counter_style=HTML5_UA_COUNTER_STYLE)
-    HTML5_PH_STYLESHEET = CSS(string=HTML5_PH.decode('utf-8'))
+HTML5_UA = read_text(css, 'html5_ua.css')
+HTML5_PH = read_text(css, 'html5_ph.css')
+HTML5_UA_STYLESHEET = CSS(
+    string=HTML5_UA, counter_style=HTML5_UA_COUNTER_STYLE)
+HTML5_PH_STYLESHEET = CSS(string=HTML5_PH)
 
-# http://whatwg.org/C#space-character
+# https://html.spec.whatwg.org/multipage/#space-character
 HTML_WHITESPACE = ' \t\n\f\r'
 HTML_SPACE_SEPARATED_TOKENS_RE = re.compile(f'[^{HTML_WHITESPACE}]+')
 
@@ -46,20 +46,20 @@ def ascii_lower(string):
     :returns: A new Unicode string.
 
     This is used for `ASCII case-insensitive
-    <http://whatwg.org/C#ascii-case-insensitive>`_ matching.
+    <https://whatwg.org/C#ascii-case-insensitive>`_ matching.
 
     This is different from the :meth:`str.lower` method of Unicode strings
     which also affect non-ASCII characters,
     sometimes mapping them into the ASCII range:
 
-    >>> keyword = u'Bac\N{KELVIN SIGN}ground'
-    >>> assert keyword.lower() == u'background'
+    >>> keyword = 'Bac\N{KELVIN SIGN}ground'
+    >>> assert keyword.lower() == 'background'
     >>> assert ascii_lower(keyword) != keyword.lower()
-    >>> assert ascii_lower(keyword) == u'bac\N{KELVIN SIGN}ground'
+    >>> assert ascii_lower(keyword) == 'bac\N{KELVIN SIGN}ground'
 
     """
     # This turns out to be faster than unicode.translate()
-    return string.encode('utf8').lower().decode('utf8')
+    return string.encode().lower().decode()
 
 
 def element_has_link_type(element, link_type):
@@ -123,13 +123,14 @@ def handle_img(element, box, get_image_from_uri, base_url):
 
     Return either an image or the alt-text.
 
-    See: http://www.w3.org/TR/html5/embedded-content-1.html#the-img-element
+    See: https://www.w3.org/TR/html5/embedded-content-1.html#the-img-element
 
     """
     src = get_url_attribute(element, 'src', base_url)
     alt = element.get('alt')
     if src:
-        image = get_image_from_uri(url=src)
+        image = get_image_from_uri(
+            url=src, orientation=box.style['image_orientation'])
         if image is not None:
             return [make_replaced_box(element, box, image)]
         else:
@@ -163,7 +164,9 @@ def handle_embed(element, box, get_image_from_uri, base_url):
     src = get_url_attribute(element, 'src', base_url)
     type_ = element.get('type', '').strip()
     if src:
-        image = get_image_from_uri(url=src, forced_mime_type=type_)
+        image = get_image_from_uri(
+            url=src, forced_mime_type=type_,
+            orientation=box.style['image_orientation'])
         if image is not None:
             return [make_replaced_box(element, box, image)]
     # No fallback.
@@ -172,8 +175,7 @@ def handle_embed(element, box, get_image_from_uri, base_url):
 
 @handler('object')
 def handle_object(element, box, get_image_from_uri, base_url):
-    """Handle ``<object>`` elements, return either an image or the fallback
-    content.
+    """Handle ``<object>`` elements, return either an image or the fallback.
 
     See: https://www.w3.org/TR/html5/embedded-content-0.html#the-object-element
 
@@ -181,51 +183,33 @@ def handle_object(element, box, get_image_from_uri, base_url):
     data = get_url_attribute(element, 'data', base_url)
     type_ = element.get('type', '').strip()
     if data:
-        image = get_image_from_uri(url=data, forced_mime_type=type_)
+        image = get_image_from_uri(
+            url=data, forced_mime_type=type_,
+            orientation=box.style['image_orientation'])
         if image is not None:
             return [make_replaced_box(element, box, image)]
     # The element’s children are the fallback.
     return [box]
 
 
-def integer_attribute(element, box, name, minimum=1):
-    """Read an integer attribute from the HTML element and set it on the box.
-
-    """
-    value = element.get(name, '').strip()
-    if value:
-        try:
-            value = int(value)
-        except ValueError:
-            pass
-        else:
-            if value >= minimum:
-                setattr(box, name, value)
-
-
 @handler('colgroup')
 def handle_colgroup(element, box, _get_image_from_uri, _base_url):
     """Handle the ``span`` attribute."""
     if isinstance(box, boxes.TableColumnGroupBox):
-        if any(child.tag == 'col' for child in element):
-            box.span = None  # sum of the children’s spans
-        else:
-            integer_attribute(element, box, 'span')
-            box.children = (
+        if not any(child.tag == 'col' for child in element):
+            box.children = [
                 boxes.TableColumnBox.anonymous_from(box, [])
-                for _i in range(box.span))
+                for _ in range(box.span)]
     return [box]
 
 
 @handler('col')
 def handle_col(element, box, _get_image_from_uri, _base_url):
     """Handle the ``span`` attribute."""
-    if isinstance(box, boxes.TableColumnBox):
-        integer_attribute(element, box, 'span')
-        if box.span > 1:
-            # Generate multiple boxes
-            # http://lists.w3.org/Archives/Public/www-style/2011Nov/0293.html
-            return [box.copy() for _i in range(box.span)]
+    if isinstance(box, boxes.TableColumnBox) and box.span > 1:
+        # Generate multiple boxes
+        # https://lists.w3.org/Archives/Public/www-style/2011Nov/0293.html
+        return [box.copy() for _i in range(box.span)]
     return [box]
 
 
@@ -235,12 +219,18 @@ def handle_td(element, box, _get_image_from_uri, _base_url):
     """Handle the ``colspan``, ``rowspan`` attributes."""
     if isinstance(box, boxes.TableCellBox):
         # HTML 4.01 gives special meaning to colspan=0
-        # http://www.w3.org/TR/html401/struct/tables.html#adef-rowspan
+        # https://www.w3.org/TR/html401/struct/tables.html#adef-rowspan
         # but HTML 5 removed it
-        # http://www.w3.org/TR/html5/tabular-data.html#attr-tdth-colspan
+        # https://html.spec.whatwg.org/multipage/tables.html#attr-tdth-colspan
         # rowspan=0 is still there though.
-        integer_attribute(element, box, 'colspan')
-        integer_attribute(element, box, 'rowspan', minimum=0)
+        try:
+            box.colspan = max(int(element.get('colspan', '').strip()), 1)
+        except ValueError:
+            box.colspan = 1
+        try:
+            box.rowspan = max(int(element.get('rowspan', '').strip()), 0)
+        except ValueError:
+            box.rowspan = 1
     return [box]
 
 
@@ -248,7 +238,6 @@ def handle_td(element, box, _get_image_from_uri, _base_url):
 def handle_a(element, box, _get_image_from_uri, base_url):
     """Handle the ``rel`` attribute."""
     box.is_attachment = element_has_link_type(element, 'attachment')
-    box.download_name = element.get('download')
     return [box]
 
 
@@ -264,36 +253,22 @@ def handle_svg(element, box, get_image_from_uri, base_url):
     context = get_image_from_uri.keywords['context']
     try:
         image = SVGImage(element, base_url, url_fetcher, context)
-    except Exception as exception:
+    except Exception as exception:  # pragma: no cover
         LOGGER.error('Failed to load inline SVG: %s', exception)
         return []
     else:
         return [make_replaced_box(element, box, image)]
 
 
-def find_base_url(html_document, fallback_base_url):
-    """Return the base URL for the document.
-
-    See http://www.w3.org/TR/html5/urls.html#document-base-url
-
-    """
-    first_base_element = next(iter(html_document.iter('base')), None)
-    if first_base_element is not None:
-        href = first_base_element.get('href', '').strip()
-        if href:
-            return urljoin(fallback_base_url, href)
-    return fallback_base_url
-
-
 def get_html_metadata(html):
     """Get metadata dictionary out of HTML object.
 
     Relevant specs:
 
-    http://www.whatwg.org/html#the-title-element
-    http://www.whatwg.org/html#standard-metadata-names
-    http://wiki.whatwg.org/wiki/MetaExtensions
-    http://microformats.org/wiki/existing-rel-values#HTML5_link_type_extensions
+    https://www.whatwg.org/html#the-title-element
+    https://www.whatwg.org/html#standard-metadata-names
+    https://wiki.whatwg.org/wiki/MetaExtensions
+    https://microformats.org/wiki/existing-rel-values#HTML5_link_type_extensions
 
     """
     title = None
@@ -304,6 +279,8 @@ def get_html_metadata(html):
     created = None
     modified = None
     attachments = []
+    custom = {}
+    lang = html.etree_element.attrib.get('lang', None)
     for element in html.wrapper_element.query_all('title', 'meta', 'link'):
         element = element.etree_element
         if element.tag == 'title' and title is None:
@@ -317,14 +294,20 @@ def get_html_metadata(html):
                         keywords.append(keyword)
             elif name == 'author':
                 authors.append(content)
-            elif name == 'description' and description is None:
-                description = content
-            elif name == 'generator' and generator is None:
-                generator = content
-            elif name == 'dcterms.created' and created is None:
-                created = parse_w3c_date(name, content)
-            elif name == 'dcterms.modified' and modified is None:
-                modified = parse_w3c_date(name, content)
+            elif name == 'description':
+                if description is None:
+                    description = content
+            elif name == 'generator':
+                if generator is None:
+                    generator = content
+            elif name == 'dcterms.created':
+                if created is None:
+                    created = parse_w3c_date(name, content)
+            elif name == 'dcterms.modified':
+                if modified is None:
+                    modified = parse_w3c_date(name, content)
+            elif name and name not in custom:
+                custom[name] = content
         elif element.tag == 'link' and element_has_link_type(
                 element, 'attachment'):
             url = get_url_attribute(element, 'href', html.base_url)
@@ -336,15 +319,15 @@ def get_html_metadata(html):
     return dict(title=title, description=description, generator=generator,
                 keywords=keywords, authors=authors,
                 created=created, modified=modified,
-                attachments=attachments)
+                attachments=attachments, lang=lang, custom=custom)
 
 
 def strip_whitespace(string):
     """Use the HTML definition of "space character",
     not all Unicode Whitespace.
 
-    http://www.whatwg.org/html#strip-leading-and-trailing-whitespace
-    http://www.whatwg.org/html#space-character
+    https://www.whatwg.org/html#strip-leading-and-trailing-whitespace
+    https://www.whatwg.org/html#space-character
 
     """
     return string.strip(HTML_WHITESPACE)
@@ -386,7 +369,11 @@ W3C_DATE_RE = re.compile('''
 
 
 def parse_w3c_date(meta_name, string):
-    """http://www.w3.org/TR/NOTE-datetime"""
+    """Parse datetimes as defined by the W3C.
+
+    See https://www.w3.org/TR/NOTE-datetime
+
+    """
     if W3C_DATE_RE.match(string):
         return string
     else:
