@@ -1,10 +1,6 @@
-"""
-    weasyprint.layout.block
-    -----------------------
+"""Page breaking and layout for block-level and block-container boxes."""
 
-    Page breaking and layout for block-level and block-container boxes.
-
-"""
+from math import inf
 
 from ..formatting_structure import boxes
 from .absolute import AbsolutePlaceholder, absolute_layout
@@ -19,9 +15,14 @@ from .table import table_layout, table_wrapper_width
 
 
 def block_level_layout(context, box, bottom_space, skip_stack,
-                       containing_block, page_is_empty, absolute_boxes,
-                       fixed_boxes, adjoining_margins, discard):
+                       containing_block, page_is_empty=True,
+                       absolute_boxes=None, fixed_boxes=None,
+                       adjoining_margins=None, discard=False, max_lines=None):
     """Lay out the block-level ``box``."""
+    absolute_boxes = [] if absolute_boxes is None else absolute_boxes
+    fixed_boxes = [] if fixed_boxes is None else fixed_boxes
+    adjoining_margins = [] if adjoining_margins is None else adjoining_margins
+
     if not isinstance(box, boxes.TableBox):
         resolve_percentages(box, containing_block)
 
@@ -31,6 +32,8 @@ def block_level_layout(context, box, bottom_space, skip_stack,
             box.margin_bottom = 0
 
         if context.current_page > 1 and page_is_empty:
+            # When an unforced break occurs before or after a block-level box,
+            # any margins adjoining the break are truncated to zero.
             # TODO: this condition is wrong, it only works for blocks whose
             # parent breaks collapsing margins. It should work for blocks whose
             # one of the ancestors breaks collapsing margins.
@@ -55,35 +58,38 @@ def block_level_layout(context, box, bottom_space, skip_stack,
 
     return block_level_layout_switch(
         context, box, bottom_space, skip_stack, containing_block,
-        page_is_empty, absolute_boxes, fixed_boxes, adjoining_margins, discard)
+        page_is_empty, absolute_boxes, fixed_boxes, adjoining_margins, discard,
+        max_lines)
 
 
 def block_level_layout_switch(context, box, bottom_space, skip_stack,
                               containing_block, page_is_empty, absolute_boxes,
-                              fixed_boxes, adjoining_margins, discard):
+                              fixed_boxes, adjoining_margins, discard,
+                              max_lines):
     """Call the layout function corresponding to the ``box`` type."""
     if isinstance(box, boxes.TableBox):
-        return table_layout(
+        result = table_layout(
             context, box, bottom_space, skip_stack, containing_block,
             page_is_empty, absolute_boxes, fixed_boxes)
     elif isinstance(box, boxes.BlockBox):
         return block_box_layout(
             context, box, bottom_space, skip_stack, containing_block,
             page_is_empty, absolute_boxes, fixed_boxes, adjoining_margins,
-            discard)
+            discard, max_lines)
     elif isinstance(box, boxes.BlockReplacedBox):
-        return block_replaced_box_layout(context, box, containing_block)
+        result = block_replaced_box_layout(context, box, containing_block)
     elif isinstance(box, boxes.FlexBox):
-        return flex_layout(
+        result = flex_layout(
             context, box, bottom_space, skip_stack, containing_block,
             page_is_empty, absolute_boxes, fixed_boxes)
     else:  # pragma: no cover
         raise TypeError(f'Layout for {type(box).__name__} not handled yet')
+    return result + (None,)
 
 
 def block_box_layout(context, box, bottom_space, skip_stack,
                      containing_block, page_is_empty, absolute_boxes,
-                     fixed_boxes, adjoining_margins, discard):
+                     fixed_boxes, adjoining_margins, discard, max_lines):
     """Lay out the block ``box``."""
     if (box.style['column_width'] != 'auto' or
             box.style['column_count'] != 'auto'):
@@ -98,12 +104,14 @@ def block_box_layout(context, box, bottom_space, skip_stack,
                 new_box.margin_bottom + new_box.padding_bottom +
                 new_box.border_bottom_width)
             if columns_bottom_space:
+                remove_placeholders(
+                    context, [new_box], absolute_boxes, fixed_boxes)
                 bottom_space += columns_bottom_space
                 result = columns_layout(
                     context, box, bottom_space, skip_stack,
                     containing_block, page_is_empty, absolute_boxes,
                     fixed_boxes, adjoining_margins)
-        return result
+        return result + (None,)
     elif box.is_table_wrapper:
         table_wrapper_width(
             context, box, (containing_block.width, containing_block.height))
@@ -111,11 +119,11 @@ def block_box_layout(context, box, bottom_space, skip_stack,
 
     result = block_container_layout(
         context, box, bottom_space, skip_stack, page_is_empty,
-        absolute_boxes, fixed_boxes, adjoining_margins, discard)
+        absolute_boxes, fixed_boxes, adjoining_margins, discard, max_lines)
     new_box = result[0]
     if new_box and new_box.is_table_wrapper:
         # Don't collide with floats
-        # http://www.w3.org/TR/CSS21/visuren.html#floats
+        # https://www.w3.org/TR/CSS21/visuren.html#floats
         position_x, position_y, _ = avoid_collisions(
             context, new_box, containing_block, outer=False)
         new_box.translate(
@@ -135,7 +143,7 @@ def block_level_width(box, containing_block):
         # TODO: what is the real text direction?
         direction = 'ltr'
 
-    # http://www.w3.org/TR/CSS21/visudet.html#blockwidth
+    # https://www.w3.org/TR/CSS21/visudet.html#blockwidth
 
     # These names are waaay too long
     margin_l = box.margin_left
@@ -177,9 +185,9 @@ def block_level_width(box, containing_block):
         width = box.width = cb_width - (
             paddings_plus_borders + margin_l + margin_r)
     margin_sum = cb_width - paddings_plus_borders - width
-    if margin_l == 'auto' and margin_r == 'auto':
-        box.margin_left = margin_sum / 2.
-        box.margin_right = margin_sum / 2.
+    if margin_l == margin_r == 'auto':
+        box.margin_left = margin_sum / 2
+        box.margin_right = margin_sum / 2
     elif margin_l == 'auto' and margin_r != 'auto':
         box.margin_left = margin_sum - margin_r
     elif margin_l != 'auto' and margin_r == 'auto':
@@ -222,11 +230,12 @@ def _out_of_flow_layout(context, box, index, child, new_children,
                         adjoining_margins, bottom_space):
     stop = False
     resume_at = None
+    new_child = None
     out_of_flow_resume_at = None
 
     child.position_y += collapse_margin(adjoining_margins)
     if child.is_absolutely_positioned():
-        placeholder = AbsolutePlaceholder(child)
+        new_child = placeholder = AbsolutePlaceholder(child)
         placeholder.index = index
         new_children.append(placeholder)
         if child.style['position'] == 'absolute':
@@ -247,9 +256,9 @@ def _out_of_flow_layout(context, box, index, child, new_children,
             last_in_flow_child = find_last_in_flow_child(new_children)
             page_break = block_level_page_break(last_in_flow_child, child)
             resume_at = {index: None}
-            if new_children and page_break in ('avoid', 'avoid-page'):
+            if new_children and avoid_page_break(page_break, context):
                 result = find_earlier_page_break(
-                    new_children, absolute_boxes, fixed_boxes)
+                    context, new_children, absolute_boxes, fixed_boxes)
                 if result:
                     new_children, resume_at = result
             stop = True
@@ -258,11 +267,12 @@ def _out_of_flow_layout(context, box, index, child, new_children,
         page = context.current_page
         context.running_elements[running_name][page].append(child)
 
-    return stop, resume_at, out_of_flow_resume_at
+    return stop, resume_at, new_child, out_of_flow_resume_at
 
 
-def _break_line(box, new_children, lines_iterator, page_is_empty, index,
-                skip_stack, resume_at):
+def _break_line(context, box, line, new_children, lines_iterator,
+                page_is_empty, index, skip_stack, resume_at, absolute_boxes,
+                fixed_boxes):
     over_orphans = len(new_children) - box.style['orphans']
     if over_orphans < 0 and not page_is_empty:
         # Reached the bottom of the page before we had
@@ -281,17 +291,22 @@ def _break_line(box, new_children, lines_iterator, page_is_empty, index,
         return True, False, resume_at
     if needed and needed <= over_orphans:
         # Remove lines to keep them for the next page
+        for child in new_children[-needed:]:
+            remove_placeholders(
+                context, child.children, absolute_boxes, fixed_boxes)
         del new_children[-needed:]
     # Page break here, resume before this line
+    remove_placeholders(context, line.children, absolute_boxes, fixed_boxes)
     return False, True, {index: skip_stack}
 
 
 def _linebox_layout(context, box, index, child, new_children, page_is_empty,
                     absolute_boxes, fixed_boxes, adjoining_margins,
                     bottom_space, position_y, skip_stack, first_letter_style,
-                    draw_bottom_decoration):
+                    draw_bottom_decoration, max_lines):
     abort = stop = False
     resume_at = None
+    new_footnotes = []
 
     assert len(box.children) == 1, 'line box with siblings before layout'
 
@@ -302,6 +317,14 @@ def _linebox_layout(context, box, index, child, new_children, page_is_empty,
         context, child, position_y, bottom_space, skip_stack,
         new_containing_block, absolute_boxes, fixed_boxes, first_letter_style)
     for i, (line, resume_at) in enumerate(lines_iterator):
+        # Break box if we reached max-lines
+        if max_lines is not None:
+            if max_lines == 0:
+                new_children[-1].block_ellipsis = box.style['block_ellipsis']
+                break
+            max_lines -= 1
+
+        # Update line resume_at and position_y
         line.resume_at = resume_at
         new_position_y = line.position_y + line.height
 
@@ -321,12 +344,13 @@ def _linebox_layout(context, box, index, child, new_children, page_is_empty,
             context.overflows_page(bottom_space, new_position_y + offset_y))
         if overflow:
             abort, stop, resume_at = _break_line(
-                box, new_children, lines_iterator, page_is_empty, index,
-                skip_stack, resume_at)
+                context, box, line, new_children, lines_iterator,
+                page_is_empty, index, skip_stack, resume_at, absolute_boxes,
+                fixed_boxes)
             break
 
         # TODO: this is incomplete.
-        # See http://dev.w3.org/csswg/css3-page/#allowed-pg-brk
+        # See https://drafts.csswg.org/css-page-3/#allowed-pg-brk
         # "When an unforced page break occurs here, both the adjoining
         #  ‘margin-top’ and ‘margin-bottom’ are set to zero."
         # See https://github.com/Kozea/WeasyPrint/issues/115
@@ -344,21 +368,31 @@ def _linebox_layout(context, box, index, child, new_children, page_is_empty,
                 descendant.footnote for descendant in line.descendants()
                 if descendant.footnote in context.footnotes)
             for footnote in footnotes:
-                context.layout_footnote(footnote)
+                overflow = context.layout_footnote(footnote)
+                new_footnotes.append(footnote)
                 overflow = (
+                    overflow or
                     context.reported_footnotes or
                     context.overflows_page(
                         bottom_space, new_position_y + offset_y))
                 if overflow:
                     context.report_footnote(footnote)
-                    if footnote.style['footnote_policy'] == 'line':
-                        abort, stop, resume_at = _break_line(
-                            box, new_children, lines_iterator, page_is_empty,
-                            index, skip_stack, resume_at)
-                        break_linebox = True
-                    elif footnote.style['footnote_policy'] == 'block':
-                        abort = break_linebox = True
-                    break
+                    # If we've put other content on this page, then we may want
+                    # to push this line or block to the next page. Otherwise,
+                    # we can't (and would loop forever if we tried), so don't
+                    # even try.
+                    if new_children or not page_is_empty:
+                        if footnote.style['footnote_policy'] == 'line':
+                            abort, stop, resume_at = _break_line(
+                                context, box, line, new_children,
+                                lines_iterator, page_is_empty, index,
+                                skip_stack, resume_at, absolute_boxes,
+                                fixed_boxes)
+                            break_linebox = True
+                            break
+                        elif footnote.style['footnote_policy'] == 'block':
+                            abort = break_linebox = True
+                            break
             if break_linebox:
                 break
 
@@ -366,23 +400,17 @@ def _linebox_layout(context, box, index, child, new_children, page_is_empty,
         position_y = new_position_y
         skip_stack = resume_at
 
-        # Break box if we reached max-lines
-        if box.style['max_lines'] != 'none':
-            if i >= box.style['max_lines'] - 1:
-                line.block_ellipsis = box.style['block_ellipsis']
-                break
-
     if new_children:
         resume_at = {index: new_children[-1].resume_at}
 
-    return abort, stop, resume_at, position_y
+    return abort, stop, resume_at, position_y, new_footnotes, max_lines
 
 
 def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
                     absolute_boxes, fixed_boxes, adjoining_margins,
                     bottom_space, position_y, skip_stack, first_letter_style,
                     draw_bottom_decoration, collapsing_with_children, discard,
-                    next_page):
+                    next_page, max_lines):
     abort = stop = False
 
     last_in_flow_child = find_last_in_flow_child(new_children)
@@ -390,15 +418,14 @@ def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
         # Between in-flow siblings
         page_break = block_level_page_break(last_in_flow_child, child)
         page_name = block_level_page_name(last_in_flow_child, child)
-        if page_name or page_break in (
-                'page', 'left', 'right', 'recto', 'verso'):
+        if page_name or force_page_break(page_break, context):
             page_name = child.page_values()[0]
             next_page = {'break': page_break, 'page': page_name}
             resume_at = {index: None}
             stop = True
             return (
                 abort, stop, resume_at, position_y, adjoining_margins,
-                next_page, new_children)
+                next_page, new_children, max_lines)
     else:
         page_break = 'auto'
 
@@ -406,15 +433,21 @@ def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
 
     if not new_containing_block.is_table_wrapper:
         resolve_percentages(child, new_containing_block)
-        if (child.is_in_normal_flow() and last_in_flow_child is None and
-                collapsing_with_children):
+        if last_in_flow_child is None and collapsing_with_children:
             # TODO: add the adjoining descendants' margin top to
             # [child.margin_top]
             old_collapsed_margin = collapse_margin(adjoining_margins)
-            if child.margin_top == 'auto':
+            # TODO: the margin-top value is set afterwards in
+            # block_level_layout, we shouldn’t duplicate this code
+            child_margin_top = child.margin_top
+            if child_margin_top == 'auto':
                 child_margin_top = 0
-            else:
-                child_margin_top = child.margin_top
+            elif context.current_page > 1 and page_is_empty:
+                if box.style['margin_break'] == 'discard':
+                    child_margin_top = 0
+                elif box.style['margin_break'] == 'auto':
+                    if not context.forced_break:
+                        child_margin_top = 0
             new_collapsed_margin = collapse_margin(
                 adjoining_margins + [child_margin_top])
             collapsed_margin_difference = (
@@ -446,10 +479,10 @@ def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
     if not getattr(child, 'first_letter_style', None):
         child.first_letter_style = first_letter_style
     (new_child, resume_at, next_page, next_adjoining_margins,
-     collapsing_through) = block_level_layout(
+     collapsing_through, max_lines) = block_level_layout(
          context, child, bottom_space, skip_stack,
          new_containing_block, page_is_empty_with_no_children, absolute_boxes,
-         fixed_boxes, adjoining_margins, discard)
+         fixed_boxes, adjoining_margins, discard, max_lines)
 
     if new_child is not None:
         # We need to do this after the child layout to have the
@@ -471,20 +504,23 @@ def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
             if page_overflow and not page_is_empty_with_no_children:
                 # The child content overflows the page area, display it on the
                 # next page.
-                remove_placeholders([new_child], absolute_boxes, fixed_boxes)
+                remove_placeholders(
+                    context, [new_child], absolute_boxes, fixed_boxes)
                 new_child = None
             elif not page_is_empty_with_no_children and context.overflows_page(
                     bottom_space, new_position_y):
                 # The child border/padding overflows the page area, do the
                 # layout again with a higher bottom_space value.
-                remove_placeholders([new_child], absolute_boxes, fixed_boxes)
+                remove_placeholders(
+                    context, [new_child], absolute_boxes, fixed_boxes)
                 bottom_space += (
                     new_child.padding_bottom + new_child.border_bottom_width)
                 (new_child, resume_at, next_page, next_adjoining_margins,
-                 collapsing_through) = block_level_layout(
+                 collapsing_through, max_lines) = block_level_layout(
                      context, child, bottom_space, skip_stack,
                      new_containing_block, page_is_empty_with_no_children,
-                     absolute_boxes, fixed_boxes, adjoining_margins, discard)
+                     absolute_boxes, fixed_boxes, adjoining_margins, discard,
+                     max_lines)
                 if new_child:
                     position_y = (
                         new_child.border_box_y() + new_child.border_height())
@@ -502,16 +538,16 @@ def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
 
     if new_child is None:
         # Nothing fits in the remaining space of this page: break
-        if page_break in ('avoid', 'avoid-page'):
+        if avoid_page_break(page_break, context):
             # TODO: fill the blank space at the bottom of the page
             result = find_earlier_page_break(
-                new_children, absolute_boxes, fixed_boxes)
+                context, new_children, absolute_boxes, fixed_boxes)
             if result:
                 new_children, resume_at = result
                 stop = True
                 return (
                     abort, stop, resume_at, position_y, adjoining_margins,
-                    next_page, new_children)
+                    next_page, new_children, max_lines)
             else:
                 # We did not find any page break opportunity
                 if not page_is_empty:
@@ -521,7 +557,7 @@ def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
                     abort = True
                     return (
                         abort, stop, resume_at, position_y, adjoining_margins,
-                        next_page, new_children)
+                        next_page, new_children, max_lines)
                 # else:
                 # ignore this 'avoid' and break anyway.
 
@@ -529,7 +565,8 @@ def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
             # This box has only rendered absolute children, keep them
             # for the next page. This is for example useful for list
             # markers.
-            remove_placeholders(new_children, absolute_boxes, fixed_boxes)
+            remove_placeholders(
+                context, new_children, absolute_boxes, fixed_boxes)
             new_children = []
 
         if new_children:
@@ -540,7 +577,7 @@ def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
             abort = True
         return (
             abort, stop, resume_at, position_y, adjoining_margins, next_page,
-            new_children)
+            new_children, max_lines)
 
     # index in its non-laid-out parent, not in future new parent
     # May be used in find_earlier_page_break()
@@ -552,12 +589,12 @@ def _in_flow_layout(context, box, index, child, new_children, page_is_empty,
 
     return (
         abort, stop, resume_at, position_y, adjoining_margins, next_page,
-        new_children)
+        new_children, max_lines)
 
 
 def block_container_layout(context, box, bottom_space, skip_stack,
                            page_is_empty, absolute_boxes, fixed_boxes,
-                           adjoining_margins, discard):
+                           adjoining_margins, discard, max_lines):
     """Set the ``box`` height."""
     # TODO: boxes.FlexBox is allowed here because flex_layout calls
     # block_container_layout, there's probably a better solution.
@@ -603,53 +640,85 @@ def block_container_layout(context, box, bottom_space, skip_stack,
 
     new_children = []
     next_page = {'break': 'any', 'page': None}
+    all_footnotes = []
+    broken_out_of_flow = {}
 
     last_in_flow_child = None
 
+    if box.style['max_lines'] != 'none':
+        max_lines = min(box.style['max_lines'], max_lines or inf)
+
     if is_start:
         skip = 0
         first_letter_style = getattr(box, 'first_letter_style', None)
     else:
-        # TODO: handle multiple skip stacks
         (skip, skip_stack), = skip_stack.items()
         first_letter_style = None
     for index, child in enumerate(box.children[skip:], start=(skip or 0)):
         child.position_x = position_x
         child.position_y = position_y  # doesn’t count adjoining_margins
+        new_footnotes = []
 
         if not child.is_in_normal_flow():
             abort = False
-            stop, resume_at, out_of_flow_resume_at = _out_of_flow_layout(
-                context, box, index, child, new_children, page_is_empty,
-                absolute_boxes, fixed_boxes, adjoining_margins,
-                bottom_space)
+            stop, resume_at, new_child, out_of_flow_resume_at = (
+                _out_of_flow_layout(
+                    context, box, index, child, new_children, page_is_empty,
+                    absolute_boxes, fixed_boxes, adjoining_margins,
+                    bottom_space))
             if out_of_flow_resume_at:
-                context.broken_out_of_flow.append(
-                    (child, box, out_of_flow_resume_at))
+                broken_out_of_flow[new_child] = (
+                    child, box, out_of_flow_resume_at)
 
         elif isinstance(child, boxes.LineBox):
-            abort, stop, resume_at, position_y = _linebox_layout(
+            (abort, stop, resume_at, position_y,
+             new_footnotes, max_lines) = _linebox_layout(
                 context, box, index, child, new_children, page_is_empty,
                 absolute_boxes, fixed_boxes, adjoining_margins, bottom_space,
                 position_y, skip_stack, first_letter_style,
-                draw_bottom_decoration)
+                draw_bottom_decoration, max_lines)
             draw_bottom_decoration |= resume_at is None
             adjoining_margins = []
+            all_footnotes += new_footnotes
 
         else:
             (abort, stop, resume_at, position_y, adjoining_margins,
-             next_page, new_children) = _in_flow_layout(
+             next_page, new_children, new_max_lines) = _in_flow_layout(
                  context, box, index, child, new_children, page_is_empty,
                  absolute_boxes, fixed_boxes, adjoining_margins, bottom_space,
                  position_y, skip_stack, first_letter_style,
                  draw_bottom_decoration, collapsing_with_children, discard,
-                 next_page)
+                 next_page, max_lines)
             skip_stack = None
 
+            if None not in (new_max_lines, max_lines):
+                max_lines = new_max_lines
+                if max_lines <= 0:
+                    stop = True
+                    last_child = (child == box.children[-1])
+                    if not last_child:
+                        children = new_children
+                        while children:
+                            last_child = children[-1]
+                            if isinstance(last_child, boxes.LineBox):
+                                last_child.block_ellipsis = (
+                                    box.style['block_ellipsis'])
+                            elif isinstance(last_child, boxes.ParentBox):
+                                children = last_child.children
+                                continue
+                            break
+
         if abort:
             page = child.page_values()[0]
-            return None, None, {'break': 'any', 'page': page}, [], False
+            remove_placeholders(
+                context, box.children[skip:], absolute_boxes, fixed_boxes)
+            for footnote in new_footnotes:
+                context.unlayout_footnote(footnote)
+            return (
+                None, None, {'break': 'any', 'page': page}, [], False,
+                max_lines)
         elif stop:
+            adjoining_margins = []
             break
 
     else:
@@ -660,10 +729,15 @@ def block_container_layout(context, box, bottom_space, skip_stack,
         resume_at = None
 
     if (box_is_fragmented and
-            box.style['break_inside'] in ('avoid', 'avoid-page') and
+            avoid_page_break(box.style['break_inside'], context) and
             not page_is_empty):
+        for footnote in all_footnotes:
+            context.unlayout_footnote(footnote)
         return (
-            None, None, {'break': 'any', 'page': None}, [], False)
+            None, None, {'break': 'any', 'page': None}, [], False, max_lines)
+
+    for key, value in broken_out_of_flow.items():
+        context.broken_out_of_flow[key] = value
 
     if collapsing_with_children:
         box.position_y += (
@@ -676,9 +750,9 @@ def block_container_layout(context, box, bottom_space, skip_stack,
         # Top and bottom margins of this box
         if (box.height in ('auto', 0) and
             get_clearance(context, box, collapsed_margin) is None and
-            all(value == 0 for value in [
+            all(value == 0 for value in (
                 box.min_height, box.border_top_width, box.padding_top,
-                box.border_bottom_width, box.padding_bottom])):
+                box.border_bottom_width, box.padding_bottom))):
             collapsing_through = True
         else:
             position_y += collapsed_margin
@@ -708,7 +782,7 @@ def block_container_layout(context, box, bottom_space, skip_stack,
         start=not is_start, end=box_is_fragmented and not discard)
 
     # TODO: See corner cases in
-    # http://www.w3.org/TR/CSS21/visudet.html#normal-block
+    # https://www.w3.org/TR/CSS21/visudet.html#normal-block
     # TODO: See float.float_layout
     if new_box.height == 'auto':
         if context.excluded_shapes and new_box.style['overflow'] != 'visible':
@@ -736,21 +810,25 @@ def block_container_layout(context, box, bottom_space, skip_stack,
         # new_box.height
         new_box.height = max(
             min(new_box.height, new_box.max_height), new_box.min_height)
-    elif bottom_space > -float('inf'):
+    elif bottom_space > -inf and not new_box.is_column:
         # Make the box fill the blank space at the bottom of the page
         # https://www.w3.org/TR/css-break-3/#box-splitting
-        new_box.height = (
+        new_box_height = (
             context.page_bottom - bottom_space - new_box.position_y -
             (new_box.margin_height() - new_box.height))
-        if draw_bottom_decoration:
-            new_box.height += (
-                box.padding_bottom + box.border_bottom_width +
-                box.margin_bottom)
+        if new_box_height > new_box.height:
+            new_box.height = new_box_height
+            if draw_bottom_decoration:
+                new_box.height += (
+                    box.padding_bottom + box.border_bottom_width +
+                    box.margin_bottom)
 
     if next_page['page'] is None:
         next_page['page'] = new_box.page_values()[1]
 
-    return new_box, resume_at, next_page, adjoining_margins, collapsing_through
+    return (
+        new_box, resume_at, next_page, adjoining_margins, collapsing_through,
+        max_lines)
 
 
 def collapse_margin(adjoining_margins):
@@ -765,7 +843,7 @@ def collapse_margin(adjoining_margins):
 def establishes_formatting_context(box):
     """Return whether a box establishes a block formatting context.
 
-    See http://www.w3.org/TR/CSS2/visuren.html#block-formatting
+    See https://www.w3.org/TR/CSS2/visuren.html#block-formatting
 
     """
     return (
@@ -800,7 +878,7 @@ def block_level_page_break(sibling_before, sibling_after):
     * 'left' or 'right' take priority over 'always', 'avoid' or 'auto'
     * Among 'left' and 'right', later values in the tree take priority.
 
-    See http://dev.w3.org/csswg/css3-page/#allowed-pg-brk
+    See https://drafts.csswg.org/css-page-3/#allowed-pg-brk
 
     """
     values = []
@@ -828,9 +906,15 @@ def block_level_page_break(sibling_before, sibling_after):
         if value in ('left', 'right', 'recto', 'verso') or (value, result) in (
                 ('page', 'auto'),
                 ('page', 'avoid'),
-                ('avoid', 'auto'),
                 ('page', 'avoid-page'),
-                ('avoid-page', 'auto')):
+                ('page', 'avoid-column'),
+                ('column', 'auto'),
+                ('column', 'avoid'),
+                ('column', 'avoid-page'),
+                ('column', 'avoid-column'),
+                ('avoid', 'auto'),
+                ('avoid-page', 'auto'),
+                ('avoid-column', 'auto')):
             result = value
 
     return result
@@ -844,7 +928,7 @@ def block_level_page_name(sibling_before, sibling_after):
         return after_page
 
 
-def find_earlier_page_break(children, absolute_boxes, fixed_boxes):
+def find_earlier_page_break(context, children, absolute_boxes, fixed_boxes):
     """Find the last possible page break in ``children``.
 
     Because of a `page-break-before: avoid` or a `page-break-after: avoid` we
@@ -866,7 +950,8 @@ def find_earlier_page_break(children, absolute_boxes, fixed_boxes):
             return None
         new_children = children[:index]
         resume_at = {0: new_children[-1].resume_at}
-        remove_placeholders(children[index:], absolute_boxes, fixed_boxes)
+        remove_placeholders(
+            context, children[index:], absolute_boxes, fixed_boxes)
         return new_children, resume_at
 
     previous_in_flow = None
@@ -880,9 +965,9 @@ def find_earlier_page_break(children, absolute_boxes, fixed_boxes):
             continue
 
         if child.is_in_normal_flow():
+            page_break = block_level_page_break(child, previous_in_flow)
             if previous_in_flow is not None and (
-                    block_level_page_break(child, previous_in_flow) not in
-                    ('avoid', 'avoid-page')):
+                    not avoid_page_break(page_break, context)):
                 index += 1  # break after child
                 new_children = children[:index]
                 # Get the index in the original parent
@@ -891,12 +976,12 @@ def find_earlier_page_break(children, absolute_boxes, fixed_boxes):
             previous_in_flow = child
 
         if child.is_in_normal_flow() and (
-                child.style['break_inside'] not in ('avoid', 'avoid-page')):
+                not avoid_page_break(child.style['break_inside'], context)):
             breakable_box_types = (
                 boxes.BlockBox, boxes.TableBox, boxes.TableRowGroupBox)
             if isinstance(child, breakable_box_types):
                 result = find_earlier_page_break(
-                    child.children, absolute_boxes, fixed_boxes)
+                    context, child.children, absolute_boxes, fixed_boxes)
                 if result:
                     new_grand_children, resume_at = result
                     new_child = child.copy_with_children(new_grand_children)
@@ -917,7 +1002,7 @@ def find_earlier_page_break(children, absolute_boxes, fixed_boxes):
         return None
 
     # TODO: don’t remove absolute and fixed placeholders found in table footers
-    remove_placeholders(children[index:], absolute_boxes, fixed_boxes)
+    remove_placeholders(context, children[index:], absolute_boxes, fixed_boxes)
     return new_children, resume_at
 
 
@@ -933,18 +1018,40 @@ def reversed_enumerate(seq):
     return zip(reversed(range(len(seq))), reversed(seq))
 
 
-def remove_placeholders(box_list, absolute_boxes, fixed_boxes):
+def remove_placeholders(context, box_list, absolute_boxes, fixed_boxes):
     """Remove placeholders from absolute and fixed lists.
 
     For boxes that have been removed in find_earlier_page_break(), remove the
     matching placeholders in absolute_boxes and fixed_boxes.
 
+    Also takes care of removed footnotes and floats.
+
     """
     for box in box_list:
         if isinstance(box, boxes.ParentBox):
-            remove_placeholders(box.children, absolute_boxes, fixed_boxes)
+            remove_placeholders(
+                context, box.children, absolute_boxes, fixed_boxes)
         if box.style['position'] == 'absolute' and box in absolute_boxes:
             # box is not in absolute_boxes if its parent has position: relative
             absolute_boxes.remove(box)
         elif box.style['position'] == 'fixed':
             fixed_boxes.remove(box)
+        if box.footnote:
+            context.unlayout_footnote(box.footnote)
+        if box in context.broken_out_of_flow:
+            context.broken_out_of_flow.pop(box)
+
+
+def avoid_page_break(page_break, context):
+    """Test whether we should avoid breaks."""
+    if context.in_column:
+        return page_break in ('avoid', 'avoid-page', 'avoid-column')
+    return page_break in ('avoid', 'avoid-page')
+
+
+def force_page_break(page_break, context):
+    """Test whether we should force breaks."""
+    if context.in_column:
+        return page_break in (
+            'page', 'left', 'right', 'recto', 'verso', 'column')
+    return page_break in ('page', 'left', 'right', 'recto', 'verso')
