@@ -1,13 +1,7 @@
-"""
-    weasyprint.layout.flex
-    ------------------------
-
-    Layout for flex containers and flex-items.
-
-"""
+"""Layout for flex containers and flex-items."""
 
 import sys
-from math import log10
+from math import inf, log10
 
 from ..css.properties import Dimension
 from ..formatting_structure import boxes
@@ -107,7 +101,6 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
             context, parent_box)
     original_skip_stack = skip_stack
     if skip_stack is not None:
-        # TODO: handle multiple skip stacks
         (skip, skip_stack), = skip_stack.items()
         if box.style['flex_direction'].endswith('-reverse'):
             children = children[:skip + 1]
@@ -130,8 +123,7 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
         child.position_x = parent_box.content_box_x()
         child.position_y = parent_box.content_box_y()
         if child.min_width == 'auto':
-            specified_size = (
-                child.width if child.width != 'auto' else float('inf'))
+            specified_size = child.width if child.width != 'auto' else inf
             if isinstance(child, boxes.ParentBox):
                 new_child = child.copy_with_children(child.children)
             else:
@@ -139,13 +131,12 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
             new_child.style = child.style.copy()
             new_child.style['width'] = 'auto'
             new_child.style['min_width'] = Dimension(0, 'px')
-            new_child.style['max_width'] = Dimension(float('inf'), 'px')
+            new_child.style['max_width'] = Dimension(inf, 'px')
             content_size = min_content_width(context, new_child, outer=False)
             child.min_width = min(specified_size, content_size)
         elif child.min_height == 'auto':
             # TODO: find a way to get min-content-height
-            specified_size = (
-                child.height if child.height != 'auto' else float('inf'))
+            specified_size = child.height if child.height != 'auto' else inf
             if isinstance(child, boxes.ParentBox):
                 new_child = child.copy_with_children(child.children)
             else:
@@ -153,10 +144,10 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
             new_child.style = child.style.copy()
             new_child.style['height'] = 'auto'
             new_child.style['min_height'] = Dimension(0, 'px')
-            new_child.style['max_height'] = Dimension(float('inf'), 'px')
+            new_child.style['max_height'] = Dimension(inf, 'px')
             new_child = block.block_level_layout(
-                context, new_child, -float('inf'), child_skip_stack,
-                parent_box, page_is_empty, [], [], [], False)[0]
+                context, new_child, -inf, child_skip_stack, parent_box,
+                page_is_empty)[0]
             content_size = new_child.height
             child.min_height = min(specified_size, content_size)
 
@@ -212,11 +203,10 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
                         new_child = child.copy_with_children(child.children)
                     else:
                         new_child = child.copy()
-                    new_child.width = float('inf')
+                    new_child.width = inf
                     new_child = block.block_level_layout(
-                        context, new_child, -float('inf'), child_skip_stack,
-                        parent_box, page_is_empty, absolute_boxes, fixed_boxes,
-                        adjoining_margins=[], discard=False)[0]
+                        context, new_child, -inf, child_skip_stack, parent_box,
+                        page_is_empty, absolute_boxes, fixed_boxes)[0]
                     child.flex_base_size = new_child.margin_height()
             elif child.style[axis] == 'min-content':
                 child.style[axis] = 'auto'
@@ -229,9 +219,8 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
                         new_child = child.copy()
                     new_child.width = 0
                     new_child = block.block_level_layout(
-                        context, new_child, -float('inf'), child_skip_stack,
-                        parent_box, page_is_empty, absolute_boxes, fixed_boxes,
-                        adjoining_margins=[], discard=False)[0]
+                        context, new_child, -inf, child_skip_stack, parent_box,
+                        page_is_empty, absolute_boxes, fixed_boxes)[0]
                     child.flex_base_size = new_child.margin_height()
             else:
                 assert child.style[axis].unit == 'px'
@@ -352,17 +341,17 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
             if unfrozen_factor_sum < 1:
                 initial_free_space *= unfrozen_factor_sum
 
-            if initial_free_space == float('inf'):
+            if initial_free_space == inf:
                 initial_free_space = sys.maxsize
-            if remaining_free_space == float('inf'):
+            if remaining_free_space == inf:
                 remaining_free_space = sys.maxsize
 
             initial_magnitude = (
                 int(log10(initial_free_space)) if initial_free_space > 0
-                else -float('inf'))
+                else -inf)
             remaining_magnitude = (
                 int(log10(remaining_free_space)) if remaining_free_space > 0
-                else -float('inf'))
+                else -inf)
             if initial_magnitude < remaining_magnitude:
                 remaining_free_space = initial_free_space
 
@@ -461,11 +450,11 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
             else:
                 child_copy = child.copy()
             block.block_level_width(child_copy, parent_box)
-            new_child, _, _, adjoining_margins, _ = (
+            new_child, _, _, adjoining_margins, _, _ = (
                 block.block_level_layout_switch(
-                    context, child_copy, -float('inf'), child_skip_stack,
-                    parent_box, page_is_empty, absolute_boxes, fixed_boxes,
-                    adjoining_margins=[], discard=False))
+                    context, child_copy, -inf, child_skip_stack, parent_box,
+                    page_is_empty, absolute_boxes, fixed_boxes,
+                    [], False, None))
 
             child._baseline = find_in_flow_baseline(new_child) or 0
             if cross == 'height':
@@ -513,7 +502,7 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
             collected_cross_size = cross_start_distance + cross_end_distance
             non_collected_cross_size = 0
             if not_collected_items:
-                non_collected_cross_size = float('-inf')
+                non_collected_cross_size = -inf
                 for child in not_collected_items:
                     if cross == 'height':
                         child_cross_size = child.border_height()
@@ -536,10 +525,10 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
         line, = flex_lines
         min_cross_size = getattr(box, f'min_{cross}')
         if min_cross_size == 'auto':
-            min_cross_size = float('-inf')
+            min_cross_size = -inf
         max_cross_size = getattr(box, f'max_{cross}')
         if max_cross_size == 'auto':
-            max_cross_size = float('inf')
+            max_cross_size = inf
         line.cross_size = max(
             min_cross_size, min(line.cross_size, max_cross_size))
 
@@ -688,7 +677,7 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
         box.content_box_y() if cross == 'height'
         else box.content_box_x())
     for line in flex_lines:
-        line.lower_baseline = 0
+        line.lower_baseline = -inf
         # TODO: don't duplicate this loop
         for i, child in line:
             align_self = child.style['align_self']
@@ -698,6 +687,8 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
                 # TODO: handle vertical text
                 child.baseline = child._baseline - position_cross
                 line.lower_baseline = max(line.lower_baseline, child.baseline)
+        if line.lower_baseline == -inf:
+            line.lower_baseline = line[0][1]._baseline if line else 0
         for i, child in line:
             cross_margins = (
                 (child.margin_top, child.margin_bottom) if cross == 'height'
@@ -843,7 +834,7 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
                 new_child, child_resume_at = block.block_level_layout_switch(
                     context, child, bottom_space, child_skip_stack, box,
                     page_is_empty, absolute_boxes, fixed_boxes,
-                    adjoining_margins=[], discard=False)[:2]
+                    adjoining_margins=[], discard=False, max_lines=None)[:2]
                 if new_child is None:
                     if resume_at:
                         index, = resume_at
@@ -853,7 +844,6 @@ def flex_layout(context, box, bottom_space, skip_stack, containing_block,
                     box.children.append(new_child)
                     if child_resume_at is not None:
                         if original_skip_stack:
-                            # TODO: handle multiple skip stacks
                             first_level_skip, = original_skip_stack
                         else:
                             first_level_skip = 0
