@@ -1,12 +1,8 @@
-"""
-    weasyprint.layout.pages
-    -----------------------
-
-    Layout for pages and CSS3 margin boxes.
-
-"""
+"""Layout for pages and CSS3 margin boxes."""
 
 import copy
+from itertools import chain
+from math import inf
 
 from ..css import PageType, computed_from_cascaded
 from ..formatting_structure import boxes, build
@@ -107,9 +103,9 @@ class HorizontalBox(OrientedBox):
 
 
 def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
-    """
-    Compute and set a margin box fixed dimension on ``box``, as described in:
-    http://dev.w3.org/csswg/css3-page/#margin-constraints
+    """Compute and set a margin box fixed dimension on ``box``.
+
+    Described in: https://drafts.csswg.org/css-page-3/#margin-constraints
 
     :param box:
         The margin box to work on
@@ -128,7 +124,7 @@ def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
 
     # Rule 2
     total = box.padding_plus_border + sum(
-        value for value in [box.margin_a, box.margin_b, box.inner]
+        value for value in (box.margin_a, box.margin_b, box.inner)
         if value != 'auto')
     if total > outer:
         if box.margin_a == 'auto':
@@ -139,7 +135,7 @@ def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
             # XXX this is not in the spec, but without it box.inner
             # would end up with a negative value.
             # Instead, this will trigger rule 3 below.
-            # http://lists.w3.org/Archives/Public/www-style/2012Jul/0006.html
+            # https://lists.w3.org/Archives/Public/www-style/2012Jul/0006.html
             box.inner = 0
     # Rule 3
     if 'auto' not in [box.margin_a, box.margin_b, box.inner]:
@@ -168,7 +164,7 @@ def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
         box.inner = (outer - box.padding_plus_border -
                      box.margin_a - box.margin_b)
     # Rule 6
-    if box.margin_a == 'auto' and box.margin_b == 'auto':
+    if box.margin_a == box.margin_b == 'auto':
         box.margin_a = box.margin_b = (
             outer - box.padding_plus_border - box.inner) / 2
 
@@ -180,7 +176,7 @@ def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
 def compute_variable_dimension(context, side_boxes, vertical, outer_sum):
     """
     Compute and set a margin box fixed dimension on ``box``, as described in:
-    http://dev.w3.org/csswg/css3-page/#margin-dimension
+    https://drafts.csswg.org/css-page-3/#margin-dimension
 
     :param side_boxes: Three boxes on a same side (as opposed to a corner.)
         A list of:
@@ -364,10 +360,10 @@ def make_margin_boxes(context, page, state):
     page_end_y = margin_top + max_box_height
 
     # Margin box dimensions, described in
-    # http://dev.w3.org/csswg/css3-page/#margin-box-dimensions
+    # https://drafts.csswg.org/css-page-3/#margin-box-dimensions
     generated_boxes = []
 
-    for prefix, vertical, containing_block, position_x, position_y in [
+    for prefix, vertical, containing_block, position_x, position_y in (
         ('top', False, (max_box_width, margin_top),
             margin_left, 0),
         ('bottom', False, (max_box_width, margin_bottom),
@@ -376,7 +372,7 @@ def make_margin_boxes(context, page, state):
             0, margin_top),
         ('right', True, (margin_right, max_box_height),
             page_end_x, margin_top),
-    ]:
+    ):
         if vertical:
             suffixes = ['top', 'middle', 'bottom']
             fixed_outer, variable_outer = containing_block
@@ -404,18 +400,18 @@ def make_margin_boxes(context, page, state):
                     variable_outer - box.margin_width())
             compute_fixed_dimension(
                 context, box, fixed_outer, not vertical,
-                prefix in ['top', 'left'])
+                prefix in ('top', 'left'))
             generated_boxes.append(box)
 
     # Corner boxes
 
-    for at_keyword, cb_width, cb_height, position_x, position_y in [
+    for at_keyword, cb_width, cb_height, position_x, position_y in (
         ('@top-left-corner', margin_left, margin_top, 0, 0),
         ('@top-right-corner', margin_right, margin_top, page_end_x, 0),
         ('@bottom-left-corner', margin_left, margin_bottom, 0, page_end_y),
         ('@bottom-right-corner', margin_right, margin_bottom,
             page_end_x, page_end_y),
-    ]:
+    ):
         box = make_box(at_keyword, (cb_width, cb_height))
         if not box.is_generated:
             continue
@@ -434,10 +430,10 @@ def make_margin_boxes(context, page, state):
 def margin_box_content_layout(context, page, box):
     """Layout a margin box’s content once the box has dimensions."""
     positioned_boxes = []
-    box, resume_at, next_page, _, _ = block_container_layout(
-        context, box, bottom_space=-float('inf'), skip_stack=None,
-        page_is_empty=True, absolute_boxes=positioned_boxes,
-        fixed_boxes=positioned_boxes, adjoining_margins=None, discard=False)
+    box, resume_at, next_page, _, _, _ = block_container_layout(
+        context, box, bottom_space=-inf, skip_stack=None, page_is_empty=True,
+        absolute_boxes=positioned_boxes, fixed_boxes=positioned_boxes,
+        adjoining_margins=None, discard=False, max_lines=None)
     assert resume_at is None
     for absolute_box in positioned_boxes:
         absolute_layout(
@@ -473,8 +469,8 @@ def page_width_or_height(box, containing_block_size):
      over-constrained, instead of ignoring any margins, the containing block
      is resized to coincide with the margin edges of the page box."
 
-    http://dev.w3.org/csswg/css3-page/#page-box-page-rule
-    http://www.w3.org/TR/CSS21/visudet.html#blockwidth
+    https://drafts.csswg.org/css-page-3/#page-box-page-rule
+    https://www.w3.org/TR/CSS21/visudet.html#blockwidth
 
     """
     remaining = containing_block_size - box.padding_plus_border
@@ -548,31 +544,31 @@ def make_page(context, root_box, page_type, resume_at, page_number,
         root_box = root_box.copy_with_children([])
 
     # TODO: handle cases where the root element is something else.
-    # See http://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo
+    # See https://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo
     assert isinstance(root_box, (boxes.BlockBox, boxes.FlexContainerBox))
     context.create_block_formatting_context()
     context.current_page = page_number
-    context.current_page_footnotes = context.reported_footnotes.copy()
+    context.current_page_footnotes = []
     context.current_footnote_area = footnote_area
 
-    if context.reported_footnotes:
-        footnote_area.children = tuple(context.reported_footnotes)
-        context.reported_footnotes = []
-        reported_footnote_area = build.create_anonymous_boxes(
-            footnote_area.deepcopy())
-        reported_footnote_area, _, _, _, _ = block_level_layout(
-            context, reported_footnote_area, -float('inf'), None,
-            footnote_area.page, True, [], [], [], False)
-        footnote_area.height = reported_footnote_area.height
-        context.page_bottom -= reported_footnote_area.margin_height()
+    reported_footnotes = context.reported_footnotes
+    context.reported_footnotes = []
+    for i, reported_footnote in enumerate(reported_footnotes):
+        context.footnotes.append(reported_footnote)
+        overflow = context.layout_footnote(reported_footnote)
+        if overflow and i != 0:
+            context.report_footnote(reported_footnote)
+            context.reported_footnotes = reported_footnotes[i:]
+            break
 
     page_is_empty = True
     adjoining_margins = []
     positioned_boxes = []  # Mixed absolute and fixed
     out_of_flow_boxes = []
-    broken_out_of_flow = []
-    for box, containing_block, skip_stack in context.broken_out_of_flow:
-        box.position_y = 0
+    broken_out_of_flow = {}
+    context_out_of_flow = context.broken_out_of_flow.values()
+    for box, containing_block, skip_stack in context_out_of_flow:
+        box.position_y = root_box.content_box_y()
         if box.is_floated():
             out_of_flow_box, out_of_flow_resume_at = float_layout(
                 context, box, containing_block, positioned_boxes,
@@ -584,16 +580,21 @@ def make_page(context, root_box, page_type, resume_at, page_number,
                 skip_stack)
         out_of_flow_boxes.append(out_of_flow_box)
         if out_of_flow_resume_at:
-            broken_out_of_flow.append(
-                (box, containing_block, out_of_flow_resume_at))
+            broken_out_of_flow[out_of_flow_box] = (
+                box, containing_block, out_of_flow_resume_at)
     context.broken_out_of_flow = broken_out_of_flow
-    root_box, resume_at, next_page, _, _ = block_level_layout(
+    root_box, resume_at, next_page, _, _, _ = block_level_layout(
         context, root_box, 0, resume_at, initial_containing_block,
-        page_is_empty, positioned_boxes, positioned_boxes, adjoining_margins,
-        discard=False)
+        page_is_empty, positioned_boxes, positioned_boxes, adjoining_margins)
     assert root_box
     root_box.children = out_of_flow_boxes + root_box.children
 
+    footnote_area = build.create_anonymous_boxes(footnote_area.deepcopy())
+    footnote_area = block_level_layout(
+        context, footnote_area, -inf, None, footnote_area.page, True,
+        positioned_boxes, positioned_boxes)[0]
+    footnote_area.translate(dy=-footnote_area.margin_height())
+
     page.fixed_boxes = [
         placeholder._box for placeholder in positioned_boxes
         if placeholder._box.style['position'] == 'fixed']
@@ -602,16 +603,12 @@ def make_page(context, root_box, page_type, resume_at, page_number,
             context, absolute_box, page, positioned_boxes, bottom_space=0,
             skip_stack=None)
 
-    footnote_area = build.create_anonymous_boxes(footnote_area.deepcopy())
-    footnote_area, _, _, _, _ = block_level_layout(
-        context, footnote_area, -float('inf'), None, footnote_area.page,
-        True, [], [], [], False)
-    footnote_area.translate(dy=-footnote_area.margin_height())
-
     context.finish_block_formatting_context(root_box)
 
     page.children = [root_box, footnote_area]
-    descendants = page.descendants()
+    descendants = chain(page.descendants(), *(
+        child.descendants() if hasattr(child, 'descendants') else (child,)
+        for child in positioned_boxes))
 
     # Update page counter values
     _standardize_page_based_counters(style, None)
@@ -648,8 +645,7 @@ def make_page(context, root_box, page_type, resume_at, page_number,
         # string-set and bookmark-labels don't create boxes, only `content`
         # requires another call to make_page. There is maximum one 'content'
         # item per box.
-        # TODO: remove attribute or set a default value in Box class
-        if hasattr(child, 'missing_link'):
+        if child.missing_link:
             # A CounterLookupItem exists for the css-token 'content'
             counter_lookup = target_collector.counter_lookup_items.get(
                 (child.missing_link, 'content'))
@@ -729,7 +725,7 @@ def set_page_type_computed_styles(page_type, html, style_for):
     style_for.set_computed_styles(
         page_type,
         # @page inherits from the root element:
-        # http://lists.w3.org/Archives/Public/www-style/2012Jan/1164.html
+        # https://lists.w3.org/Archives/Public/www-style/2012Jan/1164.html
         root=html.etree_element, parent=html.etree_element,
         base_url=html.base_url)
 
@@ -841,6 +837,7 @@ def make_all_pages(context, root_box, html, pages):
 
     """
     i = 0
+    reported_footnotes = None
     while True:
         remake_state = context.page_maker[i][-1]
         if (len(pages) == 0 or
@@ -853,15 +850,19 @@ def make_all_pages(context, root_box, html, pages):
             remake_state['anchors'] = []
             remake_state['content_lookups'] = []
             page, resume_at = remake_page(i, context, root_box, html)
+            reported_footnotes = context.reported_footnotes
             yield page
         else:
             PROGRESS_LOGGER.info(
                 'Step 5 - Creating layout - Page %d (up-to-date)', i + 1)
             resume_at = context.page_maker[i + 1][0]
+            reported_footnotes = None
             yield pages[i]
 
         i += 1
-        if resume_at is None and not context.reported_footnotes:
-            # Throw away obsolete pages
+        if resume_at is None and not reported_footnotes:
+            # Throw away obsolete pages and content
             context.page_maker = context.page_maker[:i + 1]
+            context.broken_out_of_flow.clear()
+            context.reported_footnotes.clear()
             return
