@@ -1,16 +1,15 @@
-"""
-    weasyprint.layout.preferred
-    ---------------------------
+"""Preferred and minimum preferred width.
 
-    Preferred and minimum preferred width, aka. max-content and min-content
-    width, aka. the shrink-to-fit algorithm.
+Also known as max-content and min-content width, also known as the
+shrink-to-fit algorithm.
 
-    Terms used (max-content width, min-content width) are defined in David
-    Baron's unofficial draft (http://dbaron.org/css/intrinsic/).
+Terms used (max-content width, min-content width) are defined in David
+Baron's unofficial draft (https://dbaron.org/css/intrinsic/).
 
 """
 
 import sys
+from math import inf
 
 from ..formatting_structure import boxes
 from ..text.line_break import split_first_line
@@ -23,7 +22,7 @@ def shrink_to_fit(context, box, available_content_width):
     *Warning:* both available_content_width and the return value are
     for width of the *content area*, not margin area.
 
-    http://www.w3.org/TR/CSS21/visudet.html#float-width
+    https://www.w3.org/TR/CSS21/visudet.html#float-width
 
     """
     return min(
@@ -93,7 +92,7 @@ def _block_content_width(context, box, function, outer):
     if width == 'auto' or width.unit == '%':
         # "percentages on the following properties are treated instead as
         # though they were the following: width: auto"
-        # http://dbaron.org/css/intrinsic/#outer-intrinsic
+        # https://dbaron.org/css/intrinsic/#outer-intrinsic
         children_widths = [
             function(context, child, outer=True) for child in box.children
             if not child.is_absolutely_positioned()]
@@ -114,7 +113,7 @@ def min_max(box, width):
     else:
         min_width = min_width.value
     if max_width == 'auto' or max_width.unit == '%':
-        max_width = float('inf')
+        max_width = inf
     else:
         max_width = max_width.value
 
@@ -154,7 +153,7 @@ def margin_width(box, width, left=True, right=True):
         width += box.style['border_right_width']
 
     if percentages < 100:
-        return width / (1 - percentages / 100.)
+        return width / (1 - percentages / 100)
     else:
         # Pathological case, ignore
         return 0
@@ -227,8 +226,7 @@ def column_group_content_width(context, box):
 def table_cell_min_content_width(context, box, outer):
     """Return the min-content width for a ``TableCellBox``."""
     children_widths = [
-        min_content_width(context, child, outer=True)
-        for child in box.children
+        min_content_width(context, child) for child in box.children
         if not child.is_absolutely_positioned()]
     children_min_width = margin_width(
         box, max(children_widths) if children_widths else 0)
@@ -265,7 +263,6 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
     if skip_stack is None:
         skip = 0
     else:
-        # TODO: handle multiple skip stacks
         (skip, skip_stack), = skip_stack.items()
     for child in box.children[skip:]:
         if child.is_absolutely_positioned():
@@ -307,7 +304,7 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
                         split_first_line(
                             child_text[resume_index:], child.style, context,
                             max_width, child.justification_spacing,
-                            minimum=True))
+                            is_line_start=is_line_start, minimum=True))
                     lines.append(width)
                     if first_line:
                         break
@@ -315,11 +312,11 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
                     current_line += lines[0]
                     break
         else:
-            # http://www.w3.org/TR/css3-text/#line-break-details
+            # https://www.w3.org/TR/css-text-3/#overflow-wrap
             # "The line breaking behavior of a replaced element
             #  or other atomic inline is equivalent to that
             #  of the Object Replacement Character (U+FFFC)."
-            # http://www.unicode.org/reports/tr14/#DescriptionOfProperties
+            # https://www.unicode.org/reports/tr14/#DescriptionOfProperties
             # "By default, there is a break opportunity
             #  both before and after any inline object."
             if minimum:
@@ -344,7 +341,7 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
 def _percentage_contribution(box):
     """Return the percentage contribution of a cell, column or column group.
 
-    http://dbaron.org/css/intrinsic/#pct-contrib
+    https://dbaron.org/css/intrinsic/#pct-contrib
 
     """
     min_width = (
@@ -352,7 +349,7 @@ def _percentage_contribution(box):
         box.style['min_width'].unit == '%' else 0)
     max_width = (
         box.style['max_width'].value if box.style['max_width'] != 'auto' and
-        box.style['max_width'].unit == '%' else float('inf'))
+        box.style['max_width'].unit == '%' else inf)
     width = (
         box.style['width'].value if box.style['width'] != 'auto' and
         box.style['width'].unit == '%' else 0)
@@ -368,7 +365,7 @@ def table_and_columns_preferred_widths(context, box, outer=True):
        column_intrinsic_percentages, constrainedness,
        total_horizontal_border_spacing, grid)``
 
-    http://dbaron.org/css/intrinsic/
+    https://dbaron.org/css/intrinsic/
 
     """
     from .table import distribute_excess_width
@@ -410,11 +407,9 @@ def table_and_columns_preferred_widths(context, box, outer=True):
         min_width = block_min_content_width(context, table, outer=False)
         max_width = block_max_content_width(context, table, outer=False)
         outer_min_width = adjust(
-            box, outer=True, width=block_min_content_width(
-                context, table, outer=True))
+            box, outer=True, width=block_min_content_width(context, table))
         outer_max_width = adjust(
-            box, outer=True, width=block_max_content_width(
-                context, table, outer=True))
+            box, outer=True, width=block_max_content_width(context, table))
         result = ([], [], [], [], total_horizontal_border_spacing, [])
         context.tables[table] = result = {
             False: (min_width, max_width) + result,
@@ -572,14 +567,14 @@ def table_and_columns_preferred_widths(context, box, outer=True):
 
     # Calculate the max- and min-content widths of table and columns
     small_percentage_contributions = [
-        max_content_widths[i] / (intrinsic_percentages[i] / 100.)
+        max_content_widths[i] / (intrinsic_percentages[i] / 100)
         for i in range(grid_width)
         if intrinsic_percentages[i]]
     large_percentage_contribution_numerator = sum(
         max_content_widths[i] for i in range(grid_width)
         if intrinsic_percentages[i] == 0)
     large_percentage_contribution_denominator = (
-        (100 - sum(intrinsic_percentages)) / 100.)
+        (100 - sum(intrinsic_percentages)) / 100)
     if large_percentage_contribution_denominator == 0:
         if large_percentage_contribution_numerator == 0:
             large_percentage_contribution = 0
@@ -588,7 +583,7 @@ def table_and_columns_preferred_widths(context, box, outer=True):
             # infinitely large number if the numerator is nonzero [and] the
             # denominator of that ratio is 0."
             #
-            # http://dbaron.org/css/intrinsic/#autotableintrinsic
+            # https://dbaron.org/css/intrinsic/#autotableintrinsic
             #
             # Please note that "an infinitely large number" is not "infinite",
             # and that's probably not a coincindence: putting 'inf' here breaks
@@ -609,7 +604,7 @@ def table_and_columns_preferred_widths(context, box, outer=True):
     if table.style['width'] != 'auto' and table.style['width'].unit == 'px':
         # "percentages on the following properties are treated instead as
         # though they were the following: width: auto"
-        # http://dbaron.org/css/intrinsic/#outer-intrinsic
+        # https://dbaron.org/css/intrinsic/#outer-intrinsic
         table_min_width = table_max_width = table.style['width'].value
     else:
         table_min_width = table_min_content_width
@@ -700,7 +695,7 @@ def flex_min_content_width(context, box, outer=True):
     # TODO: use real values, see
     # https://www.w3.org/TR/css-flexbox-1/#intrinsic-sizes
     min_contents = [
-        min_content_width(context, child, outer=True)
+        min_content_width(context, child)
         for child in box.children if child.is_flex_item]
     if not min_contents:
         return adjust(box, outer, 0)
@@ -716,7 +711,7 @@ def flex_max_content_width(context, box, outer=True):
     # TODO: use real values, see
     # https://www.w3.org/TR/css-flexbox-1/#intrinsic-sizes
     max_contents = [
-        max_content_width(context, child, outer=True)
+        max_content_width(context, child)
         for child in box.children if child.is_flex_item]
     if not max_contents:
         return adjust(box, outer, 0)
