@@ -1,10 +1,6 @@
-"""
-    weasyprint.layout.tables
-    ------------------------
+"""Layout for tables and internal table boxes."""
 
-    Layout for tables and internal table boxes.
-
-"""
+from math import inf
 
 from ..formatting_structure import boxes
 from ..logger import LOGGER
@@ -16,18 +12,21 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                  page_is_empty, absolute_boxes, fixed_boxes):
     """Layout for a table box."""
     from .block import (
-        block_container_layout, block_level_page_break,
-        find_earlier_page_break)
+        avoid_page_break, block_container_layout, block_level_page_break,
+        find_earlier_page_break, force_page_break)
 
-    table.remove_decoration(start=skip_stack is not None, end=False)
+    has_header = table.children and table.children[0].is_header
+    has_footer = table.children and table.children[-1].is_footer
+    collapse = table.style['border_collapse'] == 'collapse'
+    remove_start_decoration = skip_stack is not None and not has_header
+    table.remove_decoration(remove_start_decoration, end=False)
 
     column_widths = table.column_widths
 
-    if table.style['border_collapse'] == 'separate':
-        border_spacing_x, border_spacing_y = table.style['border_spacing']
+    if collapse:
+        border_spacing_x = border_spacing_y = 0
     else:
-        border_spacing_x = 0
-        border_spacing_y = 0
+        border_spacing_x, border_spacing_y = table.style['border_spacing']
 
     column_positions = table.column_positions = []
     rows_left_x = table.content_box_x() + border_spacing_x
@@ -48,7 +47,7 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
             column_positions.append(position_x)
         rows_width = rows_x - position_x
 
-    if table.style['border_collapse'] == 'collapse':
+    if collapse:
         table.skip_cell_border_top = False
         table.skip_cell_border_bottom = False
         split_cells = False
@@ -64,7 +63,7 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                 skipped_rows += len(group.children)
         else:
             skipped_rows = 0
-        if not split_cells:
+        if not split_cells and not has_header:
             _, horizontal_borders = table.collapsed_border_grid
             if horizontal_borders:
                 table.border_top_width = max(
@@ -97,7 +96,7 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
             if new_group_children:
                 page_break = block_level_page_break(
                     new_group_children[-1], row)
-                if page_break in ('page', 'recto', 'verso', 'left', 'right'):
+                if force_page_break(page_break, context):
                     next_page['break'] = page_break
                     resume_at = {index_row: None}
                     break
@@ -153,9 +152,20 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                         cell_skip_stack = {len(cell.children): None}
                 else:
                     cell_skip_stack = None
-                if cell_skip_stack:
-                    cell.remove_decoration(start=True, end=False)
-                    if table.style['border_collapse'] == 'collapse':
+
+                # Adapt cell and table collapsing borders when a row is split
+                if cell_skip_stack and collapse:
+                    if has_header:
+                        # We have a header, we have to adapt the position of
+                        # the split cell to match the header’s bottom border
+                        header_rows = table.children[0].children
+                        if header_rows and header_rows[-1].children:
+                            cell.position_y += max(
+                                header.border_bottom_width
+                                for header in header_rows[-1].children)
+                    else:
+                        # We don’t have a header, we have to skip the
+                        # decoration at the top of the table when it’s drawn
                         table.skip_cell_border_top = True
 
                 # First try to render content as if there was already something
@@ -163,18 +173,18 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                 # force to render something if the page is actually empty, or
                 # just draw an empty cell otherwise. See
                 # test_table_break_children_margin.
-                new_cell, cell_resume_at, _, _, _ = block_container_layout(
+                new_cell, cell_resume_at, _, _, _, _ = block_container_layout(
                     context, cell, bottom_space, cell_skip_stack,
                     page_is_empty=page_is_empty, absolute_boxes=absolute_boxes,
                     fixed_boxes=fixed_boxes, adjoining_margins=None,
-                    discard=False)
+                    discard=False, max_lines=None)
                 if new_cell is None:
                     cell = cell.copy_with_children([])
-                    cell, _, _, _, _ = block_container_layout(
+                    cell, _, _, _, _, _ = block_container_layout(
                         context, cell, bottom_space, cell_skip_stack,
                         page_is_empty=True, absolute_boxes=[],
                         fixed_boxes=[], adjoining_margins=None,
-                        discard=False)
+                        discard=False, max_lines=None)
                     cell_resume_at = {0: None}
                 else:
                     cell = new_cell
@@ -195,14 +205,14 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                 new_row_children.append(cell)
 
             if resume_at and not page_is_empty:
-                if row.style['break_inside'] in ('avoid', 'avoid-page'):
+                if avoid_page_break(row.style['break_inside'], context):
                     resume_at = {index_row: {}}
                     break
 
             row = row.copy_with_children(new_row_children)
 
             # Table height algorithm
-            # http://www.w3.org/TR/CSS21/tables.html#height-layout
+            # https://www.w3.org/TR/CSS21/tables.html#height-layout
 
             # cells with vertical-align: baseline
             baseline_cells = []
@@ -252,7 +262,7 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                     if cell.vertical_align == 'bottom':
                         add_top_padding(cell, extra)
                     elif cell.vertical_align == 'middle':
-                        extra /= 2.
+                        extra /= 2
                         add_top_padding(cell, extra)
                         cell.padding_bottom += extra
                     else:
@@ -276,17 +286,11 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
             # Break if one cell was broken
             break_cell = False
             if resume_at:
-                values, = list(resume_at.values())
-                if len(row.children) == len(values):
-                    for cell_resume_at in values.values():
-                        if cell_resume_at != {0: None}:
-                            break_cell = True
-                            break
-                    else:
-                        # No cell was displayed, give up row
-                        next_position_y = float('inf')
-                        page_is_empty = False
-                        resume_at = None
+                if all(child.empty for child in row.children):
+                    # No cell was displayed, give up row
+                    next_position_y = inf
+                    page_is_empty = False
+                    resume_at = None
                 else:
                     break_cell = True
 
@@ -297,9 +301,10 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                 if new_group_children:
                     previous_row = new_group_children[-1]
                     page_break = block_level_page_break(previous_row, row)
-                    if page_break == 'avoid':
+                    if avoid_page_break(page_break, context):
                         earlier_page_break = find_earlier_page_break(
-                            new_group_children, absolute_boxes, fixed_boxes)
+                            context, new_group_children, absolute_boxes,
+                            fixed_boxes)
                         if earlier_page_break:
                             new_group_children, resume_at = earlier_page_break
                             break
@@ -317,7 +322,7 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
             page_is_empty = False
             skip_stack = None
 
-            if break_cell and table.style['border_collapse'] == 'collapse':
+            if break_cell and collapse and not has_footer:
                 table.skip_cell_border_bottom = True
 
             if break_cell or resume_at:
@@ -326,7 +331,7 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
         # Do not keep the row group if we made a page break
         # before any of its rows or with 'avoid'
         if resume_at and not original_page_is_empty and (
-                group.style['break_inside'] in ('avoid', 'avoid-page') or
+                avoid_page_break(group.style['break_inside'], context) or
                 not new_group_children):
             return None, None, next_page
 
@@ -373,7 +378,7 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
             if new_table_children:
                 page_break = block_level_page_break(
                     new_table_children[-1], group)
-                if page_break in ('page', 'recto', 'verso', 'left', 'right'):
+                if force_page_break(page_break, context):
                     next_page['break'] = page_break
                     resume_at = {index_group: None}
                     break
@@ -386,9 +391,10 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                 if new_table_children:
                     previous_group = new_table_children[-1]
                     page_break = block_level_page_break(previous_group, group)
-                    if page_break == 'avoid':
+                    if avoid_page_break(page_break, context):
                         earlier_page_break = find_earlier_page_break(
-                            new_table_children, absolute_boxes, fixed_boxes)
+                            context, new_table_children, absolute_boxes,
+                            fixed_boxes)
                         if earlier_page_break is not None:
                             new_table_children, resume_at = earlier_page_break
                             break
@@ -428,9 +434,9 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
         if page_is_empty:
             header_footer_bottom_space = bottom_space
         else:
-            header_footer_bottom_space = -float('inf')
+            header_footer_bottom_space = -inf
 
-        if table.children and table.children[0].is_header:
+        if has_header:
             header = table.children[0]
             header, resume_at, next_page = group_layout(
                 header, position_y, header_footer_bottom_space,
@@ -442,7 +448,7 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
         else:
             header = None
 
-        if table.children and table.children[-1].is_footer:
+        if has_footer:
             footer = table.children[-1]
             footer, resume_at, next_page = group_layout(
                 footer, position_y, header_footer_bottom_space,
@@ -462,8 +468,8 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
         avoid_breaks = False
         for group in table.children[skip:]:
             if not group.is_header and not group.is_footer:
-                avoid_breaks = (
-                    group.style['break_inside'] in ('avoid', 'avoid-page'))
+                avoid_breaks = avoid_page_break(
+                    group.style['break_inside'], context)
                 break
 
         if header and footer:
@@ -544,9 +550,9 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
         ([header] if header is not None else []) +
         new_table_children +
         ([footer] if footer is not None else []))
-    table.remove_decoration(
-        start=skip_stack is not None, end=resume_at is not None)
-    if table.style['border_collapse'] == 'collapse':
+    remove_end_decoration = resume_at is not None and not has_footer
+    table.remove_decoration(remove_start_decoration, remove_end_decoration)
+    if collapse:
         table.skipped_rows = skipped_rows
 
     # If the height property has a bigger value, just add blank space
@@ -583,8 +589,8 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
         group.width = last.position_x + last.width - first.position_x
         group.height = columns_height
 
-    if resume_at and not page_is_empty and (
-            table.style['break_inside'] in ('avoid', 'avoid-page')):
+    avoid_break = avoid_page_break(table.style['break_inside'], context)
+    if resume_at and not page_is_empty and avoid_break:
         table = None
         resume_at = None
     adjoining_margins = []
@@ -606,7 +612,7 @@ def add_top_padding(box, extra_padding):
 def fixed_table_layout(box):
     """Run the fixed table layout and return a list of column widths.
 
-    http://www.w3.org/TR/CSS21/tables.html#fixed-table-layout
+    https://www.w3.org/TR/CSS21/tables.html#fixed-table-layout
 
     """
     table = box.get_wrapped_table()
@@ -657,9 +663,7 @@ def fixed_table_layout(box):
                 width_per_column = width / len(columns_without_width)
                 for j in columns_without_width:
                     column_widths[j] = width_per_column
-            del width
         i += cell.colspan
-    del i
 
     # Distribute the remaining space equally on columns that do not have
     # a width yet.
@@ -697,7 +701,7 @@ def fixed_table_layout(box):
 def auto_table_layout(context, box, containing_block):
     """Run the auto table layout and return a list of column widths.
 
-    http://www.w3.org/TR/CSS21/tables.html#auto-table-layout
+    https://www.w3.org/TR/CSS21/tables.html#auto-table-layout
 
     """
     table = box.get_wrapped_table()
@@ -770,10 +774,6 @@ def auto_table_layout(context, box, containing_block):
             else:
                 break
         if upper_guess == lower_guess:
-            # TODO: Uncomment the assert when bugs #770 and #628 are closed
-            # Equivalent to "assert assignable_width == sum(upper_guess)"
-            # assert abs(assignable_width - sum(upper_guess)) <= (
-            #     assignable_width * 1e-9)
             table.column_widths = upper_guess
         else:
             added_widths = [
@@ -817,7 +817,7 @@ def table_wrapper_width(context, wrapper, containing_block):
 def cell_baseline(cell):
     """Return the y position of a cell baseline from the top of its border box.
 
-    See http://www.w3.org/TR/CSS21/tables.html#height-layout
+    See https://www.w3.org/TR/CSS21/tables.html#height-layout
 
     """
     result = find_in_flow_baseline(
@@ -857,7 +857,7 @@ def distribute_excess_width(context, grid, excess_width, column_widths,
 
     Return excess width left when it's impossible without breaking rules.
 
-    See http://dbaron.org/css/intrinsic/#distributetocols
+    See https://dbaron.org/css/intrinsic/#distributetocols
 
     """
     # First group
