@@ -1,12 +1,7 @@
-"""
-    weasyprint.svg
-    --------------
-
-    Render SVG images.
-
-"""
+"""Render SVG images."""
 
 import re
+from contextlib import suppress
 from math import cos, hypot, pi, radians, sin, sqrt
 from xml.etree import ElementTree
 
@@ -156,9 +151,13 @@ class Node:
                     child.attrib[key] = child.get('color', 'black')
 
             # Handle 'inherit' values
-            for key, value in child.attrib.items():
+            for key, value in child.attrib.copy().items():
                 if value == 'inherit':
-                    child.attrib[key] = self.get(key)
+                    value = self.get(key)
+                    if value is None:
+                        del child.attrib[key]
+                    else:
+                        child.attrib[key] = value
 
             # Fix text in text tags
             if child.tag in ('text', 'textPath', 'a'):
@@ -289,6 +288,11 @@ class SVG:
         self.tree = Node(wrapper, style)
         self.url = url
 
+        # Replace 'currentColor' value
+        for key in COLOR_ATTRIBUTES:
+            if self.tree.get(key) == 'currentColor':
+                self.tree.attrib[key] = self.tree.get('color', 'black')
+
         self.filters = {}
         self.gradients = {}
         self.images = {}
@@ -384,7 +388,7 @@ class SVG:
             if is_valid_bounding_box(box):
                 coords = (box[0], box[1], box[0] + box[2], box[1] + box[3])
             else:
-                coords = (0, 0, self.concrete_width, self.concrete_height)
+                coords = (0, 0, self.inner_width, self.inner_height)
             self.stream = self.stream.add_group(coords)
 
         # Apply transform attribute
@@ -417,10 +421,8 @@ class SVG:
 
         # Draw node
         if visible and node.tag in TAGS:
-            try:
+            with suppress(PointError):
                 TAGS[node.tag](self, node, font_size)
-            except PointError:
-                pass
 
         # Draw node children
         if display and node.tag not in DEF_TYPES:
@@ -495,13 +497,13 @@ class SVG:
             marker_node = self.markers.get(marker)
 
             # Calculate position, scale and clipping
-            if 'viewBox' in node.attrib:
-                marker_width, marker_height = svg.point(
+            if 'viewBox' in marker_node.attrib:
+                marker_width, marker_height = self.point(
                     marker_node.get('markerWidth', 3),
                     marker_node.get('markerHeight', 3),
                     font_size)
                 scale_x, scale_y, translate_x, translate_y = preserve_ratio(
-                    svg, marker_node, font_size, marker_width, marker_height)
+                    self, marker_node, font_size, marker_width, marker_height)
 
                 clip_x, clip_y, viewbox_width, viewbox_height = (
                     marker_node.get_viewbox())
@@ -722,17 +724,9 @@ class SVG:
 class Pattern(SVG):
     """SVG node applied as a pattern."""
     def __init__(self, tree, svg):
+        super().__init__(tree._etree_node, svg.url)
         self.svg = svg
         self.tree = tree
-        self.url = svg.url
-
-        self.filters = {}
-        self.gradients = {}
-        self.images = {}
-        self.markers = {}
-        self.masks = {}
-        self.patterns = {}
-        self.paths = {}
 
     def draw_node(self, node, font_size, fill_stroke=True):
         # Store the original tree in self.tree when calling draw(), so that we
