@@ -1,16 +1,10 @@
-"""
-    weasyprint.text.line_break
-    --------------------------
-
-    Decide where to break text lines.
-
-"""
+"""Decide where to break text lines."""
 
 import re
+from math import inf
 
 import pyphen
 
-from ..logger import LOGGER
 from .constants import LST_TO_ISO, PANGO_STRETCH, PANGO_STYLE, PANGO_WRAP_MODE
 from .ffi import (
     ffi, gobject, pango, pangoft2, unicode_to_char_p, units_from_double,
@@ -38,7 +32,7 @@ def first_line_metrics(first_line, text, layout, resume_at, space_collapse,
                        style, hyphenated=False, hyphenation_character=None):
     length = first_line.length
     if hyphenated:
-        length -= len(hyphenation_character.encode('utf8'))
+        length -= len(hyphenation_character.encode())
     elif resume_at:
         # Set an infinite width as we don't want to break lines when drawing,
         # the lines have already been split and the size may differ. Rendering
@@ -46,30 +40,16 @@ def first_line_metrics(first_line, text, layout, resume_at, space_collapse,
         pango.pango_layout_set_width(layout.layout, -1)
 
         # Create layout with final text
-        first_line_text = text.encode('utf-8')[:length].decode('utf-8')
+        first_line_text = text.encode()[:length].decode()
 
         # Remove trailing spaces if spaces collapse
         if space_collapse:
             first_line_text = first_line_text.rstrip(' ')
 
-        # Remove soft hyphens
-        layout.set_text(first_line_text.replace('\u00ad', ''))
-
+        layout.set_text(first_line_text)
         first_line, _ = layout.get_first_line()
         length = first_line.length if first_line is not None else 0
 
-        if '\u00ad' in first_line_text:
-            soft_hyphens = 0
-            if first_line_text[0] == '\u00ad':
-                length += 2  # len('\u00ad'.encode('utf8'))
-            for i in range(len(layout.text)):
-                while i + soft_hyphens + 1 < len(first_line_text):
-                    if first_line_text[i + soft_hyphens + 1] == '\u00ad':
-                        soft_hyphens += 1
-                    else:
-                        break
-            length += soft_hyphens * 2  # len('\u00ad'.encode('utf8'))
-
     width, height = line_size(first_line, style)
     baseline = units_to_double(pango.pango_layout_get_baseline(layout.layout))
     layout.deactivate()
@@ -130,6 +110,11 @@ class Layout:
             self.font, style['font_weight'])
         pango.pango_font_description_set_absolute_size(
             self.font, units_from_double(font_size))
+        if style['font_variation_settings'] != 'normal':
+            string = ','.join(
+                f'{key}={value}' for key, value in
+                style['font_variation_settings']).encode()
+            pango.pango_font_description_set_variations(self.font, string)
         pango.pango_layout_set_font_description(self.layout, self.font)
 
         text_decoration = style['text_decoration_line']
@@ -160,52 +145,34 @@ class Layout:
             style['font_variant_east_asian'], style['font_feature_settings'])
         if features and context:
             features = ','.join(
-                f'{key} {value}' for key, value in features.items())
-
-            # TODO: attributes should be freed.
+                f'{key} {value}' for key, value in features.items()).encode()
             # In the meantime, keep a cache to avoid leaking too many of them.
-            attr = context.font_features.get(features)
-            if attr is None:
-                try:
-                    attr = pango.pango_attr_font_features_new(
-                        features.encode('ascii'))
-                except AttributeError:
-                    LOGGER.error(
-                        'OpenType features are not available '
-                        'with Pango < 1.38')
-                else:
-                    context.font_features[features] = attr
-            if attr is not None:
-                attr_list = pango.pango_attr_list_new()
-                pango.pango_attr_list_insert(attr_list, attr)
-                pango.pango_layout_set_attributes(self.layout, attr_list)
+            attr = context.font_features.setdefault(
+                features, pango.pango_attr_font_features_new(features))
+            attr_list = pango.pango_attr_list_new()
+            pango.pango_attr_list_insert(attr_list, attr)
+            pango.pango_layout_set_attributes(self.layout, attr_list)
 
     def get_first_line(self):
         first_line = pango.pango_layout_get_line_readonly(self.layout, 0)
         second_line = pango.pango_layout_get_line_readonly(self.layout, 1)
-        if second_line != ffi.NULL:
-            index = second_line.start_index
-        else:
-            index = None
+        index = None if second_line == ffi.NULL else second_line.start_index
         self.first_line_direction = first_line.resolved_dir
         return first_line, index
 
     def set_text(self, text, justify=False):
-        try:
+        index = text.find('\n')
+        if index != -1:
             # Keep only the first line plus one character, we don't need more
-            text = text[:text.index('\n') + 2]
-        except ValueError:
-            # End-of-line not found, keept the whole text
-            pass
+            text = text[:index+2]
+        self.text = text
         text, bytestring = unicode_to_char_p(text)
-        self.text = bytestring.decode('utf-8')
         pango.pango_layout_set_text(self.layout, text, -1)
 
         word_spacing = self.style['word_spacing']
         if justify:
             # Justification is needed when drawing text but is useless during
-            # layout. Ignore it before layout is reactivated before the drawing
-            # step.
+            # layout, when it can be ignored.
             word_spacing += self.justification_spacing
 
         letter_spacing = self.style['letter_spacing']
@@ -217,12 +184,12 @@ class Layout:
 
         if self.text and (word_spacing or letter_spacing or word_breaking):
             attr_list = pango.pango_layout_get_attributes(self.layout)
-            if not attr_list:
-                # TODO: list should be freed
-                attr_list = pango.pango_attr_list_new()
+            if attr_list == ffi.NULL:
+                attr_list = ffi.gc(
+                    pango.pango_attr_list_new(),
+                    pango.pango_attr_list_unref)
 
             def add_attr(start, end, spacing):
-                # TODO: attributes should be freed
                 attr = pango.pango_attr_letter_spacing_new(spacing)
                 attr.start_index, attr.end_index = start, end
                 pango.pango_attr_list_change(attr_list, attr)
@@ -240,7 +207,6 @@ class Layout:
                     position = bytestring.find(b' ', position + 1)
 
             if word_breaking:
-                # TODO: attributes should be freed
                 attr = pango.pango_attr_insert_hyphens_new(False)
                 attr.start_index, attr.end_index = 0, len(bytestring)
                 pango.pango_attr_list_change(attr_list, attr)
@@ -302,7 +268,7 @@ def create_layout(text, style, context, max_width, justification_spacing):
 
 
 def split_first_line(text, style, context, max_width, justification_spacing,
-                     minimum=False):
+                     is_line_start=True, minimum=False):
     """Fit as much as possible in the available width for one line of text.
 
     Return ``(layout, length, resume_index, width, height, baseline)``.
@@ -327,34 +293,29 @@ def split_first_line(text, style, context, max_width, justification_spacing,
         max_width = None
 
     # Step #1: Get a draft layout with the first line
-    layout = None
-    if (max_width is not None and max_width != float('inf') and
-            style['font_size']):
+    if max_width is not None and max_width != inf and style['font_size']:
+        short_text = text
         if max_width == 0:
             # Trying to find minimum size, let's naively split on spaces and
             # keep one word + one letter
             space_index = text.find(' ')
-            if space_index == -1:
-                expected_length = len(text)
-            else:
-                expected_length = space_index + 2  # index + space + one letter
+            if space_index != -1:
+                short_text = text[:space_index+2]  # index + space + one letter
         else:
-            expected_length = int(max_width / style['font_size'] * 2.5)
-        if expected_length < len(text):
-            # Try to use a small amount of text instead of the whole text
-            layout = create_layout(
-                text[:expected_length], style, context, max_width,
-                justification_spacing)
-            first_line, index = layout.get_first_line()
-            if index is None:
-                # The small amount of text fits in one line, give up and use
-                # the whole text
-                layout = None
-    if layout is None:
+            short_text = text[:int(max_width / style['font_size'] * 2.5)]
+        # Try to use a small amount of text instead of the whole text
+        layout = create_layout(
+            short_text, style, context, max_width, justification_spacing)
+        first_line, resume_index = layout.get_first_line()
+        if resume_index is None and short_text != text:
+            # The small amount of text fits in one line, give up and use
+            # the whole text
+            layout.set_text(text)
+            first_line, resume_index = layout.get_first_line()
+    else:
         layout = create_layout(
             text, style, context, original_max_width, justification_spacing)
-        first_line, index = layout.get_first_line()
-    resume_index = index
+        first_line, resume_index = layout.get_first_line()
 
     # Step #2: Don't split lines when it's not needed
     if max_width is None:
@@ -362,7 +323,7 @@ def split_first_line(text, style, context, max_width, justification_spacing,
         return first_line_metrics(
             first_line, text, layout, resume_index, space_collapse, style)
     first_line_width, _ = line_size(first_line, style)
-    if index is None and first_line_width <= max_width:
+    if resume_index is None and first_line_width <= max_width:
         # The first line fits in the available width
         return first_line_metrics(
             first_line, text, layout, resume_index, space_collapse, style)
@@ -370,16 +331,14 @@ def split_first_line(text, style, context, max_width, justification_spacing,
     # Step #3: Try to put the first word of the second line on the first line
     # https://mail.gnome.org/archives/gtk-i18n-list/2013-September/msg00006
     # is a good thread related to this problem.
-    first_line_text = text.encode('utf-8')[:index].decode('utf-8')
-    # We can’t rely on first_line_width, see
-    # https://github.com/Kozea/WeasyPrint/issues/1051
+    first_line_text = text.encode()[:resume_index].decode()
     first_line_fits = (
         first_line_width <= max_width or
         ' ' in first_line_text.strip() or
         can_break_text(first_line_text.strip(), style['lang']))
     if first_line_fits:
         # The first line fits but may have been cut too early by Pango
-        second_line_text = text.encode('utf-8')[index:].decode('utf-8')
+        second_line_text = text.encode()[resume_index:].decode()
     else:
         # The line can't be split earlier, try to hyphenate the first word.
         first_line_text = ''
@@ -392,22 +351,19 @@ def split_first_line(text, style, context, max_width, justification_spacing,
             # only try when space collapsing is allowed
             new_first_line_text = first_line_text + next_word
             layout.set_text(new_first_line_text)
-            first_line, index = layout.get_first_line()
-            first_line_width, _ = line_size(first_line, style)
-            if index is None and first_line_text:
-                # The next word fits in the first line, keep the layout
-                resume_index = len(new_first_line_text.encode('utf-8')) + 1
-                return first_line_metrics(
-                    first_line, text, layout, resume_index, space_collapse,
-                    style)
-            elif index:
-                # Text may have been split elsewhere by Pango earlier
-                resume_index = index
-            else:
-                # Second line is none
-                resume_index = first_line.length + 1
-                if resume_index >= len(text.encode('utf-8')):
-                    resume_index = None
+            first_line, resume_index = layout.get_first_line()
+            if resume_index is None:
+                if first_line_text:
+                    # The next word fits in the first line, keep the layout
+                    resume_index = len(new_first_line_text.encode()) + 1
+                    return first_line_metrics(
+                        first_line, text, layout, resume_index, space_collapse,
+                        style)
+                else:
+                    # Second line is None
+                    resume_index = first_line.length + 1
+                    if resume_index >= len(text.encode()):
+                        resume_index = None
     elif first_line_text:
         # We found something on the first line but we did not find a word on
         # the next line, no need to hyphenate, we can keep the current layout
@@ -419,10 +375,12 @@ def split_first_line(text, style, context, max_width, justification_spacing,
     lang = style['lang'] and pyphen.language_fallback(style['lang'])
     total, left, right = style['hyphenate_limit_chars']
     hyphenated = False
-    soft_hyphen = '\u00ad'
+    soft_hyphen = '\xad'
 
-    try_hyphenate = False
+    auto_hyphenation = manual_hyphenation = False
     if hyphens != 'none':
+        manual_hyphenation = soft_hyphen in first_line_text + next_word
+    if hyphens == 'auto' and lang:
         next_word_boundaries = get_next_word_boundaries(second_line_text, lang)
         if next_word_boundaries:
             # We have a word to hyphenate
@@ -434,99 +392,87 @@ def split_first_line(text, style, context, max_width, justification_spacing,
                 space = max_width - first_line_width
                 if style['hyphenate_limit_zone'].unit == '%':
                     limit_zone = (
-                        max_width * style['hyphenate_limit_zone'].value / 100.)
+                        max_width * style['hyphenate_limit_zone'].value / 100)
                 else:
                     limit_zone = style['hyphenate_limit_zone'].value
                 if space > limit_zone or space < 0:
                     # Available space is worth the try, or the line is even too
                     # long to fit: try to hyphenate
-                    try_hyphenate = True
+                    auto_hyphenation = True
 
-    if try_hyphenate:
-        # Automatic hyphenation possible and next word is long enough
-        auto_hyphenation = hyphens == 'auto' and lang
-        manual_hyphenation = False
-        if auto_hyphenation:
-            if soft_hyphen in first_line_text or soft_hyphen in next_word:
-                # Automatic hyphenation opportunities within a word must be
-                # ignored if the word contains a conditional hyphen, in favor
-                # of the conditional hyphen(s).
-                # See https://drafts.csswg.org/css-text-3/#valdef-hyphens-auto
-                manual_hyphenation = True
-        else:
-            manual_hyphenation = hyphens == 'manual'
+    # Automatic hyphenation opportunities within a word must be ignored if the
+    # word contains a conditional hyphen, in favor of the conditional
+    # hyphen(s).
+    # See https://drafts.csswg.org/css-text-3/#valdef-hyphens-auto
+    if manual_hyphenation:
+        # Manual hyphenation: check that the line ends with a soft
+        # hyphen and add the missing hyphen
+        if first_line_text.endswith(soft_hyphen):
+            # The first line has been split on a soft hyphen
+            if ' ' in first_line_text:
+                first_line_text, next_word = first_line_text.rsplit(' ', 1)
+                next_word = f' {next_word}'
+                layout.set_text(first_line_text)
+                first_line, _ = layout.get_first_line()
+                resume_index = len((f'{first_line_text} ').encode())
+            else:
+                first_line_text, next_word = '', first_line_text
+        soft_hyphen_indexes = [
+            match.start() for match in re.finditer(soft_hyphen, next_word)]
+        soft_hyphen_indexes.reverse()
+        dictionary_iterations = [
+            next_word[:i + 1] for i in soft_hyphen_indexes]
+        start_word = 0
+    elif auto_hyphenation:
+        dictionary_key = (lang, left, right, total)
+        dictionary = context.dictionaries.get(dictionary_key)
+        if dictionary is None:
+            dictionary = pyphen.Pyphen(lang=lang, left=left, right=right)
+            context.dictionaries[dictionary_key] = dictionary
+        dictionary_iterations = [
+            start for start, end in dictionary.iterate(next_word)]
+    else:
+        dictionary_iterations = []
 
-        if manual_hyphenation:
-            # Manual hyphenation: check that the line ends with a soft
-            # hyphen and add the missing hyphen
-            if first_line_text.endswith(soft_hyphen):
-                # The first line has been split on a soft hyphen
-                if ' ' in first_line_text:
-                    first_line_text, next_word = (
-                        first_line_text.rsplit(' ', 1))
-                    next_word = f' {next_word}'
-                    layout.set_text(first_line_text)
-                    first_line, index = layout.get_first_line()
-                    resume_index = len((first_line_text + ' ').encode('utf8'))
-                else:
-                    first_line_text, next_word = '', first_line_text
-            soft_hyphen_indexes = [
-                match.start() for match in re.finditer(soft_hyphen, next_word)]
-            soft_hyphen_indexes.reverse()
-            dictionary_iterations = [
-                next_word[:i + 1] for i in soft_hyphen_indexes]
-        elif auto_hyphenation:
-            dictionary_key = (lang, left, right, total)
-            dictionary = context.dictionaries.get(dictionary_key)
-            if dictionary is None:
-                dictionary = pyphen.Pyphen(lang=lang, left=left, right=right)
-                context.dictionaries[dictionary_key] = dictionary
-            dictionary_iterations = [
-                start for start, end in dictionary.iterate(next_word)]
-        else:
-            dictionary_iterations = []
+    if dictionary_iterations:
+        for first_word_part in dictionary_iterations:
+            new_first_line_text = (
+                first_line_text +
+                second_line_text[:start_word] +
+                first_word_part)
+            hyphenated_first_line_text = (
+                new_first_line_text + style['hyphenate_character'])
+            new_layout = create_layout(
+                hyphenated_first_line_text, style, context, max_width,
+                justification_spacing)
+            new_first_line, index = new_layout.get_first_line()
+            new_first_line_width, _ = line_size(new_first_line, style)
+            new_space = max_width - new_first_line_width
+            hyphenated = (
+                index is None and (
+                    new_space >= 0 or
+                    first_word_part == dictionary_iterations[-1]))
+            if hyphenated:
+                layout = new_layout
+                first_line = new_first_line
+                resume_index = len(new_first_line_text.encode())
+                if text[len(new_first_line_text)] == soft_hyphen:
+                    # Recreate the layout with no max_width to be sure that
+                    # we don't break before the soft hyphen
+                    pango.pango_layout_set_width(layout.layout, -1)
+                    resume_index += len(soft_hyphen.encode())
+                break
 
-        if dictionary_iterations:
-            for first_word_part in dictionary_iterations:
-                new_first_line_text = (
-                    first_line_text +
-                    second_line_text[:start_word] +
-                    first_word_part)
-                hyphenated_first_line_text = (
-                    new_first_line_text + style['hyphenate_character'])
-                new_layout = create_layout(
-                    hyphenated_first_line_text, style, context, max_width,
-                    justification_spacing)
-                new_first_line, new_index = new_layout.get_first_line()
-                new_first_line_width, _ = line_size(new_first_line, style)
-                new_space = max_width - new_first_line_width
-                if new_index is None and (
-                        new_space >= 0 or
-                        first_word_part == dictionary_iterations[-1]):
-                    hyphenated = True
-                    layout = new_layout
-                    first_line = new_first_line
-                    index = new_index
-                    resume_index = len(new_first_line_text.encode('utf8'))
-                    if text[len(new_first_line_text)] == soft_hyphen:
-                        # Recreate the layout with no max_width to be sure that
-                        # we don't break before the soft hyphen
-                        pango.pango_layout_set_width(
-                            layout.layout, units_from_double(-1))
-                        resume_index += len(soft_hyphen.encode('utf8'))
-                    break
-
-            if not hyphenated and not first_line_text:
-                # Recreate the layout with no max_width to be sure that
-                # we don't break before or inside the hyphenate character
-                hyphenated = True
-                layout.set_text(hyphenated_first_line_text)
-                pango.pango_layout_set_width(
-                    layout.layout, units_from_double(-1))
-                first_line, index = layout.get_first_line()
-                resume_index = len(new_first_line_text.encode('utf8'))
-                if text[len(first_line_text)] == soft_hyphen:
-                    resume_index += len(soft_hyphen.encode('utf8'))
+        if not hyphenated and not first_line_text:
+            # Recreate the layout with no max_width to be sure that
+            # we don't break before or inside the hyphenate character
+            hyphenated = True
+            layout.set_text(hyphenated_first_line_text)
+            pango.pango_layout_set_width(layout.layout, -1)
+            first_line, _ = layout.get_first_line()
+            resume_index = len(new_first_line_text.encode())
+            if text[len(first_line_text)] == soft_hyphen:
+                resume_index += len(soft_hyphen.encode())
 
     if not hyphenated and first_line_text.endswith(soft_hyphen):
         # Recreate the layout with no max_width to be sure that
@@ -535,18 +481,21 @@ def split_first_line(text, style, context, max_width, justification_spacing,
         hyphenated_first_line_text = (
             first_line_text + style['hyphenate_character'])
         layout.set_text(hyphenated_first_line_text)
-        pango.pango_layout_set_width(
-            layout.layout, units_from_double(-1))
-        first_line, index = layout.get_first_line()
-        resume_index = len(first_line_text.encode('utf8'))
+        pango.pango_layout_set_width(layout.layout, -1)
+        first_line, _ = layout.get_first_line()
+        resume_index = len(first_line_text.encode())
 
     # Step 5: Try to break word if it's too long for the line
     overflow_wrap = style['overflow_wrap']
     first_line_width, _ = line_size(first_line, style)
     space = max_width - first_line_width
     # If we can break words and the first line is too long
-    if space < 0 and (overflow_wrap == 'anywhere' or
-                      (overflow_wrap == 'break-word' and not minimum)):
+    can_break = (
+        style['word_break'] == 'break-all' or (
+            is_line_start and (
+                overflow_wrap == 'anywhere' or
+                (overflow_wrap == 'break-word' and not minimum))))
+    if space < 0 and can_break:
         # Is it really OK to remove hyphenation for word-break ?
         hyphenated = False
         # TODO: Modify code to preserve W3C condition:
@@ -562,7 +511,7 @@ def split_first_line(text, style, context, max_width, justification_spacing,
             layout.layout, PANGO_WRAP_MODE['WRAP_CHAR'])
         first_line, index = layout.get_first_line()
         resume_index = index or first_line.length
-        if resume_index >= len(text.encode('utf-8')):
+        if resume_index >= len(text.encode()):
             resume_index = None
 
     return first_line_metrics(
